package edu.mit.csail.aeolus.api;

import java.io.Serializable;

import edu.mit.csail.aeolus.runtime.UserProxyInterface;
import edu.mit.csail.aeolus.runtime.UserProxyMap;
import edu.mit.csail.aeolus.infoflow.InfoFlowState;
import edu.mit.csail.aeolus.security.*;

import edu.mit.csail.aeolus.auditing.AeolusApiConstants;
import edu.mit.csail.aeolus.auditing.EventID;
import edu.mit.csail.aeolus.exceptions.InfoFlowControlException;
import edu.mit.csail.aeolus.exceptions.AuthorityException;

/**
 * This class is used to create share-able objects in Aeolus.
 * These objects will propagate changes between threads in Aeolus.
 * 
 * Trying to serialize these objects will cause an exception, leave serialization
 * of these objects to the Aeolus runtime.
 * 
 * @author aaron
 *
 * @param <T> 
 */
public final class AeolusBox<T extends Serializable> extends AeolusShared {
	private static final long serialVersionUID = 8093917470092286062L;
	// private transient String content = null;
        // private transient SharedStateMap map;
        private T object;
   
    /**
      * Construct an AeolusBox with the specified labels and a copy of o for the contents.
      * @throws InfoFlowControlException - if thread's labels are more constraining than those provided.
      */
    public AeolusBox(AeolusLabel objSLabel, AeolusLabel objILabel, T o)
		throws InfoFlowControlException {
    	super(objSLabel, objILabel);
	this.object = AeolusCloner.Clone(o);
    }

    /**
      * Construct an AeolusBox with the current thread's labels and a copy of o for the contents.
      */
    public AeolusBox(T o) {
    	super();
	this.object = AeolusCloner.Clone(o);
    }

    /**
      * The put method puts a copy of the argument object into the AeolusBox
      * @throws InfoFlowControlException - if thread's labels don't allow the write
      */
    public void put(T o) throws InfoFlowControlException {
	UserProxyInterface po = UserProxyMap.getMyProxyObject();
	InfoFlowState istate = po.aeolusStartSharedWrite(this);
	this.object = AeolusCloner.Clone(o);
	po.aeolusEndSharedWrite(this, istate);
    }

    /**
      * The get method returns a copy of the object in the AeolusBox
      * @throws InfoFlowControlException - if thread's labels don't allow the read
      */
    public T get() throws InfoFlowControlException {
	UserProxyInterface po = UserProxyMap.getMyProxyObject();
	InfoFlowState istate = po.aeolusStartSharedRead(this);
	T o =  AeolusCloner.Clone(this.object);
	po.aeolusEndSharedRead(this, istate);
	return o;
    }
}
