package edu.mit.csail.aeolus.api;

import java.io.Serializable;
import java.util.ArrayList;

// import edu.mit.csail.aeolus.api.PID;
import edu.mit.csail.aeolus.infoflow.*;
import edu.mit.csail.aeolus.exceptions.*;
import edu.mit.csail.aeolus.security.*;
import edu.mit.csail.aeolus.runtime.*;
import edu.mit.csail.aeolus.auditing.*;

/**
 *  This class is used to create share-able executable objects in Aeolus.
 *  The closure includes the labels specified by the creator of the closure.
 */

public abstract class AeolusClosure implements AeolusSafe {
    private final PID PID;
    private final ArrayList<Integer> basis;

    /*
     * XXX UserLoggingProxy modifies this directly so we don't have to change
     * the interface and do extra work when logging is turned off.  It's
     * important that user code can't access this, though!
     *
     * XXX A second point is that we *can't* track the eid from call to call
     * right now because closure objects are immutable...
     * @deprecated Not for public use
     */
    @Deprecated public EventID closureLastEid;

    /**
     * Construct a closure that will run with the authority of pid
     * @throws AuthorityException - if caller's thread doesn't actFor pid
     */
    protected AeolusClosure(PID pid) throws AuthorityException {
	UserProxy po = (((UserDomain)Thread.currentThread()).getProxyObject());
	// ADD ACTFOR CHECK
	this.PID = pid;
	this.basis = new ArrayList<Integer>(po.getIFState().basis);

        if (po instanceof UserLoggingProxy) {
		UserLoggingProxy logUserD = (UserLoggingProxy)po;
		EventID pPredEID = logUserD.getLastEID();
		this.closureLastEid = logUserD.getEID();
		EventRecord createEvent = new EventRecord(this.closureLastEid, 
							  pPredEID, AeolusApiConstants.CREATE_CLOSURE);
		logUserD.submitLogRecord(createEvent);
	} else {
		this.closureLastEid = null;
	}
    }

    /**
     * Return the PID associated with the closure
     * @return the PID associated with the closure
     */
    public final PID getPID() {
	return (this.PID);
    }

    /*
     * @deprecated not for public use
     */
    @Deprecated public final ArrayList<Integer> getBasisCopy() {
	return (new ArrayList<Integer>(basis));
    }

    /**
     * This method is invoked by our instrumentation when the closure is
     * called.  It is responsible for doing the appropriate logging and
     * principal change.
     *
     * @return the information flow state prior to the call
     * @deprecated - not for pulic use
     */
    @Deprecated protected final InfoFlowState callStart() {
	UserProxyInterface po = ((UserDomain)Thread.currentThread()).getProxyObject();
	return (po.aeolusStartClosureCall(this));
    }

    /**
     * This method is invoked by our instrumentation when the closure call
     * returns.  It is responsible for doing the appropriate logging and
     * principal change.
     *
     * @param oldState   the information flow state returned by callStart()
     *                   before the invocation
     * @deprecated - not for pulic use
     */
    @Deprecated protected final void callEnd(InfoFlowState oldState) {
	UserProxyInterface po = ((UserDomain)Thread.currentThread()).getProxyObject();
	po.aeolusEndClosureCall(this, oldState);
    }
}
