package edu.mit.csail.aeolus.api;

/**
 * Stores information about a filestream
 * - file system name
 * - file path
 * - file name
 * - open mode (read, write, read/write)
 * - handle
 * - opened/closed state
 * 
 * viq
 *
 */

public class AeolusFilestreamInfo {

	/** File path */
	private String filePath;
	
	/** Filestream mode (read, write, read/write) */
	private AeolusFilestreamMode mode;
	
	/** Filestream handle */
	private int assignedHandle;
	
	/** Filestream opened/closed state */
	private boolean isOpened = false;
    
	/**
	 * Constructor 
	 * @param myFilePath - file path
	 * @param myMode - filestream mode 
	 * @param opened - opened/closed state
	 */
	public AeolusFilestreamInfo(String myFilePath, AeolusFilestreamMode myMode, boolean opened) 
	{
		this.filePath = myFilePath;
		this.mode = myMode;
		this.isOpened = opened;
	}

	/**
	 * Returns the file path
	 * @return file path
	 */
	public String getFilePath()
	{
		return this.filePath;
	}
	
	/**
	 * Returns the filestream mode
	 * @return filestream mode
	 */
	public AeolusFilestreamMode getMode()
	{
		return this.mode;
	}
	
	/**
	 * Returns the assigned handle number
	 * @return assigned handle 
	 */
	public int getAssignedHandle()
	{
		return this.assignedHandle;
	}
	
	/**
	 * Returns true if filestream is opened
	 * @return true if FS opened
	 */
	public boolean isOpened()
	{
		return this.isOpened;
	}
	
	/**
	 * Sets filestream state to open
	 */
	public void setOpened()
	{
		this.isOpened = true;
	}

	/**
	 * Sets filestream state to closed
	 */
	public void setClosed()
	{
		this.isOpened = false;
	}
	
	/**
	 * Sets the assigned handle number
	 * @param value - assigned handle value
	 */
	public void setAssignedHande(int value)
	{
		this.assignedHandle = value;
	}
	
}
