package edu.mit.csail.aeolus.api;

import java.util.List;
import java.util.ArrayList;

// import edu.mit.csail.aeolus.api3.AeolusTag;

import edu.mit.csail.aeolus.infoflow.Label;
import edu.mit.csail.aeolus.infoflow.TagID;
import edu.mit.csail.aeolus.exceptions.AeolusException;
import edu.mit.csail.aeolus.exceptions.AuthorityException;
import edu.mit.csail.aeolus.exceptions.InfoFlowControlException;

import java.io.Serializable;

/**
 * An AeolusLabel is a collection of AeolusTags.
 */
public final class AeolusLabel implements Serializable, Cloneable {
	private Label l;

	/**
	 * Creates a new empty label
	 */
	public AeolusLabel () {
		this.l = new Label();
	}

	/**
	 * Returns a copy of label l
	 * @deprecated Not for public use.
	 */
	@Deprecated public AeolusLabel (Label l) {
		this.l = new Label(l);
	}
	/**
	 * Returns a copy of label l
	 */
	public AeolusLabel (AeolusLabel l) {
		this.l = new Label(l.getLabel());
	}

	/**
	 * Returns the label inside this
	 * @deprecated Not for public use
	 */
	@Deprecated public Label getLabel() {
		return this.l;
	}
	/**
	 * Adds tag t to this
	 */
	public void addTag(AeolusTag t) {
		this.l.addTag(t.getTagID());
	}
	/**
	 * Removes tag t from this
	 */
	public void removeTag(AeolusTag t) {
		this.l.removeTag(t.getTagID());
	}

	/**
	 * Checks if the tags in this are a subset of the tags in l
	 * @return true if the tags in this are a subset of the tags in l
	 */
	public boolean isSubsetOf(AeolusLabel  l) {
		return this.l.isSubsetOf(l.l);
	}

	/**
	 * Checks if this and l have the same tags
	 * @param l
	 * @return true if this and l have the same tags, else false
	 */
	public boolean equals(AeolusLabel  l) {
		return this.l.equals(l.getLabel());
	}
	/**
	 * Checks whether this is empty
	 * @return true if this is contains no tags, else false
	 */
	public boolean isEmpty() {
		return this.l.isEmpty();
	}
	/**
         * Checks if this contains tag t
	 * @return true if this contains tag t or a supertag of t
	 */
	public boolean hasTag(AeolusTag t) {
		return this.l.hasTag(t.getTagID());
	}
	/**
	 * Form the union of this and l
	 * @return the union of this and l
	 */
	public AeolusLabel  union(AeolusLabel  l) {
		return new AeolusLabel (this.l.union(l.getLabel()));
	}
	/**
	 * Intersect this with l
	 * @return the intersection of this and l
	 */
	public AeolusLabel  intersection(AeolusLabel  l) {
		return new AeolusLabel (this.l.intersection(l.getLabel()));
	}
	///**
	// * Returns the difference of this and l
	// * @return
	// */
	//AeolusLabel  difference(AeolusLabel  l) {
	//	return new AeolusLabel (this.l.intersection(l.getLabel()));
	//}

	/**
	 * Returns a list of tags in the label
	 * @return a list of tags in the label
	 */
        public List<AeolusTag> members() {
                ArrayList<AeolusTag> tagVals = new ArrayList<AeolusTag>();
		TagID [] tagids = l.getTags();
                for(TagID tagid : tagids){
			tagVals.add(new AeolusTag(tagid));
                }
                return tagVals;
        }

	/**
	 * Returns a copy of this
	 * @return a copy of this
	 */
	public AeolusLabel  clone () {
		return new AeolusLabel (new Label(this.l));
	}

	/**
	 * Returns the number of tags in this label
	 * @return the number of tags in this label
	 */
        public int size()
        {
                return this.l.size();
        }

	/**
	 * Returns the string representation of this label
	 * @return the string representation of this label
	 */
	public String toString() {
		return l.toString();
	}

}
