package edu.mit.csail.aeolus.api;

import java.util.concurrent.Semaphore;

import edu.mit.csail.aeolus.infoflow.InfoFlowState;
import edu.mit.csail.aeolus.auditing.AeolusApiConstants;
import edu.mit.csail.aeolus.exceptions.InfoFlowControlException;
import edu.mit.csail.aeolus.exceptions.AuthorityException;
import edu.mit.csail.aeolus.runtime.UserProxyInterface;
import edu.mit.csail.aeolus.runtime.UserProxyMap;

/**
 * Implementation of a shared lock.
 * @author blanks
 *
 */
public final class AeolusLock  extends AeolusShared {
	private static final long serialVersionUID = -6434887916231838392L;
	private transient Semaphore mutex;
	// private transient long holderId;

    /**
      * Construct an AeolusLock with the specified labels.
      * @throws InfoFlowControlException if thread's labels are more constrained than given labels
      */

	public AeolusLock(AeolusLabel sLabel, AeolusLabel iLabel) 
			throws InfoFlowControlException {
		super(sLabel, iLabel);
		mutex = new Semaphore(1);
		// holderId = -1;
	}

    /**
      * Construct an AeolusLock with the current thread's labels.
      */

	public AeolusLock() {
		super();
		mutex = new Semaphore(1);
		// holderId = -1;
	}

	/**
	 * Acquires the lock if it is not currently held and returns true, 
	 * otherwise returns false.
	 * @return true if lock acquired, else false
	 * @throws InfoFlowControlException if the thread can't read and write the lock
	 */
	public boolean tryLock() throws InfoFlowControlException {
		UserProxyInterface po = UserProxyMap.getMyProxyObject();
		// super.canReadWriteFromContext(po);
		InfoFlowState istate = po.aeolusStartSharedRead(this);
		if (mutex.tryAcquire()) {
			// holderId = po.getUserAppDomainId();
			// this.logSharedStateEvent(po, AeolusApiConstants.TRY_LOCK, true);
			po.aeolusEndSharedRead(this, istate);
			return true;
		}
		// this.logSharedStateEvent(po, AeolusApiConstants.TRY_LOCK, false);
		po.aeolusEndSharedRead(this, istate);
		return false;
	}

	/**
	 * Blocks until the lock is acquired.
	 * @throws InfoFlowControlException if the thread can't write the lock
	 */
	public void lock() throws InfoFlowControlException {
		UserProxyInterface po = UserProxyMap.getMyProxyObject();
		// super.canReadWriteFromContext(po);
		InfoFlowState istate = po.aeolusStartSharedWrite(this);
		try {
			mutex.acquire();
			// holderId = po.getUserAppDomainId();;
			// this.logSharedStateEvent(po, AeolusApiConstants.LOCK, true);
			po.aeolusEndSharedWrite(this, istate);
		} catch (InterruptedException e) {
			e.printStackTrace();
			po.aeolusEndSharedWrite(this, istate);
		}
	}

	/**
	 * Releases the lock 
	 * @throws InfoFlowControlException if the thread can't write the lock
	 */
	public void unlock() throws InfoFlowControlException {
		UserProxyInterface po = UserProxyMap.getMyProxyObject();
		// super.canWriteFromContext(po);
		InfoFlowState istate = po.aeolusStartSharedWrite(this);
		// if (po.getUserAppDomainId() == holderId) {
			// holderId = -1;
			mutex.release();
			// this.logSharedStateEvent(po, AeolusApiConstants.UNLOCK);
			po.aeolusEndSharedWrite(this, istate);
		// }
		// TODO: throw exception?
	}
}
