package edu.mit.csail.aeolus.api;

/**
 * AeolusSafe is a marker interface for classes that are safe to share between
 * threads without copying.  In order to be AeolusSafe, a class must either
 * extend AeolusShared, or it must contain no mutable state. All subclasses
 * of the class must also be AeolusSafe.
 *
 * An AeolusSafe class that is not AeolusShared is must meet the following
 * requirements, which are verified at load time.
 * <UL>
 *   <LI>All of its fields must be declared <TT>final</TT>.
 *   <LI>The types of its fields must also implement <TT>AeolusSafe</TT>,
 *       or be primitive types (other than
 *       arrays), i.e., they all must be "Safe to Share."
 *   <LI>No inner classes are allowed.
 * </UL>
 */
public interface AeolusSafe {
}
