package edu.mit.csail.aeolus.apps.mint1;

// Make workers and fork them

// create customers and give them to the workers

/// Running
// ask each worker to fetch recent transactions for each of their customers
//	from each of their customers' banks

// collect the results

import edu.mit.csail.aeolus.api.*;
import java.util.concurrent.Callable;
import edu.mit.csail.aeolus.exceptions.AuthorityException;
import edu.mit.csail.aeolus.exceptions.AeolusException;
import java.util.*;

public class MintServer1 {

	private AeolusLabel emptyLabel = new AeolusLabel();


	private Vector<Bank> Banks = new Vector<Bank>();
	private void create_banks(int n, int ntrans) {
	        for (int i=0; i< n; i++) {
		    try {
			AeolusTag btag = new AeolusTag();
			PID bpid = new PID();
			Bank b = new Bank(i, ntrans, btag, bpid);
			Banks.add(b);
		    } catch (Exception e) {
				System.out.println("MS: Exception in create_banks " + e);
		    }
		}
	}

	private Vector<Customer> Customers = new Vector<Customer>();
	private Vector<AeolusBox<Customer>> SOCustomers = new Vector<AeolusBox<Customer>>();

	private void create_customers(int n, Vector<Bank> Banks, AeolusTag all_cust_tag, int work_time) {
		boolean debug = true;
		Customer c = null;
	        for (int i=0; i< n; i++) {
			try {
	                	// c = new Customer(i, Banks, null, null);
	                	c = new Customer(i, Banks, all_cust_tag, work_time);
			} catch (Exception e) {
				System.out.println("MS: Exception 1 in create_customers " + e);
			}
	                if (debug) System.out.println("Creating customer named " + c.getName() + " " + c.getPID());
	                Customers.addElement(c);
			// AeolusTag Stag = new AeolusTag(all_cust_tag);
			// Slabel.addTag(all_cust_tag);
			// AeolusTag Itag = new AeolusTag();
			// AeolusLabel Ilabel = new AeolusLabel();
			try {
				AeolusBox<Customer> asoc = new AeolusBox<Customer>(emptyLabel, emptyLabel, c);
				// asoc.put(c);
				SOCustomers.addElement(asoc);
			} catch (Exception e) {
				System.out.println("MS: Exception 2 in create_customers " + e);
			}
			if (debug) System.out.println("SOCustomers " + SOCustomers.size());
	        }
	}


	private Vector<MintWorker> MintWorkers = new Vector<MintWorker>();
	// private Vector<AeolusQueue<Command>> MintWorkerInputQueues = new Vector<AeolusQueue<Command>>();
	private int nrequests = 1;
	private int nresults = 1;
	private int nskips = 1;
	private int nworkers = 1;
	private int ncustomers = 1;
	private int bankwait = 1;		// msec
	private int ntrans = 5;
	private AeolusQueue<Command> output = null;
	private AeolusQueue<Command> input = null;
	private PID mint_pid = null;
	private boolean use_closure = false;

	private MintServer1(int numWorkers, boolean use_closures,
				int numcustomers, int nbanks, int numrequests,
				int numskips, int numresults, int BankWait, int work_time)
				throws AuthorityException, AeolusException {
		nworkers = numWorkers;
		nresults = numresults;
		nskips = numskips;
		nrequests = numrequests;
		ncustomers = numcustomers;
		bankwait = BankWait;
		use_closure = use_closures;
		boolean debug = true;
		// u = new AeolusUser();
		mint_pid = AeolusLib.getPID();
		System.out.println("MINT_PID " + mint_pid);
		AeolusTag all_cust_tag = new AeolusTag();		// super tag of all customer tags
		System.out.println("ALL_CUST_TID " + all_cust_tag);

		System.out.println("MS: NWORKERS = " + nworkers);
		System.out.println("MS: NCUSTOMERS = " + ncustomers);
		System.out.println("MS: NBANKS = " + nbanks);
		if (debug) System.out.println("MS principal " + mint_pid);

		// create banks
		// System.out.println("Starting to create banks");
		create_banks(nbanks, ntrans);
		// System.out.println("Finished creating banks");

		// create customers
		System.out.println("Starting to create customers");
		create_customers(ncustomers, Banks, all_cust_tag, work_time);
		System.out.println("Finished creating customers");

		// create workers
		// 	now all workers share the same input queue,
		//	as well as the same output queue
		//	maybe consider control queue vs work queue?
		// System.out.println("Starting to create workers");
		input = new AeolusQueue<Command>(emptyLabel, emptyLabel);
		output =  new AeolusQueue<Command>(emptyLabel, emptyLabel);
		for (int i = nworkers-1; i > -1; i--) {
			if (debug) System.out.println("MS Before name " + i);
			String name = "MW-" + i;
			if (debug) System.out.println("MS Before principal " + i);
			PID mw_pid = new PID();
			if (debug) System.out.println("MS After principal " + i + " " + mw_pid);
			if (debug) System.out.println("MS Before command creation " + i);
			if (debug) System.out.println("MS Before mw " + i);
			MintWorker mw = new MintWorker(name, mw_pid, input, output, use_closure);
			if (debug) System.out.println("MS Before mw " + i);
			mw.setBanks(Banks, bankwait);
			for (Bank b: Banks) {
				PID bpid = b.getPID();
				AeolusTag btag = b.getTag();
				bpid.actFor(mw_pid);
				btag.delegate(mint_pid, mw_pid);
			}
			all_cust_tag.delegate(mint_pid, mw_pid);
			MintWorkers.addElement(mw);
			// MintWorkerInputQueues.addElement(input);
			// if (debug) System.out.println("MS Before invoking " + i + " " + mw_pid);
			// u.fork(mw, mw_pid);
			// try { Thread.sleep(10); // Let all threads get initialized 
			// } catch (Exception e)
			// { System.out.println("MS: Exception creating workers create_customers " + e); }
		}
		try {
		for (MintWorker mw : MintWorkers) {
			PID mw_pid = mw.getPID();
			if (debug) System.out.println("MS Before forking " + mw_pid);
			AeolusLib.fork(mw, mw_pid);
		}
		} catch (Exception e)
		{ System.out.println("MS: Exception creating workers " + e); }


		// Collect READY messages
		int nw = 0;
		System.out.println("MS: Starting to collect READY messages");
		while (true) {
		try {
			// Collect the command completions from the workers
			Command r = output.dequeue();
			if (r == null) {
				System.out.println("MS: Command is null ");
				continue;
			}
			System.out.println("MS: Command = " + r.get_command());
			// if (r.get_command().equals("UserInput")) {
			// 	System.out.println("MS: UserInput received ");
			// 	continue;
			// }
			nw++;
			if (nw == nworkers) break;
		} catch (Exception e) {
			System.out.println("Exception #1.1 in worker creation " + e);
			e.printStackTrace();
		}
		}
		// if (debug) System.out.println("Finished creating workers");
		System.out.println("Finished creating workers");

	}

	private void runtest(MintServer1 ms) {
		AeolusLabel mySecrecy = AeolusLib.getSecrecy();
		System.out.println("Enter runtest: My SLabel = " + mySecrecy.toString());
		boolean debug = true;
		// SET UP FOR A TEST

		// create lots of commands and put them on the input queue
		Command c0 = null;
		int ci = 0;
		int ncustomers = Customers.size();
		for (int i = 0; i < nrequests; i++) {
			Customer c = Customers.elementAt(ci);
			AeolusBox<Customer> asoc = SOCustomers.elementAt(ci);
			c0 = new Command("FetchRecentTransactions", asoc, c.getTag());
			System.out.println("MS: ASOC " + asoc + " " + c0.get_arg());
			mySecrecy = AeolusLib.getSecrecy();
			System.out.println("My SLabel = " + mySecrecy.toString());
			// add them to the workers' input queue
			try {
				input.enqueue(c0);
		 	} catch (Exception e) { System.out.println("Exception in runtest XX " + e); }
			System.out.println("MS: ASOC after enqueue");
			// if (i == 1000 || i >= 2000 || i == 3000 || i == 4000 || i == 5000) {
			// 	System.out.println("MintServer1: " + i + " commands created.");
			// }
			ci++;
			if (ci == ncustomers) ci = 0;
     		}
		System.out.println("MintServer1: "  + nrequests
			+ " commands added to the workers' input queue");
		
		// clear off some of the queue
		if (debug)  System.out.println("MintServer1: starting " + nskips + " skips.");
		for (int i = 0; i < nskips ; i++) {
			try {
				// Command r = output.waitAndDequeue();
				Command r = output.dequeue();
				if (r == null) {
					Thread.yield();
				}
			} catch (Exception e) { System.out.println("Exception in runtest " + e); }
		}
		if (debug)  System.out.println("MintServer1: finished " + nskips + " skips.");
		
		try {
		} catch (Exception e) { System.out.println("Exception in runtest " + e); }

		// try {
		// 	output.clearQueue(); System.out.println("MintServer1: queue cleared off");
		// } catch (Exception e) { System.out.println("Exception in runtest " + e); }
		System.gc();

		// THE TEST

		// start timer, count results and stop timer when nresults have been received
		long start_time, end_time;
		int n = 0;
		// start timer
		start_time = System.currentTimeMillis();
		while (true) {
		try {
			// Collect the command completions from the workers
			Command r = output.dequeue();
			if (r == null) continue;
			n++;
			if (n == nresults) break;
		} catch (Exception e) {
			System.out.println("Exception #2.1 in runtest " + e);
			e.printStackTrace();
		}
		}
		// stop timer
		end_time = System.currentTimeMillis();
		long delta_time_msec = end_time - start_time;
		double delta_time_sec = (double) (delta_time_msec / 1000.0);
		double cmds_per_sec = (double) nresults / delta_time_sec;
		System.out.println("end_time = " + end_time + " start_time " + start_time);
		System.out.println("delta_time_msec = " + delta_time_msec
				+ " delta_time_sec " + delta_time_sec);
		System.out.println("Throughput = " + nresults +
				" commands processed in " + delta_time_sec + " sec for " +
				cmds_per_sec + " commands processed per sec with " +
				nworkers + " workers"
				);
	

		// CLEAN UP AFTER A TEST
		// for (int i = 0; i < MintWorkers.size(); i++) {
		// 	int mwqueue = MintWorkerInputQueues.elementAt(i);
		// 	Command c = new Command("QUIT", null, null);
		// 	try {
		// 		u.enqueue(mwqueue, c);
		// 	} catch (Exception e) {
		// 		System.out.println("Exception in runtest " + e);
		// 	}
		// }
	}

	private static void usage() {
		System.out.println(
			   " -w n for number of workers \n" +
			   " -b n for number of banks \n" +
			   " -bw n for bank wait (msec) \n" +
			   " -wt n for workt time (msec) \n" +
			   " -c n for number of customers \n" +
			   " -nq n for number of requests \n" +
			   " -sk n for number of skipped results \n" +
			   " -nr n for number of results \n" +
			   " -m call for calls \n" +
			   " -m closure for closures \n" +
			   " -h for help\n" +
			   "Example: \n" +
			   "    java monster \"-r b -w 1 -c call\"\n" +
			   "NOTE: quotes around arglist");
		System.exit(0);
			   // " -n n for number msec to fetch transactions per bank per customer \n" +
	}
	public static void main(String[] input_args) {
		// System.out.println("Entering main " + input_args.length);
		// System.out.println(input_args);
		boolean set_nrequests = false;
		int nrequests = 10;
		boolean set_nskips = false;
		int nskips = 1;
		boolean set_nresults = false;
		int nresults = 5;
		boolean set_nworkers = false;
		int nworkers = 10;
		boolean set_ncustomers = false;
		int nbanks = 10;
		boolean set_work_time = false;
		int work_time = 1;
		boolean set_bankwait = false;
		int bankwait = 1;
		boolean set_nbanks = false;
		int ncustomers = 10;
		boolean set_call_v_closure  = false;
		boolean use_closure = false;
		// boolean set_proctime = false;
		// int nmsecproc = 1;
		String arglist = input_args[0];
		System.out.println("ARG List: " + arglist);
		String[] args = arglist.split(" ");
		System.out.println("ARG count: " + args.length);
		for (int i = 0; i < args.length; i++) {
		 	String s = args[i];
		 	System.out.println("ARG: " +s);
		}
		for (int i = 0; i < args.length; i++) {
			String s = args[i];
			if (s.equals("-h")) usage();
			if (s.equals("-w")) {
				set_nworkers = true;
				continue;
			}
			if (set_nworkers == true) {
				nworkers = Integer.parseInt(s);
				set_nworkers = false;
			}
			// if (s.equals("-n")) {
			// 	set_proctime = true;
			// 	continue;
			// }
			// if (set_proctime == true) {
			// 	nmsecproc = Integer.parseInt(s);
			// 	set_proctime = false;
			// }
			if (s.equals("-b")) {
				set_nbanks = true;
				continue;
			}
			if (set_nbanks == true) {
				nbanks = Integer.parseInt(s);
				set_nbanks = false;
			}
			if (s.equals("-c")) {
				set_ncustomers = true;
				continue;
			}
			if (set_ncustomers == true) {
				ncustomers = Integer.parseInt(s);
				set_ncustomers = false;
			}
			if (s.equals("-bw")) {
				set_bankwait = true;
				continue;
			}
			if (set_bankwait == true) {
				bankwait = Integer.parseInt(s);
				set_bankwait = false;
			}

			if (s.equals("-wt")) {
				set_work_time = true;
				continue;
			}
			if (set_work_time == true) {
				work_time = Integer.parseInt(s);
				set_work_time = false;
			}
			if (s.equals("-nq")) {
				set_nrequests = true;
				continue;
			}
			if (set_nrequests == true) {
				nrequests = Integer.parseInt(s);
				set_nrequests = false;
			}
			if (s.equals("-sk")) {
				set_nskips = true;
				continue;
			}
			if (set_nskips == true) {
				nskips = Integer.parseInt(s);
				set_nskips = false;
			}
			if (s.equals("-nr")) {
				set_nresults = true;
				continue;
			}
			if (set_nresults == true) {
				nresults = Integer.parseInt(s);
				set_nresults = false;
			}
			if (s.equals("-m")) {
				set_call_v_closure = true;
				continue;
			}
			if (set_call_v_closure == true) {
				if (s.equals("call")) use_closure = false;
				if (s.equals("closure")) use_closure = true;
				if (!s.equals("closure") && !s.equals("call")) {
					usage();	
				}
				set_call_v_closure = false;
			}
		}
		System.out.print(
				   "NWORKERS = " + nworkers + "\n" + 
				   "NBANKS = " + nbanks + "\n" + 
				   "NCUSTOMERS = " + ncustomers + "\n" +
				   "NREQUESTS = " + nrequests + "\n" +
				   "NRESULTS = " + nresults + "\n" +
				   "SKIP = " + nskips + "\n" +
				   "BANKWAIT = " + bankwait + "\n" +
				   "WORKTIME = " + work_time + "\n"
				);
		if (use_closure) System.out.println("USE CLOSURES");
		else System.out.println("USE CALLS");
				   
		try {
			MintServer1 ms = new MintServer1(nworkers,
				use_closure, ncustomers, nbanks, nrequests, nskips, nresults,
				bankwait, work_time);
			System.out.println("MintServer1 created.");
		AeolusLabel mySecrecy = AeolusLib.getSecrecy();
		System.out.println("Main after ms creation: My SLabel = " + mySecrecy.toString());
			ms.runtest(ms);
			System.exit(0);
		} catch (Exception e) {
			System.out.println("Mint Server terminating due to exception " + e);
			e.printStackTrace();
		}
	}

}
