package edu.mit.csail.aeolus.apps.mint1;
import java.util.*;

import edu.mit.csail.aeolus.api.*;
import java.util.concurrent.Callable;

public class MintWorker implements Runnable {
	private String MyName;
	private Command MyCommand = null;
	private String MyCommandName = "";
	private Vector<Customer> Customers;
	private Vector<Bank> Banks;
	private PID MyPID;
	private AeolusQueue<Command> MyInputQ = null;
	private AeolusQueue<Command> MyOutputQ = null;
	// private AeolusUser u;
	private Vector<Transaction> RecentTransactions = new Vector<Transaction>(100);
	private final long serialVersionUID = 1L;
	private int BankWait = 1;
	boolean use_closure = true;

	MintWorker(String id, PID pid, AeolusQueue<Command> inputQ, AeolusQueue<Command> outputQ, boolean use_closures) {
		MyName = id;
		MyPID = pid;
		Customers = new Vector<Customer>();
		MyInputQ = inputQ;
		MyOutputQ = outputQ;
		use_closure = use_closures;
		// if (use_closure) System.out.println("MW: USING CLOSURES");
		// if (!use_closure) System.out.println("MW: USING CALLS");
		// System.out.println("initialization complete " + id + " IQ " + inputQ + " OQ " + outputQ);
		// System.out.println(MyName + ": Initialization Complete ");
	}

	public void addCustomer(Customer c) throws Exception {
		Customers.addElement(c);
		// runs ok, but not the right effect
		// u.actFor(MyPID, c.getPID());
	}

	public void setBanks(Vector<Bank> banks, int bankwait) {
		this.Banks = banks;
		this.BankWait = bankwait;
		// System.out.println("MW: BankWait = " + BankWait);
	}

	public void setInputQueue(AeolusQueue InputQ) {
		this.MyInputQ = InputQ;
	}

	public PID getPID() {
		return MyPID;
	}

	// public void setCommand(String cmd) throws Exception {
	// 	MyCommand = cmd;
	// }

	public Command getNextCommand() throws Exception {
		boolean debug = true;
		if (debug) System.out.println("MW " + MyName + " Entering getNextCommand " + MyInputQ);
		while (MyInputQ == null) {
			Thread.yield();
			try {
				Thread.sleep(10);
			} catch (Exception e) {}
		}
		Command cmd = MyInputQ.dequeue();
		if (cmd  == null) return null;
		// if (debug) System.out.println("MW returning from dequeue");
		if (debug) {
			String cmd_name = cmd.get_command();
			System.out.println("MW " + MyName + " CMD = " + cmd_name);
		}
		return cmd;
	}

	// public void run() throws Exception {
	public void run() {
	        // try {
		boolean debug = true;
		long start_time, end_time, delta_time;
		// u = new AeolusUser();
		// First sleep for a while...  so that server can launch all workers before outputq has content
		Thread.yield();
		try {Thread.sleep(100);} catch (Exception e) {}
		// First tell the server that this thread is ready.
		Command started = new Command ("STARTED", MyName, null);
		if (debug) System.out.println("MW " + MyName + " Sending STARTED ");
		try {MyOutputQ.enqueue(started); }  catch (Exception e) {}
		while (true) {
		try { MyCommand = getNextCommand(); }  catch (Exception e) {}
		if (MyCommand == null) continue;
		MyCommandName = MyCommand.get_command();
		
		if (MyCommandName.equals("QUIT")) {
			System.out.println("MW " + MyName + " Received QUIT command ");
			Command done = new Command ("DONE", MyName, null);
			try { MyOutputQ.enqueue(done); }  catch (Exception e) {}
			System.out.println("MW " + MyName + " Terminating due QUIT command ");
			return;
			}
		// System.out.println("MW " +  MyCommand);

		if (MyCommandName.equals("AddCustomer")) {
			AeolusTag customer_tag = MyCommand.get_tag();
			@SuppressWarnings("unchecked")
			AeolusBox<Customer> cID =
				(AeolusBox) MyCommand.get_arg();
			try { AeolusLib.addSecrecy(customer_tag); }  catch (Exception e) {}
			// System.out.println("MW: after secrecy " + my_infoflow);
			try { AeolusLib.endorse(customer_tag); }  catch (Exception e) {}
			// System.out.println("MW: after endorse " + my_infoflow);
			Customer c = null;
			try { c = cID.get(); } catch (Exception e) {}
			try { addCustomer(c); }  catch (Exception e) {}
			try { AeolusLib.removeIntegrity(customer_tag); }  catch (Exception e) {}
			try { AeolusLib.declassify(customer_tag); } catch (Exception e) {}
		}
		if (MyCommandName.equals("PrintStatus")) {
			// System.out.println("MW: STATUS " + MyName + " " + Companies.size() + " companies + resume for " + CurrentUser.getName());
			System.out.println("MW: STATUS " + MyName + " " + Customers.size() + " Customers");
		}
		// Um... can't use input queue which doesn't exist yet to get a Command containing the inputq...
		if (MyCommandName.equals("SetInputQ")) {
			// System.out.println("MW: STATUS " + MyName + " " + Companies.size() + " companies + resume for " + CurrentUser.getName());
			AeolusQueue<Command> q = (AeolusQueue<Command>) MyCommand.get_arg();
			this.MyInputQ = q;
			System.out.println("MW: STATUS " + MyName + " " + Customers.size() + " Customers");
		}
		if (MyCommandName.equals("FetchRecentTransactions")) {
			if (debug) System.out.println("MW: " + MyName + " Starting FetchRecentTransactions");
			if (debug) System.out.println("MW: " + MyName + " # Customers " + Customers.size());
			if (debug) System.out.println("MW: " + MyName + " # Banks " + Banks.size());
			// results.removeAllElements();
		        AeolusTag ctag = MyCommand.get_tag();
			AeolusBox<Customer> asoc = (AeolusBox<Customer>) MyCommand.get_arg();
			if (debug) System.out.println("MW: " + MyName + " ctag " + ctag);
			Customer c = null;
			try { c = asoc.get(); } catch (Exception e) {System.out.println(e); System.exit(-1);}
			try { AeolusLib.addSecrecy(ctag); }  catch (Exception e) {System.out.println(e); System.exit(-1);}
			if (debug) System.out.println("MW: " + MyName + " Customer " + c);
			for (Bank b : Banks) {
				Callable<Integer> bank_callable = null;
				// Bank.CC closure = null;
				AeolusClosure closure = null;
				AeolusTag btag = b.getTag();
				String bank_name = b.getName();
				try { AeolusLib.addSecrecy(btag); }  catch (Exception e) {}
				// u.endorse(ctag);
				try { Credentials creds = c.getCredentials(bank_name); }  catch (Exception e) {}
				try { AeolusLib.declassify(ctag); }  catch (Exception e) {}
				if (debug) System.out.println("Customer " + c.getName() + " Bank "  + b.getName());
				// Would like to pass the credentials into the Bank, but other workers
				// 	are using the same bank: well, we have our own copy...
				if (use_closure == false) {
					PID bank_pid = b.getPID();
					bank_callable = b.fetchRecentTransactions;
					start_time = System.currentTimeMillis();
					Thread.yield();
					try { Thread.sleep(BankWait); }  catch (Exception e) {}
					end_time = System.currentTimeMillis();
					delta_time = end_time - start_time;
					if (debug) System.out.println("MW: " + MyName + " " + delta_time);
					try { AeolusLib.call(bank_callable, bank_pid); }  catch (Exception e) {}
				} 
				if (use_closure == true) {
					// closure_objid = b.getClosureID();
					closure = b.bank_closure;
					Thread.yield();
					try { Thread.sleep(BankWait); }  catch (Exception e) {}
					// closure.invoke();
					// AeolusLib.callClosure(closure_objid, closure_code);
				}
				// u.removeIntegrity(btag);
				try { AeolusLib.declassify(btag); }  catch (Exception e) {}
				try { AeolusLib.addSecrecy(ctag); }  catch (Exception e) {}
				// put Recent Transactions into Customer object
				this.RecentTransactions = b.getRecentTransactions();
				start_time = System.currentTimeMillis();
				c.updateRecentTransactions(bank_name, this.RecentTransactions);
				end_time = System.currentTimeMillis();
				delta_time = end_time - start_time;
				if (debug) System.out.println("MW: " + MyName + " " + delta_time);
				}
			try { AeolusLib.declassify(ctag); }  catch (Exception e) {}
			} // end FetchRecentTransactions
			Command summary = new Command ("RESULTS", MyName, null);
			if (debug) System.out.println("MW " + MyName + " Sending RESULTS ");
			try { MyOutputQ.enqueue(summary); }  catch (Exception e) {}
		    } // while 
		// } // try
		// catch (Exception e) {
		// 	throw e;
		// }
	} // end run

}
