package edu.mit.csail.aeolus.apps.mint3;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import edu.mit.csail.aeolus.api.AeolusLib;
import edu.mit.csail.aeolus.api.AeolusShared;
import edu.mit.csail.aeolus.api.AeolusTag;
import edu.mit.csail.aeolus.api.PID;
import edu.mit.csail.aeolus.exceptions.AeolusException;
import edu.mit.csail.aeolus.exceptions.AuthorityException;
import edu.mit.csail.aeolus.exceptions.InfoFlowControlException;

/**
 * banksMap - stores banks that this customer regsitered to on Mint.
 * userDataTag - subtag of allUserDataTag in MintServer
 * 
 * 
 * @author wjarjoui
 *
 */
public class Customer extends AeolusShared implements MintObject, Serializable, MintPrincipal{

	/**
	 * Change this class to be immutable.
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private String username;
	private PID customerPID;
	private AeolusTag userDataTag;
	private String password;
	private List<BankCredentialsWrapper> bankCreds = new ArrayList<BankCredentialsWrapper>();;
	
//	public Customer(String username, String password){
//		this.username = username;
//		this.password = password;
//	}
	
	public Customer(String username, String password, AeolusTag userDataTag) throws InfoFlowControlException{
		super(Utils.createLabel(userDataTag), Utils.createLabel(userDataTag));
		this.username = username;
		this.password = password;

		try {
			this.customerPID = new PID();
			AeolusLib.getPID().actFor(customerPID);
			this.userDataTag = userDataTag;
			this.userDataTag.delegate(AeolusLib.getPID(), this.customerPID);
		} catch (InfoFlowControlException e) {
			e.printStackTrace();
		} catch (AuthorityException e) {
			e.printStackTrace();
		} catch (AeolusException e) {
			e.printStackTrace();
		}
		checkrep();
	}
	
	private void checkrep() {
//		assert(userDataTag.hasAuthority(customerPID));
//		assert(getSLabel().hasTag(userDataTag));
//		assert(getILabel().hasTag(userDataTag));
	}
	
	public String getUsername() {
		return username;
	}

	public PID getCustomerPID() {
		return customerPID;
	}
	
	@Override
	public int hashCode() {
		return username.hashCode();
	}
	
	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (!(obj instanceof Customer)) return false;
		Customer cus = (Customer) obj;
		return cus.getUsername().equals(username);
	}

	public String getPassword() {
		return password;
	}
	
//	public AeolusTag getUserDataTag() {
//		return userDataTag;
//	}

	@Override
	public String toString() {
		StringBuffer print = new StringBuffer();
		print.append("Customer\n  ");
		print.append("username: ");
		print.append(username);
		print.append("\n  ");
		print.append("PID: ");
		print.append(customerPID);
		print.append("\n  ");
		print.append("tag: ");
		print.append(userDataTag);
		print.append("\n  ");
		print.append("password: ");
		print.append(password);
		return print.toString();
	}
	
	@Override
	public String getFileName() {
		return username;
	}

	public List<BankCredentialsWrapper> getBankCredentials() {
		return bankCreds;
	}
	
	public void addBankCredentials(BankCredentialsWrapper creds){
		bankCreds.add(creds);
	}

	public void removeBankCredentials(String bankName) {
		// TODO Auto-generated method stub
	}
}
