package edu.mit.csail.aeolus.apps.mint3;

import java.math.BigInteger;
import java.security.SecureRandom;
import java.util.ArrayList;
import java.util.List;

import edu.mit.csail.aeolus.exceptions.AeolusException;
import edu.mit.csail.aeolus.exceptions.AuthorityException;

public class LoadGenerator {
	
	private InterfaceUserActions stub;
	private List<User> users;
	
	public LoadGenerator(InterfaceUserActions stub) {
		super();
		this.stub = stub;
		this.users = new ArrayList<User>();
		genRandomUsers();
	}

	public InterfaceUserActions getStub() {
		return stub;
	}

	public List<User> getUsers() {
		return users;
	}
	
	public void setup() throws AuthorityException, AeolusException, Exception{
		if (users.size() == 0)
			System.out.println("Please add users before signing them up");
		for (User u: users){
			System.out.println("Created User with username = '" + u.getName() + 
					"', and password = '" + u.getPassword() +"'");
			stub.userSignup(u.getName(), u.getPassword());
		}
	}
	
	
	public void addUser(User user){
		users.add(user);
	}
	
	public void randomActions(){
		for(int i = 0; i< 20; i++){
			for (final User u : users){
				stub.userLogin(u.getName(), u.getPassword());
				performRandomActivity(u);
			}
		}
	}
	
	private void performRandomActivity(User u) {
		double rand = Math.random();
		String x = Double.toString(rand);
		if (rand > 0.15){
			if (rand > 0.75){
				System.out.println("Attempting userCollectReport for user " + u.getName());
				boolean result = stub.userCollectReport(u.getName());
				if (result)
					System.out.println("Collected report for user " + u.getName());
			}
			else if (rand > 0.60){
				System.out.println("Attempting to log in user: " + u.getName());
				boolean result = stub.userLogin(u.getName(), u.getPassword());
				if (result)
					System.out.println("Logged in user " + u.getName());
			}
			else{
				String bankToAdd = Banks.getRandomBank();
				System.out.println("Attempting userAddBank for user " + u.getName() + ", Bank "
						+ bankToAdd);
				boolean result = stub.userAddBank(u.getName(), bankToAdd, u.getName(), x + u.getPassword());
				if (result)
					System.out.println("Added Bank " + bankToAdd + " to user " + u.getName());
			}
		} else {
			String victim = getRandomUser().getName();
			System.out.println("Attempting attack from " + u.getName()
					+ " victim: " + victim);
			String victimPassword = stub.attack(u.getName(), victim);
			System.out.println(u.getName() + " attacked " + victim
					+ " with password: " + victimPassword);
		}
	}
	
	public void randomActions(User u){
		for (int i = 0; i<50; i++)
			performRandomActivity(u);
	}

	public User getRandomUser(){
		int rand = (int) (Math.random()*users.size());
		return users.get(rand);
	}
	
	public String createRandomPassword(){
		return new BigInteger(130, new SecureRandom()).toString(32);
	}
	
	public void genRandomUsers(){
		for (Users u: Users.values()){
			users.add(new User(u.toString(), createRandomPassword()));
		}
	}
}
