package edu.mit.csail.aeolus.apps.mint3;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.Serializable;
import java.util.Map;

import edu.mit.csail.aeolus.api.AeolusFile;
import edu.mit.csail.aeolus.api.AeolusLabel;
import edu.mit.csail.aeolus.api.AeolusLib;
import edu.mit.csail.aeolus.api.AeolusTag;
import edu.mit.csail.aeolus.api.PID;
import edu.mit.csail.aeolus.auditing.EventID;
import edu.mit.csail.aeolus.exceptions.AeolusException;
import edu.mit.csail.aeolus.exceptions.AuthorityException;
import edu.mit.csail.aeolus.exceptions.InfoFlowControlException;

public class MintServer implements Serializable {

	private MintServerState state;
	private AeolusTag bTag;
	private AeolusTag uTag;
	
	public MintServer(MintServerState state) {
		this.state = state;
		uTag = state.getAllUserDataTag();
		bTag = state.getAllBanksTag();
	}

	public Bank getBank(String bankName){
		return getBank(bankName, true);
	}

	public BankClosure getBankClosure(String bankName){
		return state.getBankClousre(bankName);
	}
	
	public Bank getBank(String bankName, boolean create) {
		//check if bank already exists
		Bank bank = state.getBank(bankName);
		if ((bank != null) || !create)
			return bank;
		AeolusTag bankTag;
		try {
			bankTag = new AeolusTag(state.getAllBanksTag());
			AeolusLib.endorse(bankTag);
			bank = new Bank(bankName, bankTag);
			PID pid = bank.getBankPID();
			AeolusLib.removeIntegrity(bankTag);
			state.addBankTag(bankName, bankTag);
			state.addBank(bankName, bank);
			state.addBankClosure(bankName, new BankClosure(pid));
//			pid.actFor(getPublicPID());
			addBankToDisk(bankName, bank);
		} catch (InfoFlowControlException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (AuthorityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (AeolusException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return bank;
	}
	private Customer readCustomer(String username) throws MintServerException {
		try {
			AeolusTag userDataTag = state.getUserTag(username);
			AeolusLib.addSecrecy(userDataTag);
			Customer cus = FSReader.readCustomer(username + "/data");
			if (cus == null)
				throw new MintServerException("Customer does not exist");
			AeolusLib.declassify(userDataTag);
			return cus;
		} catch (AuthorityException e) {
		 	e.printStackTrace();
		} catch (AeolusException e) {
		 	e.printStackTrace();
		}
		return null;
	}
	
	public boolean addCustomer(String username, String password) {
		try{
			AeolusTag userDataTag = new AeolusTag(uTag);
			AeolusLib.endorse(userDataTag);
			AeolusLib.addSecrecy(userDataTag);
			Customer cus = new Customer(username, password, userDataTag);
			PID pid = cus.getCustomerPID();
			String path = MintConfig.customers + "/" + cus.getFileName();
			String toString = cus.getUsername() + ",[" + userDataTag + "]";
			AeolusLib.declassify(userDataTag);
			AeolusLib.removeIntegrity(userDataTag);
			state.addUserTag(username, userDataTag);
			state.addCustomer(username, cus);
//			AeolusLib.call(new AddCustomerEvent(toString), pid);
			AddCustomerEventClosure clos = new AddCustomerEventClosure(pid);
			clos.addCustomerEvent(toString);
			AeolusLib.endorse(uTag);
			AeolusFile f = new AeolusFile(MintConfig.host, path);
			AeolusLabel userLabel = new AeolusLabel();
			userLabel.addTag(userDataTag);
			f.mkdir(userLabel, userLabel);
			AeolusLib.removeIntegrity(uTag);
			if(!AeolusLib.call(new CustomerWriter(cus, userDataTag), pid))
				throw new MintServerException("Could not create customer");
			else
				return true;
			
		}catch(Exception e){
			e.printStackTrace();
		}
		return false;
	}
	
	public boolean isLoggedIn(String username) {
		return state.isLoggedIn(username);
	}
		
	private boolean addBankToDisk(String bankName, Bank bank){
		try {
			AeolusTag bankTag = getBankTag(bankName);
			AeolusLib.endorse(bankTag);
			String path = MintConfig.banks + "/" + bank.getFileName();
			AeolusFile f = new AeolusFile(MintConfig.host, path);
			AeolusLib.endorse(bTag);
			AeolusLabel label = new AeolusLabel();
			label.addTag(bankTag);
//			f.createNewFile(new AeolusLabel(), label);
//			MintSerializer.serializeObjectToFile(path, bank);
			return true;
//		} catch (FileNotFoundException e) {
//			e.printStackTrace();
//		} catch (InfoFlowControlException e) {
//			e.printStackTrace();
//		} catch (IOException e) {
//			e.printStackTrace();
		} catch (AeolusException e) {
			e.printStackTrace();
		}
		finally{
			try {
				AeolusLib.removeIntegrity(bTag);
			} catch (AeolusException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return false;
	}

	public boolean authenticateCustomer(String username, String password) {
		boolean ret = false;
		AeolusTag userTag = getUserTag(username);
		Customer cus = getCustomer(username);
		if (cus == null)
			return false;
		try {
			AeolusLib.addSecrecy(userTag);
			AeolusLib.endorse(userTag);
			ret = cus.getPassword().equals(password);
			AeolusLib.declassify(userTag);
			AeolusLib.removeIntegrity(userTag);
		} catch (AeolusException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return ret;
	}

	public void userLogout(String username) {
		state.userLogout(username);
	}
	
	public void updateUserActivity(String username) {
		state.updateUserActivity(username);
	}

	public Customer getCustomer(String username) {
		if (isLoggedIn(username)) return state.getCustomer(username);
		else
			try {
				Customer cus = readCustomer(username);
				if (cus != null){
					state.addCustomer(username, cus);
					return cus;
				}
			} catch (MintServerException e) {
				e.printStackTrace();
			}
		return null;
	}

	public boolean addBankCredential(Customer cus, String username, String bankName, String bankUsername, String bankPassword) {
		try {
			AeolusTag userTag = state.getUserTag(username);
			AeolusTag bankTag = state.getBankTag(bankName);
			AeolusLib.endorse(userTag);
			AeolusLib.addSecrecy(userTag);
			PID pid = cus.getCustomerPID();
			BankCredentialsWrapper bankCreds = new BankCredentialsWrapper(username, bankName, userTag, bankTag, bankUsername, bankPassword);
			cus.addBankCredentials(bankCreds);
			AeolusLib.declassify(userTag);
			AeolusLib.removeIntegrity(userTag);
			AeolusLib.call(new BankCredentialsWriter(bankCreds, userTag, bankTag), pid);
			AeolusLib.call(new CustomerWriter(cus, userTag), pid);
			return true;
		} catch (AuthorityException e) {
			e.printStackTrace();
		} catch (AeolusException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;
	}

	public PID getPublicPID() {
		return state.getPublicPID();
	}

	public AeolusTag getUserTag(String username){
		return state.getUserTag(username);
	}
	
	public AeolusTag getBankTag(String bankName){
		return state.getBankTag(bankName);
	}

	public Map<String, BankClosure> getClosures() {
		return state.getClosures();
	}
}
