package edu.mit.csail.aeolus.apps.mint3;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import edu.mit.csail.aeolus.api.AeolusLib;
import edu.mit.csail.aeolus.api.AeolusShared;
import edu.mit.csail.aeolus.api.AeolusTag;
import edu.mit.csail.aeolus.api.PID;
import edu.mit.csail.aeolus.exceptions.AeolusException;
import edu.mit.csail.aeolus.exceptions.AuthorityException;

/**
 * Server maintains a list of customers and their Mint credentials.
 * 
 * @author wjarjoui
 *
 */
public class MintServerState extends AeolusShared implements Serializable {
	
	/**
	 * 
	 */
	private final long serialVersionUID = 1L;
	
	private PID serverpid;
	private PID publicPID;
	private AeolusTag allUserDataTag;
	private AeolusTag allBanksTag;
	private HashMap<String, Long> activeSessions;
	private HashMap<String, AeolusTag> bankNameToTags;
	private HashMap<String, AeolusTag> userNameToTags;
	private HashMap<String, Customer> customers;
	private HashMap<String, Bank> banks;
	private HashMap<String, BankClosure> bankNameToClosure;
	
	private long sessionTimeout;// = 360*1000l;

	
	public MintServerState(long sessionTimeout, AeolusTag allUserDataTag, AeolusTag allBanksTag) throws AuthorityException, AeolusException, FileNotFoundException, IOException{
		super();
		this.allUserDataTag = allUserDataTag;
		this.allBanksTag = allBanksTag;
		activeSessions = new HashMap<String,Long>();
		bankNameToTags = new HashMap<String, AeolusTag>();
		userNameToTags = new HashMap<String, AeolusTag>();
		customers = new HashMap<String, Customer>();
		banks = new HashMap<String, Bank>();
		bankNameToClosure = new HashMap<String, BankClosure>();
		this.sessionTimeout = sessionTimeout;
		serverpid = new PID();
		allUserDataTag.delegate(AeolusLib.getPID(), serverpid);
		allBanksTag.delegate(AeolusLib.getPID(), serverpid);
		publicPID = new PID();
		serverpid.actFor(publicPID);
	}

	public AeolusTag getAllUserDataTag() {
		return allUserDataTag;
	}

	public long getSessionTimeout() {
		return sessionTimeout;
	}
	
	public boolean isLoggedIn(String username) {
		if(!activeSessions.keySet().contains(username)){
			return false;
		}
		else if((System.currentTimeMillis() - activeSessions.get(username)) > sessionTimeout){
			//remove use from memory
			activeSessions.remove(username);
			customers.remove(username);
			return false;
		}
		else{
			updateUserActivity(username);
			return true;
		}
	}
	
	public void updateUserActivity(String username) {
		activeSessions.put(username, System.currentTimeMillis());
	}
	
	public boolean userLogout(String username) {
		activeSessions.remove(username);
		customers.remove(username);
		return true;
	}

	public AeolusTag getAllBanksTag() {
		return allBanksTag;
	}

	public AeolusTag getBankTag(String bankName) {
		return bankNameToTags.get(bankName);
	}
	
	public void addBankTag(String bankName, AeolusTag bankTag){
		bankNameToTags.put(bankName, bankTag);
	}
	
	public AeolusTag getUserTag(String username) {
		return userNameToTags.get(username);
	}
	
	public void addUserTag(String username, AeolusTag userDataTag){
		userNameToTags.put(username, userDataTag);
	}
	
	public Customer getCustomer(String username) {
		return customers.get(username);
	}

	public Bank getBank(String bankName) {
		return banks.get(bankName);
	}
	
	public void addCustomer(String username, Customer cus){
		customers.put(username, cus);
		updateUserActivity(username);
	}
	public void addBank(String bankName, Bank bank){
		banks.put(bankName, bank);
	}

	public PID getPid() {
		return serverpid;
	}

	public PID getPublicPID() {
		return publicPID;
	}
	
	public BankClosure getBankClousre(String bankName) {
		return bankNameToClosure.get(bankName);
	}
	
	public void addBankClosure(String bankName, BankClosure closure){
		bankNameToClosure.put(bankName, closure);
	}

	public Map<String, BankClosure> getClosures() {
		return bankNameToClosure;
	}

	
	// THE NEXT METHODS TO BE USED FOR TESTING ONLY!
	public HashMap<String, Customer> getCustomers() {
		return customers;
	}

	public HashMap<String, Bank> getBanks() {
		return banks;
	}
}
