package edu.mit.csail.aeolus.apps.mint3;

import java.util.List;
import java.util.Map;
import java.util.concurrent.Callable;

import edu.mit.csail.aeolus.api.AeolusLib;
import edu.mit.csail.aeolus.api.AeolusTag;
import edu.mit.csail.aeolus.api.PID;
import edu.mit.csail.aeolus.exceptions.AeolusException;
import edu.mit.csail.aeolus.exceptions.AuthorityException;
import edu.mit.csail.aeolus.exceptions.InfoFlowControlException;

public class UserActions implements InterfaceUserActions {

//	private PrincipalID allUsers;
//	private TagID allUserDataTag;
//	private Map<String, Customer> customers;
//	private Map<String, Long> activeSessions;
//	private long sessionTimeout = 360*1000l;
	private MintServer server;
	

	public UserActions() throws InfoFlowControlException, AeolusException{
		server = new MintServer((MintServerState) AeolusLib.getRoot());
	}

	private boolean isLoggedIn(String username){
		return server.isLoggedIn(username);
	}
	
	@Override
	public boolean userSignup(String username, String password){
		if (server.isLoggedIn(username)) return false;
		if(!server.addCustomer(username, password)) return false; //return false if customer already exists
		server.updateUserActivity(username);
		return userLogin(username, password);
	}
	

	@Override
	public boolean userLogin(String username, String password){
		return server.authenticateCustomer(username, password);
	}

	@Override
	public boolean userLogout(String username) {
		server.userLogout(username);
		return true;
	}

	@Override
	public boolean userAddBank(String username, String bankName,
			String bankUsername, String bankPassword) {
		if(!isLoggedIn(username)) return false;
		Bank bank = server.getBank(bankName, true);
		if(bank == null)
			return false;
		Customer cus = server.getCustomer(username);
		server.addBankCredential(cus, username, bankName, bankUsername, bankPassword);
		return true;
	}

	@Override
	public boolean userRemoveBank(String username, String bankName) {
		if(!isLoggedIn(username)) return false;
		Customer cus = server.getCustomer(username);
//		cus.removeBankCredentials(bankName);
		return true;
	}

	@Override
	public boolean userCollectReport(String username) {
		Customer cus = server.getCustomer(username);
		AeolusTag userTag = server.getUserTag(username);
		UserInfo info;
		try {
			Map<String, BankClosure> closures = server.getClosures();
			AeolusLib.addSecrecy(userTag);
			AeolusLib.endorse(userTag);
			List<BankCredentialsWrapper> creds = cus.getBankCredentials();
			info = AeolusLib.call(new TransactionDownloader(creds, closures), cus.getCustomerPID());
			AeolusLib.declassify(userTag);
			AeolusLib.removeIntegrity(userTag);
			return true;
		} catch (AuthorityException e) {
			e.printStackTrace();
		} catch (AeolusException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return true;
	}

	@Override
	public String attack(String attacker, String victim) {
		AeolusTag attackerTag = server.getUserTag(attacker);
		AeolusTag victimTag = server.getUserTag(victim);
		Customer vict = server.getCustomer(victim);
		Customer attk = server.getCustomer(attacker);
		String ret = "If you see this then an exception took place, attack failed";
		
		try {
			//retrieve victim PID
			AeolusLib.addSecrecy(victimTag);
			AeolusLib.endorse(victimTag);
			PID victimPID = vict.getCustomerPID();
			AeolusLib.declassify(victimTag);
			AeolusLib.removeIntegrity(victimTag);
			
			//retrieve attacker PID
			AeolusLib.addSecrecy(attackerTag);
			AeolusLib.endorse(attackerTag);
			PID attackerPID = attk.getCustomerPID();
			AeolusLib.declassify(attackerTag);
			AeolusLib.removeIntegrity(attackerTag);
			
			//should never really happen - this is obviously a faked attack
//*/
			victimTag.delegate(victimPID, attackerPID);
			/*/
			attackerPID.actFor(victimPID);
			System.out.println(victimPID);
			System.out.println(attackerPID);
			System.out.println("This thread acts for victim");
			System.out.println(AeolusLib.getPID().doesActFor(victimPID));
			System.out.println("attacker acts for victim");
			System.out.println(attackerPID.doesActFor(victimPID));
			System.out.println("attacker has authority over victim tag");
			System.out.println(victimTag.hasAuthority(attackerPID));
			System.out.println("victim has authority over victim tag");
			System.out.println(victimTag.hasAuthority(victimPID));
			System.out.println("specifically delegating the victim tag to attacker");
			victimTag.delegate(victimPID, attackerPID);
			System.out.println("attacker has authority over victim tag");
			System.out.println(victimTag.hasAuthority(attackerPID));
			System.out.println("flipping the way actFor is called");
			victimPID.actFor(attackerPID);
			System.out.println("attacker acts for victim");
			System.out.println(attackerPID.doesActFor(victimPID));
			System.out.println("attacker has authority over victim tag");
			System.out.println(victimTag.hasAuthority(attackerPID));
//*/
			//expose the victim's data using the attacker PID
			AttackClosure clos = new AttackClosure(attackerPID);
			ret = clos.attack(new AttackCallable(victimTag, vict));
//			ret = AeolusLib.call(new AttackCallable(victimTag, vict), attackerPID);
			
		} catch (AeolusException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return ret;
	}
}
