package edu.mit.csail.aeolus.auditing;

import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Queue;

import edu.mit.csail.aeolus.runtime.RuntimeConfig;

/**
 * Sends the log record data to the central logging server
 * 
 * @author viq
 */

public class AeolusLoggingClient {
	
	LoggingServiceInterface service;
	private final String aeolusLoggingServiceName, loggingHostName;
	private final int loggingPort;
	
	public AeolusLoggingClient(){
		RuntimeConfig c = RuntimeConfig.get();
		aeolusLoggingServiceName = c.getAeolusLoggingServiceName();
		loggingHostName = c.getLoggingHostName();
		loggingPort = c.getLoggingPort();
	}
	
	public void sendToServer(Integer platformid,  Date timestamp, HashMap<Long, ArrayList<EventRecord>> eventsTable)
	{
		Registry registry;
    	try{
    		registry = LocateRegistry.getRegistry(loggingHostName, loggingPort); 
    		service = (LoggingServiceInterface) registry.lookup(aeolusLoggingServiceName);
    		service.receiveLogs(platformid,  timestamp, eventsTable);
    	}
    	catch (Exception e){
    		e.printStackTrace();
    	}
	}
	
	public void sendFileLogToServer(String hostname, Date timestamp, Queue<EventRecord> eventsList)
	{
		Registry registry;
    	try{
    		registry = LocateRegistry.getRegistry(loggingHostName, loggingPort);         
    		service = (LoggingServiceInterface) registry.lookup(aeolusLoggingServiceName);
    		service.receiveFileSystemLogs(hostname, timestamp, eventsList);
    	}
    	catch (Exception e){
    		e.printStackTrace();
    	}
	}

	public void sendAuthLogToServer(Date timestamp, ArrayList<EventRecord> eventsList)
	{
		Registry registry;
    	try{
    		registry = LocateRegistry.getRegistry(loggingHostName, loggingPort);         
    		service = (LoggingServiceInterface) registry.lookup(aeolusLoggingServiceName);
    		service.receiveAuthLogs(timestamp, eventsList);
    	}
    	catch (Exception e){
    		e.printStackTrace();
    	}
	}

	
	public int getSessionUid()
	{
		Registry registry;
    	try{
    		registry = LocateRegistry.getRegistry(loggingHostName, loggingPort);         
    		service = (LoggingServiceInterface) registry.lookup(aeolusLoggingServiceName);	
    		return service.getSessionUid();
    	}
    	catch (Exception e){
    		e.printStackTrace();
    	}
    	return -1;
	}

}
