package edu.mit.csail.aeolus.auditing;

import java.io.Serializable;
import java.util.LinkedList;

import edu.mit.csail.aeolus.infoflow.PrincipalID;

/**
 * This class is a representation of HOW a particular authority was derived.
 * 
 * Proofs are constructed by adding grants/act-fors from the top level 
 * toward the bottom level (ultimate recipient of authority)
 * @author blanks
 *
 */
public class AuthorityProof implements Serializable {
	private static final long serialVersionUID = 1L;

	LinkedList<ProofEntry> proof;
	private final short ACT_FOR = 0;
	private final short DELEGATE = 1;

	public AuthorityProof(){
		proof = new LinkedList<ProofEntry>();
	}

	public AuthorityProof(String fromStr){
		String[] entries = fromStr.split(";");
		proof = new LinkedList<ProofEntry>();
		for(String entry : entries){
			if("".equals(entry))
				continue;
			String[] parts = entry.split(":");
			proof.add(new ProofEntry(new Short(parts[0]),
					new Integer(parts[1]),
					new Integer(parts[2])));
		}
	}

	public void addExplicitDelegate(int pidGrantor, int pidGrantee){
		proof.add(new ProofEntry(DELEGATE, pidGrantor, pidGrantee));
	}

	/**
	 * @param actor ACTS FOR actee
	 */
	public void addActFor(int actee, int actor) {
		proof.add(new ProofEntry(ACT_FOR, actee, actor));
	}

	public boolean usesAuthorityOf(PrincipalID pid){
		int val = pid.getValue();
		for(ProofEntry p : proof){
			if(p.pidGrantor == val)
				return true;
		}
		return false;
	}

	private static class ProofEntry{
		short type; int pidGrantor, pidGrantee;
		public ProofEntry(short type, int pidGrantor, int pidGrantee) {
			this.type = type;
			this.pidGrantor = pidGrantor;
			this.pidGrantee = pidGrantee;
		}
	}

	public String toString(){
		StringBuffer out = new StringBuffer();
		for(ProofEntry entry : this.proof){
			out.append(entry.type);
			out.append(":");
			out.append(entry.pidGrantor);
			out.append(":");
			out.append(entry.pidGrantee);
			out.append(";");
		}
		return out.toString();
	}
}
