package edu.mit.csail.aeolus.auditing;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Serializable;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.ArrayList;
import java.util.List;

import edu.mit.csail.aeolus.runtime.RuntimeConfig;

/**
 * Generates graphviz format files
 * to create event dependency graphs
 * 
 * @author viq
 *
 */
public class CausalityGraphGenerator {
	
	public CausalityGraphGenerator(){}
	public static void main(String[] args){
		RuntimeConfig c = RuntimeConfig.get();
		c.setLoggingHostName("localhost");
		c.finalize();

		String aeolusLoggingServiceName = c.getAeolusLoggingServiceName();
		String loggingHostName = c.getLoggingHostName();
		int loggingPort = c.getLoggingPort();
		try{
    		Registry registry = LocateRegistry.getRegistry(loggingHostName, loggingPort); 
    		LoggingServiceInterface svc = (LoggingServiceInterface) registry.lookup(aeolusLoggingServiceName);
    		svc.getAllRecords();
    		List<EventRecord> events = svc.getAllRecords();
    		CausalityGraphGenerator.generateGraphvizScript(events);
		}
    	catch (Exception e){
    		e.printStackTrace();
    	}
//		service.shutdownService();

		System.exit(0);
	}
	
	public static void generateGraphvizScript(ArrayList<CausalityLink> links)
	{
		try 
		{
			BufferedWriter out = new BufferedWriter(new FileWriter("/tmp/gv.dot", false));
			out.write("digraph G {");
			out.newLine();
			out.write("size =\"10,4\";");
			
			for(CausalityLink link: links)
			{
				//out.write("\"" + link.node + "\"" + "[label=\"" + link.opName + "\\neid=" + 
					//										 link.node + "\"];");
				out.write("\"" + link.node + "\"" + "[label=\"" + AeolusApiConstants.getMethodName(new Integer(link.opName)) + 
															"\\neid=" + link.node + "\"];");
				
				out.newLine();
				
				if(!link.predNode.isEmpty())
				{
					out.write("\"" + link.predNode + "\"" + "->" + "\"" + link.node + "\";");
					out.newLine();
				}
				
				System.out.println("Method " + AeolusApiConstants.getMethodName(new Integer(link.opName)));
			}
			
			out.write("}");
			out.close();
		} 
		catch (IOException e) 
		{
			e.printStackTrace();
		}
	}
	
	public static void generateGraphvizScript(List<EventRecord> events)
	{
		try 
		{
			BufferedWriter out = new BufferedWriter(new FileWriter("/tmp/gv.dot", false));
			out.write("digraph G {");
			out.newLine();
			out.write("size =\"10,4\";");
			
			for(EventRecord e: events)
			{
				//out.write("\"" + link.node + "\"" + "[label=\"" + link.opName + "\\neid=" + 
					//										 link.node + "\"];");
				out.write("\"" + e.getEventId().toString() + "\"" + 
						"[label=\"" + AeolusApiConstants.getMethodName(e.getMethod()) + 
								    "\\neid=" + e.getEventId() + 
								    "\\nparms=" + getParams(e.getParamValues()) +
								    "\\nret-val=" + getRetValue(e.getReturnValue()) + 
									"\"];");
				
				out.newLine();
				
				if(!e.getOtherPreds().isEmpty()){
					for(EventID pred : e.getOtherPreds()){
						out.write("\"" + pred.toString() + "\"" + "->" + "\"" + e.getEventId().toString() + "\";");
						out.newLine();
					}
				}
				
				System.out.println("Method " + AeolusApiConstants.getMethodName(e.getMethod()));
			}
			
			out.write("}");
			out.close();
		} 
		catch (IOException e) 
		{
			e.printStackTrace();
		}
	}
	
	public static String getParams(List<Serializable> list){
		String result = "";
		if(list == null)
			return result;
		for(Serializable param: list)
			result += param.toString();
		
		return result;
	}
	
	public static String getRetValue(Serializable ret)
	{
		String result = "";
		if(ret!=null)
			result = ret.toString();
		
		return result;
	}
}
