package edu.mit.csail.aeolus.auditing;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map.Entry;
import java.util.Date;

import edu.mit.csail.aeolus.infoflow.PlatformInstanceID;
import edu.mit.csail.aeolus.runtime.RuntimeConfig;

/**
 * Collects log records from all user activity
 * 
 * @author viq
 *
 */
public class Collector {
	
	/** Table of events based on User-D */
	private SynchronizedEventsTable<Long, EventRecord> eventsTable;
	private PlatformInstanceID platformId;
	AeolusLoggingClient loggingClient;
	
	public Collector(PlatformInstanceID platformInstId)
	{
		eventsTable = new SynchronizedEventsTable<Long, EventRecord>();
		platformId = platformInstId;
		RuntimeConfig c = RuntimeConfig.get();
		if(c.isEnableLogging() && c.isEnableLogShipment())
			loggingClient = new AeolusLoggingClient();
	}
	
	public void submitLogRecord(long appDomainId, EventRecord rec)
	{	
		rec.setTimestamp(new Date());
		eventsTable.addEvent(appDomainId, rec);
	}
	
	/**
	 * Sends all the logs available so far to the logging server
	 */
	public void sendLogs()
	{
		// snapshot of the currently logged events
		HashMap<Long, ArrayList<EventRecord>> contents = eventsTable.getAllAndReset();	
		// Check if its empty...
		boolean empty = true;
		for(Entry<Long, ArrayList<EventRecord>> e : contents.entrySet()){
			if(! e.getValue().isEmpty()){
				empty = false;
				break;
			}
		}
		if(!empty )
		{
			// do not send the direct predecessor if in eliding mode
			if(LoggingConstants.enableEliding)
			{
				for(ArrayList<EventRecord> list : contents.values())
				{
					if(list.size() > 1)
					{
						// do not remove the p-pred of the first element in the list
						Iterator<EventRecord> it = list.iterator();
						it.next();
						while(it.hasNext())
						{
							EventRecord rec = it.next();
							// only remove the p-pred if it's the previous event in the list
							// (these are first events in a new user thread, their p-pred
							// is different - should not be removed)
							if(!((rec.getMethod() == AeolusApiConstants.USER_THREAD_LAUNCH)
									|| (rec.getMethod() == AeolusApiConstants.RECEIVE_RPC)))
							{
//								rec.setPPredId(null);
							}
						}
					}
				}
			}
			loggingClient.sendToServer(platformId.getValue(), Calendar.getInstance().getTime(), contents);
		}
		
	}
}
