package edu.mit.csail.aeolus.auditing;

import java.io.Serializable;
import java.util.UUID;
import edu.mit.csail.aeolus.api.AeolusSafe;

import edu.mit.csail.aeolus.infoflow.PlatformInstanceID;

/**
 * User/system event ID associated with a log record
 * 
 * @author viq
 *
 */
public final class EventID implements Serializable, AeolusSafe, Cloneable {

	private static final long serialVersionUID = 1397512671619670612L;

	private final String eid;
	
	@Deprecated public final static EventID systemEvent = new EventID("SYS");
	
	/**
	 * Generates a random event id  
	 * Uses Java random UUID (version 4) - random number of 122 sig bits
	 * @deprecated - not for public use
	 */
	@Deprecated public EventID()
	{
		this.eid = UUID.randomUUID().toString();
	}
	
	/**
	 * Generates a unique event id from the given 4-tuple
	 * Just concatenates the values for now
	 * To be used for User-d event id generation
	 * @deprecated - not for public use
	 */
	@Deprecated public EventID(int pi, long userd, int generation, int eventCounter)
	{
		this.eid = pi + "-" + userd + "-" + generation + "-" + eventCounter;
	}
	
	/**
	 * Generates a unique file system event id
	 * To be used by file manager for event id generation
	 * @deprecated - not for public use
	 */
	@Deprecated public EventID(String hostname, long sessionToken, long eventCounter)
	{
		this.eid = hostname + "-" + sessionToken + "-" + eventCounter;
	}
	
	/**
	 * Generates eventID for use by the class loader
	 * param pi
	 * @deprecated - not for public use
	 */
	@Deprecated public EventID(PlatformInstanceID pi, int counter){
		this.eid = pi + "-" + "acl" + "-" + counter;
	}
	
	/**
	 * Generates eventID given a string
	 * param eventID in string form
	 * @deprecated - not for public use
	 */
	@Deprecated public EventID(String eventId)
	{
		this.eid = eventId;
	}
	
	public String toString() 
	{
		return this.eid;
	}
	
	@Override
	public int hashCode(){
		return this.eid.hashCode();
	}
	
	public boolean equals(Object e)
	{
		if(e == null)
			return false;
		if(!(e instanceof EventID))
			return false;
		return ((EventID)e).toString().equals(this.toString());
	}

        /**
          * Returns a new EventID which will be equal to this.
          */
        public EventID clone() {
                return this;
        }

}
