package edu.mit.csail.aeolus.auditing;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.concurrent.LinkedBlockingQueue;


/**
 * 
 * Background thread in charge of writing audit trails
 * 
 * @author viq
 *
 */

public class LoggerThread extends Thread {

	private static LoggerThread instance; 
	
	/** Logging request queue **/
	private LinkedBlockingQueue<EventRecord> logRequestQueue;
	
	private BufferedWriter out;
	
	/** ID of the system node at which the events are logged. */
	private int nodeID = 0;
	
	/** 
    
    /**
     * Returns an instance of the SYS-D thread
     */
	public static synchronized LoggerThread getInstance()
	{
		if(instance == null)
		{
			instance = new LoggerThread();
		}
		return instance;
	}
	
	private LoggerThread()
	{
		// should be passed the name of the application running
		// or the ID of the SYS-D thread (to differentiate different node entries)
		
        this.logRequestQueue = new LinkedBlockingQueue<EventRecord>();
		
        // TODO: retrieve the last log number 
        int logSequenceNumber = 0;
			
        try 
		{
			out = new BufferedWriter(new FileWriter("log" + nodeID + "_" + logSequenceNumber + ".txt", true));
			// TODO: handle when file reaches max capacity; start writing to a new log file
			// file name should be unique on each node, can be composed of the node ID and
			// a counter value
			this.start();
		} 
		catch (IOException e) 
		{
			e.printStackTrace();
		} 
	}
	
	 /** 
     * Runs the Logger thread
     */
    public void run()
    {
    	try 
    	{
    		while(true) 
    		{ 
    			this.processLogRecord(this.logRequestQueue.take());
    		}
    	} 
    	catch (InterruptedException e) 
    	{ 
    		e.printStackTrace();
    	}
    	
		// close the stream
    	try 
    	{
			out.close();
		} 
    	catch (IOException e) 
    	{
			e.printStackTrace();
		} 
    }

    /**
     * Add a new log record to the request queue
     * Returns true if the request was successfully submitted
     */
    public boolean submitLogRecord(EventRecord rec)
    {
    	// use put() instead?
    	return this.logRequestQueue.add(rec);
    }
    
    /**
     * Process a log record from the queue
     */
    private void processLogRecord(EventRecord rec)
    {
    	try 
    	{ 
    		//out.write(XmlUtils.format(rec, FormatType.STANDARD)); 
    		out.newLine();
    		out.flush(); // for now, flush after each line
    	}
    	catch (IOException e) 
    	{ 
    		e.printStackTrace();
    	} 
    }
}
