package edu.mit.csail.aeolus.auditing;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import edu.mit.csail.aeolus.utilities.HelperUtils;
 
/**
 * This is class provides {@link java.sql.Array} interface for PostgreSQL <code>int4</code> array.
 *
 * @author Valentine Gogichashvili
 * http://tech.valgog.com/2009/02/passing-arrays-to-postgresql-database.html
 */
 
public class PostgreSQLInt4Array implements java.sql.Array {
 
    private final Integer[] array;
    
    public PostgreSQLInt4Array(Integer[] array) {
        this.array = array;
    }
 
    public String toString() {
    	return toPGSQLString(array);
    }
  
    public static String toPGSQLString(Integer[] array){
    	if(array == null || array.length == 0)
    		return "{}";
    	List<Integer> l = Arrays.asList(array);
    	String innards = HelperUtils.joinString(", ", l);
    	
    	return "{" + innards + "}";    	
    }
    
    public Object getArray() throws SQLException {
        return array == null ? null : Arrays.copyOf(array, array.length);
    }
 
    public Object getArray(Map<String, Class<?>> map) throws SQLException {
        return getArray();
    }
 
    public Object getArray(long index, int count) throws SQLException {
        return array == null ? null : Arrays.copyOfRange(array, (int)index, (int)index + count );
    }
 
    public Object getArray(long index, int count, Map<String, Class<?>> map) throws SQLException {
        return getArray(index, count);
    }
 
    public int getBaseType() throws SQLException {
        return java.sql.Types.INTEGER;
    }
 
    public String getBaseTypeName() throws SQLException {
        return "int4";
    }
 
    public ResultSet getResultSet() throws SQLException {
        throw new RuntimeException();
    }
 
    public ResultSet getResultSet(Map<String, Class<?>> map) throws SQLException {
        throw new RuntimeException();
    }
 
    public ResultSet getResultSet(long index, int count) throws SQLException {
        throw new RuntimeException();
    }
 
    public ResultSet getResultSet(long index, int count, Map<String, Class<?>> map) throws SQLException {
        throw new RuntimeException();
    }
 
    public void free() throws SQLException {
    }
 
}