package edu.mit.csail.aeolus.auditing.graphdb;

import java.util.LinkedList;
import java.util.List;

import edu.mit.csail.aeolus.auditing.AuthorityProof;
import edu.mit.csail.aeolus.auditing.CacheLineID;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.ASActFor;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.ASCreatePrincipal;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.ASCreateSubtag;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.ASCreateTag;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.ASDelegate;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.ASRevokeActFor;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.ASRevokeDelegate;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.AddIntegrityEvent;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.AuthorityServerEvent;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.DeclassifyEvent;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.EventVisitor;
import edu.mit.csail.aeolus.authority.api.ClosureTableEntry;
import edu.mit.csail.aeolus.authority.api.CompoundTagTableEntry;
import edu.mit.csail.aeolus.authority.api.GrantTableEntry;
import edu.mit.csail.aeolus.authority.api.LinkTableEntry;
import edu.mit.csail.aeolus.authority.api.PrincipalTableEntry;
import edu.mit.csail.aeolus.authority.api.TaintTableEntry;
import edu.mit.csail.aeolus.authority.api.LinkTableEntry.LinkType;
import edu.mit.csail.aeolus.infoflow.PlatformInstanceID;
import edu.mit.csail.aeolus.infoflow.PrincipalID;
import edu.mit.csail.aeolus.infoflow.TagID;

/**
 * This class maintains a running version of the authority state as the 
 * event graph is processed.
 * @author blanks
 *
 */
public class AnalyseAuthority extends EventVisitor<Void>{
	List<TaintTableEntry> tagEntries;
	List<PrincipalTableEntry> principalEntries;
	List<GrantTableEntry> delegateEntries;
	List<LinkTableEntry> actForEntries;
	List<ClosureTableEntry> closureEntries;
	List<CompoundTagTableEntry> compoundEntries;

	PrincipalID systemPrincipalID = new PrincipalID(1);;
	SQLEventStore indexer = SQLEventStore.get();

	public Void processAuthServerEvent(AuthorityServerEvent e){
		init();
		return null;
	}

	public Void processEvent(DeclassifyEvent d){
		CacheLineID cache = d.getCacheLineID();
		PlatformInstanceID vnID = d.getVirtualNodeID();
		if(cache.proof != null){
			indexer.setCacheLine(cache, vnID, cache.proof);
		}else{
			AuthorityProof proof = indexer.getCacheLineProof(cache, vnID);
			d.setProof(proof);
		}
		return null;
	}

	public Void processEvent(AddIntegrityEvent e){
		CacheLineID cache = e.getCacheLineID();
		PlatformInstanceID vnID = e.getVirtualNodeID();
		if(cache.proof != null){
			indexer.setCacheLine(cache, vnID, cache.proof);
		}else{
			AuthorityProof proof = indexer.getCacheLineProof(cache, vnID);
			e.setProof(proof);
		}
		return null;
	}

	public Void processEvent(ASCreatePrincipal e){
		PrincipalID created = e.getPrincipalCreated();
		PrincipalID creator = e.getPrincipalCreator();
		principalEntries.add(new PrincipalTableEntry(created, creator, 0, 0));
		actForEntries.add(new LinkTableEntry(creator, created, LinkType.L_OWNS));
		return null;
	}

	public Void processEvent(ASCreateTag e){
		TagID created = e.getCreatedTag();
		PrincipalID creator = e.getPrincipalCreator();
		tagEntries.add(new TaintTableEntry(creator, created, 0));

		GrantTableEntry entryGrant = new GrantTableEntry(systemPrincipalID, creator, created, 0);
		delegateEntries.add(entryGrant);
		return null;
	}
	public Void processEvent(ASCreateSubtag e){
		TagID child = e.getCreatedTag();
		TagID parent = e.getParentTag();
		CompoundTagTableEntry entry = 
			new CompoundTagTableEntry(child.getValue(), parent.getValue(), 0);
		compoundEntries.add(entry);
		return null;
	}
	public Void processEvent(ASDelegate e){
		PrincipalID grantor = e.getGrantor();
		PrincipalID grantee = e.getGrantee();
		TagID tid = e.getDelegatedTag();
		GrantTableEntry entryGrant = new GrantTableEntry(grantor, grantee, tid, 0);
		delegateEntries.add(entryGrant);
		return null;
	}
	public Void processEvent(ASActFor e){
		PrincipalID actor = e.getActor();
		PrincipalID actee = e.getActee();

		actForEntries.add(new LinkTableEntry(actor, actee, LinkType.L_ACTSFOR));
		return null;
	}
	public Void processEvent(ASRevokeDelegate e){
		return null;
	}
	public Void processEvent(ASRevokeActFor e){
		return null;
	}


	private void init(){
		tagEntries = new LinkedList<TaintTableEntry>();
		principalEntries = new LinkedList<PrincipalTableEntry>();
		delegateEntries = new LinkedList<GrantTableEntry>();
		actForEntries = new LinkedList<LinkTableEntry>();
		closureEntries = new LinkedList<ClosureTableEntry>();
		compoundEntries = new LinkedList<CompoundTagTableEntry>();
	}
}
