package edu.mit.csail.aeolus.auditing.graphdb;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import edu.mit.csail.aeolus.auditing.graphdb.wrappers.AbstractEvent;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.AddIntegrityEvent;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.AddSecrecyEvent;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.CallReply;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.CallClosureReply;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.DeclassifyEvent;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.LabelEvent;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.LaunchVirtualNode;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.PlatformUserEvent;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.ProcessEvent;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.RemoveIntegrityEvent;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.SendRPCReply;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.UserThreadLaunch;
import edu.mit.csail.aeolus.infoflow.Label;
import edu.mit.csail.aeolus.infoflow.PrincipalID;
import edu.mit.csail.aeolus.infoflow.TagID;

/**
 * The class handles analyzing events to reconstruct user thread label 
 * information. Events that modify the label or start new threads need
 * to be treated by this class.
 *
 * @author blanks
 */
public class AnalyseLabel {

	// TODO: RPC events are weird when it comes to labels.
	public void visit(PlatformUserEvent event){
		// we need to add process information to the event.
		AbstractEvent parent = event.getProcessParent();
		
		if(parent == null || ! (parent instanceof PlatformUserEvent)){
			if(event instanceof UserThreadLaunch){
				// Then this is a start node, so we need to 
				// add the default process information
				LaunchVirtualNode launch = 
					(LaunchVirtualNode) event.getCommunicationParent();
				PrincipalID pi = launch.getPrincipal();
				ArrayList<PrincipalID> stack = 
					new ArrayList<PrincipalID>(Collections.singleton(pi));
				event.setPrincipalStack(stack);
				event.setProcessIntegrity(new Label());
				event.setProcessSecrecy(new Label());
			}else{
				System.out.println("..bad..");
			}
		}else{
			
			PlatformUserEvent uParent = (PlatformUserEvent) parent;
			ArrayList<PrincipalID> parentStack = null;
			if (parent.getPrincipalStack() != null)
			    parentStack = (ArrayList<PrincipalID>)parent.getPrincipalStack().clone();

			Label secrecy = new Label(uParent.getProcessSecrecy());
			Label integrity = new Label(uParent.getProcessIntegrity());
			if(secrecy == null){
				System.out.println("null parent secrecy >> " + parent.getOp());
				secrecy = new Label();
			}
			if(integrity == null)
				integrity = new Label();
			if(parentStack == null){
				System.out.println("null parent stack >> " + parent.getOp());
				parentStack = new ArrayList<PrincipalID>();
			}
			
			// Principal Changes
			if(parent instanceof ProcessEvent){
				// handles CALL and FORK
				parentStack.add(((ProcessEvent) parent).getRaisedPrincipal());
			}
			if(event instanceof CallReply || event instanceof CallClosureReply){
				// pop one!
				parentStack.remove(parentStack.size() - 1);
			}

			// Label changes
			if(parent instanceof LabelEvent){
				if(parent instanceof DeclassifyEvent){
					List<TagID> tags = ((DeclassifyEvent) parent).getRemovedTags();
					secrecy.removeTags(tags);
				}else if(parent instanceof AddSecrecyEvent){
					List<TagID> tags = ((AddSecrecyEvent) parent).getAddedTags();
					secrecy.addTags(tags);
				}else if(parent instanceof RemoveIntegrityEvent){
					List<TagID> tags = ((RemoveIntegrityEvent) parent).getRemovedTags();
					integrity.removeTags(tags);						
				}else if(parent instanceof AddIntegrityEvent){
					List<TagID> tags = ((AddIntegrityEvent) parent).getAddedTags();
					integrity.addTags(tags);
				}
			}
			if(parent instanceof SendRPCReply){
				integrity = ((SendRPCReply)parent).getNewIntegrity();
				secrecy = ((SendRPCReply)parent).getNewSecrecy();
				
			}

			// shove it in the child.
			event.setPrincipalStack(parentStack);
			event.setProcessIntegrity(integrity);
			event.setProcessSecrecy(secrecy);
		}
	}
}
