package edu.mit.csail.aeolus.auditing.graphdb;

import java.io.Serializable;
import java.sql.Connection;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Iterator;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.postgresql.ds.PGConnectionPoolDataSource;

import edu.mit.csail.aeolus.api.SystemNodeID;
import edu.mit.csail.aeolus.auditing.AeolusApiConstants;
import edu.mit.csail.aeolus.auditing.AuthorityProof;
import edu.mit.csail.aeolus.auditing.CacheLineID;
import edu.mit.csail.aeolus.auditing.EventCounter;
import edu.mit.csail.aeolus.auditing.PostgreSQLInt4Array;
import edu.mit.csail.aeolus.auditing.graphdb.predicates.EventAttribute;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.ASActFor;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.ASCreatePrincipal;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.ASCreateSubtag;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.ASCreateTag;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.ASDelegate;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.ASRevokeActFor;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.ASRevokeDelegate;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.ASStart;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.AbstractEvent;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.ActForClient;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.AddIntegrityEvent;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.AddSecrecyEvent;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.CallClosure;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.CallClosureReply;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.CallEvent;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.CallReply;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.CloseFileStream;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.CreateDirectory;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.CreateFile;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.CreateNode;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.CreatePrincipalClient;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.CreateSubTagClient;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.CreateTagClient;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.DeclassifyEvent;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.DelegateClient;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.EventVisitor;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.ForkEvent;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.GetIntegrityLabel;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.GetRemoteRPCService;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.GetSecrecyLabel;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.LaunchVNRequest;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.LaunchVirtualNode;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.ListDirectory;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.LoadClass;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.OpenFileInputStream;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.OpenFileOutputStream;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.OtherEvent;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.ReadFile;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.ReceiveRPC;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.RegisterRPCService;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.RemoveDirectory;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.RemoveFile;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.RemoveIntegrityEvent;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.RevokeActForClient;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.RevokeDelegateClient;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.SendRPC;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.SendRPCReply;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.ShutdownVNRequest;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.ShutdownVirtualNode;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.UserEvent;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.UserThreadLaunch;
import edu.mit.csail.aeolus.auditing.graphdb.wrappers.WriteFile;
import edu.mit.csail.aeolus.authority.api.AuthorityConfiguration;
import edu.mit.csail.aeolus.infoflow.Label;
import edu.mit.csail.aeolus.infoflow.PlatformInstanceID;
import edu.mit.csail.aeolus.infoflow.PrincipalID;
import edu.mit.csail.aeolus.infoflow.TagID;
import edu.mit.csail.aeolus.runtime.RuntimeConfig;
import edu.mit.csail.aeolus.utilities.Pair;


/**
 * This class handles creating the tables and storing events in a PostgreSQL 
 * database. I know that it is insane that we use PGSQL here and MYSQL for the
 * authority server, but that's the way that it is for the moment.
 * 
 * This class works by creating a table that fits all of the event fields defined
 * in the EventAttribute enum. Events are stored using a private EventVisitor 
 * instance that maps from EventAttribute to the method of each AbstractEvent. If 
 * you add a new AbstractEvent type, you need to add a method to that visitor.
 * 
 * <h1>Guide to Setting up PostgreSQL</h1>
 * 
 * (1) Create the postgres user for aeolus (usually just your unix name)
 *   $ sudo -u postgres createuser --superuser YOUR_UNIX_USERNAME
 * (2) Create the database:
 * $ createdb AeolusAuthority
 * (3) Enable trusted access over TCP on localhost...
 * (!) this is really insecure if postgres accepts remote connections,
 * by default it doesn't, so don't change that.
 * Edit the file /etc/postgresql/8.4/main/pg_hba.conf (this is on Ubuntu):
 *  change the entry for 127.0.01 from "md5" type to "trust"
 * and restart postgresql (ubuntu: sudo /etc/init.d/postgresql restart)
 * 
 * @author blanks
 *
 */
public class SQLEventStore {

	private final boolean logging;
	// indices ... 
	private final String CACHE_LINE_IX = "CacheLineProofIx";
	private final String CACHE_LINE_ID = "CacheLineID";
	private final String VN_ID = "VirtualNodeID";
	private final String PROOF = "AuthorityProof";
	
	
	private final String CREATE_CACHE_LINE_IX =
		String.format("CREATE TABLE %s (%s BIGINT , %s INT , %s TEXT, PRIMARY KEY (%s, %s))",
				CACHE_LINE_IX, CACHE_LINE_ID, VN_ID, PROOF, CACHE_LINE_ID, VN_ID);
	private final String INSERT_CACHE_LINE_IX =
		String.format("INSERT INTO %s (%s, %s, %s) VALUES (?,?,?)",
				CACHE_LINE_IX, CACHE_LINE_ID, VN_ID, PROOF);
	private final String LOOKUP_CACHE_LINE_IX =
		String.format("SELECT %s FROM %s WHERE %s = ? AND %s = ?",
				PROOF, CACHE_LINE_IX, CACHE_LINE_ID, VN_ID);

	private final String EVENT_STORE = "Events";

	private final String INSERT = String.format("INSERT INTO %s (%%s) VALUES (%%s)",
			EVENT_STORE);
	private final String CREATE_EVENT_STORE = 
		String.format("CREATE TABLE %s (%%s)", EVENT_STORE);
	private final String indexSQL =
		String.format("CREATE INDEX %%s ON %s (%%s);", EVENT_STORE);
	private final EventAttribute[] indexingAttrs 
		= {EventAttribute.EVENT_COUNTER, 
		   EventAttribute.FILENAME, 
		   EventAttribute.GRANTOR, 
		   EventAttribute.GRANTEE,
		   EventAttribute.OP_NAME};

	private final String INSERT_FULL_SKEL 
		= String.format("INSERT INTO %s (%%s) VALUES (%%s)",
						EVENT_STORE);
	private String INSERT_FULL = null;

	private final boolean quitWhenDone = RuntimeConfig.get().quitWhenEventTableDone();
	private final boolean timing = RuntimeConfig.get().isTimingEventTable();

	private PGConnectionPoolDataSource cpds;
	
	private SQLEventStore(boolean logging){
		this.logging = logging;

		if(timing){
			System.out.println("Timing!");
		}
		if(quitWhenDone){
			System.out.println("Quiting when done!");
		}

		StringBuffer insertCols = new StringBuffer();
		StringBuffer insertQs = new StringBuffer();
		for(EventAttribute attr : EventAttribute.values()){
			insertCols.append(attr.name() + ", ");
			insertQs.append("?, ");
		}
		INSERT_FULL = String.format(INSERT_FULL_SKEL,
									insertCols.substring(0, insertCols.length() - 2),
									insertQs.substring(0, insertQs.length() - 2));


		System.out.println("Logging to DB? " + logging);
		if(logging){
			cpds = new PGConnectionPoolDataSource();
			Logger.getLogger("").setLevel(Level.OFF);
			cpds.setDatabaseName(AuthorityConfiguration.DB_Name);
			cpds.setPortNumber(5432);
			cpds.setServerName("localhost");
			cpds.setDefaultAutoCommit(true);
			clearTables();
			createIndices();
			startEventWatcher();
		}
	}

	private String getCreateEventTableString(){
		StringBuffer colDefs = new StringBuffer();//"DB_ID BIGINT AUTO_INCREMENT PRIMARY KEY, ");
		for(EventAttribute attr : EventAttribute.values()){
			colDefs.append(attr.name() + " " + attr.type + ", ");
		}
		
		return String.format(CREATE_EVENT_STORE, 
				colDefs.substring(0, colDefs.length() - 2));
	}
	
	public void clearTables(){
		Connection conn = getConnection();
		try {
			Statement s = conn.createStatement();
			String dropStmt = "DROP TABLE IF EXISTS " + CACHE_LINE_IX;
			s.execute(dropStmt);
			s = conn.createStatement();
			dropStmt = "DROP TABLE IF EXISTS " + EVENT_STORE;
			s.execute(dropStmt);
			conn.close();
		} catch (SQLException e) {
			e.printStackTrace();
			System.out.println("Indexer failed to clear");
		}		
	}
	
	public String getEventTableIndexString(EventAttribute attr){
		return String.format(indexSQL, attr.name() + "_IX", attr.name());
	}

	public void createIndices(){
		Connection c = getConnection();
		try{
			Statement s = c.createStatement();
			s.execute(CREATE_CACHE_LINE_IX);
			s = c.createStatement();
			s.execute(getCreateEventTableString());
			if(RuntimeConfig.get().indexEventTable()){
				for(EventAttribute attr : indexingAttrs){
					s = c.createStatement();
					s.execute(getEventTableIndexString(attr));
				}
			}
			c.close();
		}catch (Exception ex){
			ex.printStackTrace();
			System.out.println("Failed to create Event store tables.");
		}	
	}

	private Connection getConnection(){
		try {
			return cpds.getConnection();
		} catch (SQLException e) {
			e.printStackTrace();
			return null;
		}
	}
	
    private static String joinString(Object delim, List<?> input){
        if (input == null) {
        	return "";
        }

        if (input.size() == 0) {
        	return "";
        }

        StringBuilder sb = new StringBuilder();
        
        // append delimiter to all but the first integer 
        boolean first = true;
        for (Object i : input){
        	if(!first)
            	sb.append(delim.toString());
            sb.append(i.toString());
            first = false;
        }
        return sb.toString();
    }
    
    EventWriter myWriter = new EventWriter();
    private BlockingQueue<AbstractEvent> eventsToAdd = new LinkedBlockingQueue<AbstractEvent>();
    
    public void insertEvent(AbstractEvent e){
		if(!logging){
			return;
		}
    	try {
			eventsToAdd.offer(e, 50, TimeUnit.MILLISECONDS );
		} catch (InterruptedException ex) {
			ex.printStackTrace();
		}
    }
    private Timer myTimer = new Timer();
    public void startEventWatcher(){
    	myTimer.schedule(new WatchTask(), 5000);
    }

	private boolean startedPutting = false;
	private long totalTime = 0;
	private long totalCount = 0;

    private class WatchTask extends TimerTask{
		@Override
		public void run() {
			if(eventsToAdd.isEmpty()){
				if(quitWhenDone && startedPutting){
					System.out.println("Finished PGSQL storage.");
					if(timing){
						double timeSecs = totalTime / 1000.0;
						double throughput = totalCount / timeSecs;
						System.out.println("THROUGHPUT: " + throughput);
					}
					System.exit(0);	
				}else{		
					myTimer.schedule(new WatchTask(), 5000);					
					return;
				}
			}

			startedPutting = true;
			ArrayList<AbstractEvent> buffer = new ArrayList<AbstractEvent>();
			eventsToAdd.drainTo(buffer);
			Connection c = getConnection();
			// Partition by label!
			long timeStart = System.currentTimeMillis();
			if(RuntimeConfig.get().isPuttingByLabel()){		
				System.out.println("partioning...");
				HashMap<Label, ArrayList<AbstractEvent>> partitionedBuffer
					= new HashMap<Label, ArrayList<AbstractEvent>>();
				for(AbstractEvent e : buffer){
					Label l = e.getProcessSecrecy();
					if(partitionedBuffer.containsKey(l)){
						partitionedBuffer.get(l).add(e);
					}else{
						ArrayList<AbstractEvent> k = new ArrayList<AbstractEvent>();
						k.add(e);
						partitionedBuffer.put(l, k);
					}
				}
				for(Label l : partitionedBuffer.keySet()){
				    System.out.println("putting..." + l);
					try {
						putAll(partitionedBuffer.get(l), l, c);
					} catch (SQLException e1) {
						System.out.println("Timer dump problems!");
						e1.printStackTrace();
						System.exit(-1);					
					}
				}
			}else{
				for(AbstractEvent e : buffer){
					try {
						put(e, c);
					} catch (SQLException e1) {
						System.out.println("Timer dump problems!");
						e1.printStackTrace();
						System.exit(-1);
					}
				}
				try {
					c.close();
				} catch (SQLException e1) {
					e1.printStackTrace();
				}
			}
			long timeEnd = System.currentTimeMillis();
			totalTime += timeEnd - timeStart;
			totalCount += buffer.size();
			myTimer.schedule(new WatchTask(), 5000);				
		}
	};
    
	private int putCount = 0;
    private void putAll(List<AbstractEvent> events, 
						Label secrecy, Connection c) throws SQLException{
		c.setAutoCommit(false);
		
		java.util.Date date = new java.util.Date();
		try{
			/* THIS IS CODE FOR DAVID'S PGSQL EXTENSION.
			
			PreparedStatement setLabel = c.prepareStatement("SELECT setlabel(?);");
			setLabel.setArray(1, 
							  new PostgreSQLInt4Array(secrecy.getTagValues()));
			setLabel.execute();
			*/

			PreparedStatement stmt = c.prepareStatement(INSERT_FULL);
			for(AbstractEvent e : events){

				Pair<List<EventAttribute>, List<? extends Object>> aux 
					= e.accept(myWriter);
    	
				List<EventAttribute> cols = new ArrayList<EventAttribute>(aux.first);
				List<Object> vals = new ArrayList<Object>(aux.second);

				populateFields(e, cols, vals, date);
				Iterator<EventAttribute> colIter = cols.iterator();
				Iterator<Object> valIter = vals.iterator();
				HashMap<EventAttribute, Object> map = new HashMap<EventAttribute, Object>();
				while(valIter.hasNext()){
					Object curVal = valIter.next();
					EventAttribute curCol = colIter.next();
					map.put(curCol, curVal);
				}
				int index = 1;
				for(EventAttribute attr : EventAttribute.values()){
					Object val = map.get(attr);

					if(val == null){
						stmt.setObject(index, null);
					}else if(val instanceof EventCounter){
						stmt.setLong(index, ((EventCounter)val).identifier);
					}else if(val instanceof Integer){
						stmt.setInt(index, (Integer) val);
					}else if(val instanceof Long){
						stmt.setLong(index, (Long) val);
					}else if(val instanceof TagID){
						stmt.setInt(index, ((TagID)val).getValue());
					}else if(val instanceof PrincipalID){
						stmt.setInt(index, ((PrincipalID)val).getValue());
					}else if(val instanceof PlatformInstanceID){
						stmt.setInt(index, ((PlatformInstanceID)val).getValue());
					}else if(val instanceof SystemNodeID){
						stmt.setInt(index, ((SystemNodeID)val).getValue());
					}else if(val instanceof Timestamp){
						stmt.setTimestamp(index, (Timestamp) val);
					}else if(val instanceof Label){
						stmt.setArray(index, 
									  new PostgreSQLInt4Array(((Label) val).getTagValues()));
					}else{
						stmt.setString(index, val.toString());
					}

					index++;
				}
				stmt.execute();
				stmt.clearParameters();
			}

			c.commit();
		}catch(SQLException ex){
			c.rollback();
			throw ex;
		}finally{
			c.setAutoCommit(true);
		}
	}

    private void populateFields(AbstractEvent e, List<EventAttribute> cols, List<Object> vals, java.util.Date date)
    {

	cols.add(EventAttribute.EVENT_COUNTER);
	vals.add(e.getEventCounter());
    	
    	cols.add(EventAttribute.OP_NAME);
    	vals.add(e.getOp());
    	
    	cols.add(EventAttribute.TIMESTAMP);
    	vals.add(new Timestamp(e.dispatchTime));
    	
    	cols.add(EventAttribute.SECRECY);
    	vals.add(e.getProcessSecrecy());
    	cols.add(EventAttribute.INTEGRITY);
    	vals.add(e.getProcessIntegrity());

	cols.add(EventAttribute.METHOD_NAME);
	vals.add(e.getMethodName());

	try {
	    vals.add(e.getVirtualNodeID());
	    cols.add(EventAttribute.VIRTUAL_NODE_ID);
	} catch (NumberFormatException ex) {
	    /* XXX getVirtualNodeID() does something horrible and buggy,
	     * which causes it to always throw an exception when called
	     * on AS events.  It's not clear what reasonable answer it
	     * ought to even return, so we kludge around the problem instead.
	     * (How can we even *tell* if we're looking at an AS event?)
	     */
	}

	ArrayList<PrincipalID> pidStack = e.getPrincipalStack();
	if (pidStack != null && !pidStack.isEmpty()) {
	    cols.add(EventAttribute.RUNNING_PRINCIPAL);
	    vals.add(pidStack.get(pidStack.size() - 1));
	}	

	if(cols.size() != vals.size()){
	    throw new RuntimeException(AeolusApiConstants.getMethodName(e.getOp()));
	}

    }

	private void put(AbstractEvent e, Connection c) throws SQLException{
    	if(putCount++ % 5000 == 0){
//    		System.out.println("Put " + (putCount - 1) + " events in SQL store.");
    	}
    	Pair<List<EventAttribute>, List<? extends Object>> aux = e.accept(myWriter);
    	
    	List<EventAttribute> cols = new ArrayList<EventAttribute>(aux.first);
    	List<Object> vals = new ArrayList<Object>(aux.second);

	populateFields(e, cols, vals, new java.util.Date());
    	
    	String qs = "";
		for(int i = 0;i < cols.size();  i++){
			if(i != 0)
				qs += ", ";
			qs += "?";
		}

		if(cols.size() != vals.size()){
			throw new RuntimeException(AeolusApiConstants.getMethodName(e.getOp()));
		}

//		PGConnection castCon = (PGConnection) c;
//		castCon.addDataType("SETLABEL", MyVoid.class);
		
		c.setAutoCommit(false);

		String sql = String.format(INSERT, joinString(", ", cols), qs);

		try{
			PreparedStatement setLabel = c.prepareStatement("SELECT setlabel(?);");
			setLabel.setArray(1, new PostgreSQLInt4Array(e.getProcessSecrecy().getTagValues()));
//			setLabel.execute("SELECT setlabel(?);" 
//					+ PostgreSQLInt4Array.toPGSQLString()
//					+ ";");
			setLabel.execute();
			
			PreparedStatement stmt = c.prepareStatement(sql);
			int index = 1;
			for(Object val : vals){
				if(val == null){
					stmt.setObject(index, null);
				}else if(val instanceof EventCounter){
					stmt.setLong(index, ((EventCounter)val).identifier);
				}else if(val instanceof Integer){
					stmt.setInt(index, (Integer) val);
				}else if(val instanceof Long){
					stmt.setLong(index, (Long) val);
				}else if(val instanceof TagID){
					stmt.setInt(index, ((TagID)val).getValue());
				}else if(val instanceof PrincipalID){
					stmt.setInt(index, ((PrincipalID)val).getValue());
				}else if(val instanceof PlatformInstanceID){
					stmt.setInt(index, ((PlatformInstanceID)val).getValue());
				}else if(val instanceof SystemNodeID){
					stmt.setInt(index, ((SystemNodeID)val).getValue());
				}else if(val instanceof Timestamp){
					stmt.setTimestamp(index, (Timestamp) val);
				}else if(val instanceof Label){
					stmt.setArray(index, new PostgreSQLInt4Array(((Label) val).getTagValues()));
				}else{
					stmt.setString(index, val.toString());
				}
				index++;
			}
			stmt.execute();

			c.commit();
		}catch(SQLException ex){
			c.rollback();
			throw ex;
		}finally{
			c.setAutoCommit(true);
		}
    }

	public void setCacheLine(CacheLineID cache, PlatformInstanceID vnID, 
			AuthorityProof proof){
		if(!logging){
			return;
		}

		try{
			Connection c = getConnection();
			PreparedStatement s = c.prepareStatement(INSERT_CACHE_LINE_IX);
			s.setLong(1, cache.id);
			s.setInt(2, vnID.getValue());
			s.setString(3, proof.toString());
			s.execute();
			c.close();
		}catch (Exception ex){
			System.out.println(ex);
			System.out.println("Failed to index cache line");
		}
	}

	public AuthorityProof getCacheLineProof(CacheLineID cache, 
			PlatformInstanceID vnID) {
		if(!logging){
			return null;
		}
		try{
			Connection c = getConnection();
			PreparedStatement s = c.prepareStatement(LOOKUP_CACHE_LINE_IX);
			s.setLong(1, cache.id);
			s.setInt(2, vnID.getValue());
			ResultSet rs = s.executeQuery();
			AuthorityProof r = null;
			while(rs.next()){
				String fromStr = rs.getString(1);
				r = new AuthorityProof(fromStr);
				break;
			}
			c.close();
			return r;
		}catch(Exception ex){
			return null;
		}
	}

	public static SQLEventStore get(){
		return get(true);
	}

	static SQLEventStore instance = null;
	public static synchronized SQLEventStore get(boolean toStore) {
		if(instance == null)
			instance = new SQLEventStore(toStore);
		return instance;
	}
	
	private static final Pair<List<EventAttribute>, List<? extends Object>> EMPTY_AUX_INFO 
	= new Pair<List<EventAttribute>, List<? extends Object>>(
			Arrays.asList(new EventAttribute[0]),
			Collections.emptyList());
	
	/**
	 * This visitor handles most of the work of writing specific events to the
	 * SQL database. It returns two lists, the first is the EventAttributes used
	 * to store the event and the second is the corresponding values to store.
	 * @author blanks
	 *
	 */
	private static class EventWriter extends 
	EventVisitor<Pair<List<EventAttribute>, List<? extends Object>>>{

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(ActForClient e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
				Arrays.asList(EventAttribute.GRANTOR, EventAttribute.GRANTEE),
				Arrays.asList(e.getActedFor(),e.getActor())
			);
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(AddIntegrityEvent e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.TAGS_MODIFIED, EventAttribute.AUTH_PROOF),
					Arrays.asList(e.getAddedTags(),e.getProof())
				);
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(AddSecrecyEvent e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.TAGS_MODIFIED),
					Collections.singletonList(e.getAddedTags())
				);
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(ASActFor e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.GRANTOR, EventAttribute.GRANTEE),
					Arrays.asList(e.getActee(), e.getActor())
				);
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(ASCreatePrincipal e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.PRINCIPAL_CREATED, EventAttribute.PRINCIPAL_CREATOR),
					Arrays.asList(e.getPrincipalCreated(), e.getPrincipalCreator())
				);
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(ASCreateSubtag e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.TAG_PARENT, EventAttribute.TAG_CREATED),
					Arrays.asList(e.getParentTag(), e.getCreatedTag())
				);		
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(ASCreateTag e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.TAG_CREATED, EventAttribute.PRINCIPAL_CREATOR),
					Arrays.asList(e.getCreatedTag(), e.getPrincipalCreator())
			);
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(ASDelegate e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.GRANTEE, EventAttribute.GRANTOR, 
							EventAttribute.TAG_DELEGATED),
					Arrays.asList(e.getGrantee(), e.getGrantor(), e.getDelegatedTag())
			);
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(ASRevokeActFor e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.GRANTOR, EventAttribute.GRANTEE, EventAttribute.REVOKER),
					Arrays.asList(e.getActee(), e.getActor(), e.getRevoker())
			);
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(ASRevokeDelegate e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.GRANTOR, EventAttribute.GRANTEE, 
							EventAttribute.TAG_DELEGATED, EventAttribute.REVOKER),
					Arrays.asList(e.getGrantor(), e.getGrantee(), e.getTag(), e.getRevoker())
			);
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(ASStart asStart) {
			return EMPTY_AUX_INFO;
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(CallClosure e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Collections.EMPTY_LIST,
					Collections.EMPTY_LIST
			);
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(CallClosureReply e) {
		    return EMPTY_AUX_INFO;
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(CallEvent e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.SWITCHED_PRINCIPAL),
					Collections.singletonList(e.getRaisedPrincipal())
			);
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(CallReply e) {
			return EMPTY_AUX_INFO;
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(CreateDirectory e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.FILENAME, EventAttribute.HOSTNAME, 
							EventAttribute.OBJ_INTEGRITY, EventAttribute.OBJ_SECRECY),
					Arrays.asList(e.getFilePath(), e.getHostName(), e.getIntegrity(), e.getSecrecy())
			);
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(CreateFile e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.FILENAME, EventAttribute.HOSTNAME, 
							EventAttribute.OBJ_INTEGRITY, EventAttribute.OBJ_SECRECY),
					Arrays.asList(e.getFilePath(), e.getHostName(), e.getIntegrity(), e.getSecrecy())
			);
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(CreateNode e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.HOSTNAME, EventAttribute.ROOT_PRINCIPAL),
					Arrays.asList(e.getHostname(), e.getRootPrincipal())
			);
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(CreatePrincipalClient e) {
			return EMPTY_AUX_INFO;
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(CreateSubTagClient e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.TAG_PARENT),
					Collections.singletonList(e.getParent())
			);
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(CreateTagClient e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.TAG_CREATED),
					Collections.singletonList(e.getCreatedTag())
			);
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(DeclassifyEvent e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.TAGS_MODIFIED, EventAttribute.AUTH_PROOF),
					Arrays.asList(e.getRemovedTags(), e.getProof())
			);
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(DelegateClient e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.GRANTOR, EventAttribute.GRANTEE, 
							EventAttribute.TAG_DELEGATED),
					Arrays.asList(e.getGrantor(), e.getGrantee(), e.getTag()));
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(ForkEvent e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.SWITCHED_PRINCIPAL),
					Collections.singletonList(e.getRaisedPrincipal())
			);
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(GetRemoteRPCService e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.SERVICE_CLASS_NAME, EventAttribute.SERVICE_NAME,
							EventAttribute.HOSTNAME),
					Arrays.asList(e.getCanonicalNameServiceClass(), e.getServiceName(), 
							e.getRemoteHostName()));
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(LaunchVirtualNode e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.APP_ARG, EventAttribute.APP_NAME, 
							EventAttribute.VIRTUAL_NODE_ID, EventAttribute.LAUNCHED_PRINCIPAL),
					Arrays.asList(e.getAppArg(), e.getAppName(), e.getVirtualNodeID(), e.getPrincipal()));
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(LaunchVNRequest e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.APP_ARG, EventAttribute.APP_NAME, 
							EventAttribute.VIRTUAL_NODE_ID, EventAttribute.LAUNCHED_PRINCIPAL),
					Arrays.asList(e.getAppArg(), e.getAppName(), e.getVirtualNodeID(), e.getPrincipal()));			
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(ListDirectory e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.HOSTNAME, EventAttribute.FILENAME, EventAttribute.FS_START_TLM, EventAttribute.FS_END_TLM),
					Arrays.asList(e.getHostName(), e.getFilePath(), (Serializable)e.getFSStartTLM(), (Serializable)e.getFSEndTLM()));
		}
		
		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(GetSecrecyLabel e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.HOSTNAME, EventAttribute.FILENAME, EventAttribute.FS_START_TLM, EventAttribute.FS_END_TLM),
					Arrays.asList(e.getHostName(), e.getFilePath(), (Serializable)e.getFSStartTLM(), (Serializable)e.getFSEndTLM()));
		}
		
		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(GetIntegrityLabel e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.HOSTNAME, EventAttribute.FILENAME, EventAttribute.FS_START_TLM, EventAttribute.FS_END_TLM),
					Arrays.asList(e.getHostName(), e.getFilePath(), (Serializable)e.getFSStartTLM(), (Serializable)e.getFSEndTLM()));
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(LoadClass loadClass) {
			return EMPTY_AUX_INFO;
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(OpenFileInputStream e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.HOSTNAME, EventAttribute.FILENAME, EventAttribute.FS_START_TLM),
					Arrays.asList(e.getHostName(), e.getFilePath(), (Serializable)e.getFSStartTLM()));
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(OpenFileOutputStream e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.HOSTNAME, EventAttribute.FILENAME, EventAttribute.FS_START_TLM),
					Arrays.asList(e.getHostName(), e.getFilePath(), (Serializable)e.getFSStartTLM()));
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(OtherEvent e) {
			return EMPTY_AUX_INFO;
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(ReadFile e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.HOSTNAME, EventAttribute.FILENAME, EventAttribute.FS_START_TLM, EventAttribute.FS_END_TLM),
					Arrays.asList(e.getHostName(), e.getFilePath(), (Serializable)e.getFSStartTLM(), (Serializable)e.getFSEndTLM()));
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(ReceiveRPC e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.HOSTNAME, EventAttribute.FILENAME, EventAttribute.METHOD_NAME),
					Arrays.asList(e.getCallerPrincipal(), e.getServiceName(), e.getMethodName())); //e.getStringParams() not supported yet
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(RegisterRPCService e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.SERVICE_NAME, EventAttribute.SERVICE_CLASS_NAME),
					Arrays.asList(e.getServiceName(), e.getCanonicalNameServiceClass()));
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(RemoveDirectory e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.HOSTNAME, EventAttribute.FILENAME),
					Arrays.asList(e.getHostName(), e.getFilePath()));
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(RemoveFile e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.HOSTNAME, EventAttribute.FILENAME),
					Arrays.asList(e.getHostName(), e.getFilePath()));
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(RemoveIntegrityEvent e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.TAGS_MODIFIED),
					Collections.singletonList(e.getRemovedTags()));
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(RevokeActForClient e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.GRANTEE, EventAttribute.GRANTOR),
					Arrays.asList(e.getRevokedActor(), e.getRevokedActedFor()));
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(RevokeDelegateClient e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.GRANTEE, EventAttribute.GRANTOR, 
							EventAttribute.TAG_DELEGATED),
					Arrays.asList(e.getGrantee(), e.getGrantor(), e.getTag()));
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(SendRPC e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.HOSTNAME, 
							EventAttribute.SERVICE_NAME, EventAttribute.METHOD_NAME),
					Arrays.asList(e.getRemoteHostName(), e.getServiceName(), e.getMethodName()));
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(
				UserEvent e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.APP_NAME, EventAttribute.APP_ARG),
					Arrays.asList(e.getAppName(), e.getAppArgs()));
		}
		
		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(SendRPCReply e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.OBJ_INTEGRITY, EventAttribute.OBJ_SECRECY),
					Arrays.asList(e.getNewIntegrity(), e.getNewSecrecy()));
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(ShutdownVirtualNode e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.VIRTUAL_NODE_ID),
					Arrays.asList(e.getVirtualNodeID()));
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(ShutdownVNRequest e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.HOSTNAME, EventAttribute.VIRTUAL_NODE_ID, EventAttribute.IS_IMMEDIATE),
					Arrays.asList(e.getHostName(), e.getVirtualNodeID(), e.getShutdownImmediately()));
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(UserThreadLaunch e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.SWITCHED_PRINCIPAL),
					Arrays.asList(e.getRaisedPrincipal()));
		}

		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(WriteFile e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.HOSTNAME, EventAttribute.FILENAME, EventAttribute.FS_START_TLM, EventAttribute.FS_END_TLM),
					Arrays.asList(e.getHostName(), e.getFilePath(), (Serializable)e.getFSStartTLM(), (Serializable)e.getFSEndTLM()));
		}
		
		@Override
		public Pair<List<EventAttribute>, List<? extends Object>> visit(CloseFileStream e) {
			return new Pair<List<EventAttribute>, List<? extends Object>>(
					Arrays.asList(EventAttribute.HOSTNAME, EventAttribute.FILENAME, EventAttribute.FS_END_TLM),
					Arrays.asList(e.getHostName(), (Serializable)e.getFilePath(), (Serializable)e.getFSEndTLM()));
		}
		
	}
}
