package edu.mit.csail.aeolus.auditing.graphdb.wrappers;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import edu.mit.csail.aeolus.auditing.EventCounter;
import edu.mit.csail.aeolus.auditing.EventID;
import edu.mit.csail.aeolus.auditing.EventRecord;
import edu.mit.csail.aeolus.infoflow.Label;
import edu.mit.csail.aeolus.infoflow.PlatformInstanceID;
import edu.mit.csail.aeolus.infoflow.PrincipalID;

public abstract class AbstractEvent implements Serializable {
	private static final long serialVersionUID = 1L;

//	private transient Node node;
	
	private Serializable[] paramsMemo = null;
	private Object returnValueMemo = null;
	private String methodName = null;

	private EventID eid = null;
	private ArrayList<PrincipalID> pStack = null;
	private int op = -1;

	private transient AbstractEventCache eventCache;
	private transient EventID commPred;
	private transient EventID procPred;
	private transient List<EventID> oPreds;
	
	public transient long dispatchTime;
	public transient long processTime;
	
	private EventCounter eventCounter = null;
	// Hmmmmmmmmmm .... 
	//   -- how to do this one ??
	private List<EventCounter> preds = new ArrayList<EventCounter>();

	private boolean isProcessed = false;
	
	public abstract <U> U accept(EventVisitor<U> v);

	public AbstractEvent(EventRecord er){
		this.eid = er.getEventId();
		this.op = er.getMethod();
		this.paramsMemo = er.getParamValues().toArray(new Serializable[0]);
		this.returnValueMemo = er.getReturnValue();
		this.oPreds = er.getOtherPreds();
		this.commPred = er.getCommunicationPred();
		this.procPred = er.getProcessPred();
		this.eventCache = AbstractEventCache.getInstance();
		this.methodName = er.getMethodName();
		this.dispatchTime = er.getTimestamp().getTime();
	}

	public AbstractEvent(int op, Serializable[] params) {
		this.eid = new EventID("test");
		this.op  = op;
		this.paramsMemo = params;
		this.oPreds = new ArrayList<EventID>();
		this.eventCounter = new EventCounter(new Random().nextInt());
		this.eventCache = AbstractEventCache.getInstance();
	}

	/**
	 * @sideEffect this will also fetch the EventCounter of the predecessors.
	 * so make sure that the predecessors have been set before this is
	 * called.
	 * @param eventCounter
	 */
	public void setEventCounter(EventCounter eventCounter){
		this.eventCounter = eventCounter;
		for(AbstractEvent ev : getAllPreds()){
			this.preds.add(ev.getEventCounter());
		}
	}
	public EventCounter getEventCounter(){
		return this.eventCounter;
	}
	
	public Label getProcessSecrecy(){
		return new Label();
	}
	public Label getProcessIntegrity(){
		return new Label();
	}

	public String getMethodName() {
		return methodName;
	}

	private EventID getEID(){
		if(this.eid == null){
		}
		return this.eid;	
	}

	// TODO: override this in auth-server events
	// and in filesystem manager events
	// or just move this shit to the subclass ?
	public PlatformInstanceID getVirtualNodeID(){
		EventID eid = getEID();
		String[] split = eid.toString().split("-");
		int vnID = new Integer(split[0]);
		return new PlatformInstanceID(vnID);
	}

	public long getUserDomainID(){
		EventID eid = getEID();
		String[] split = eid.toString().split("-");
		long userD = new Long(split[1]);
		return userD;	
	}

	public int getUserDomainGeneration(){
		EventID eid = getEID();
		String[] split = eid.toString().split("-");
		int userD = new Integer(split[2]);
		return userD;
	}

	/**
	 * @return List of principals comprising the current basis
	 * of the process. The list grows down, so that the
	 * current PrincipalID is head of the list.
	 * 
	 * null if the PrincipalStack has not been set
	 * yet on this node.
	 */
	public ArrayList<PrincipalID> getPrincipalStack(){
		return pStack;
//		Node my = getNode();
//		Object o = my.getProperty("PrincipalStack", null);
//		if(o == null)
//			return null;
//		pStack = (ArrayList<PrincipalID>) HelperUtils.deserializeFromB64String((String) o);
//		return pStack;
	}

	public int getOp(){
		return op;		
	}

	public void setPrincipalStack(ArrayList<PrincipalID> stack){
//		String b64Rep = HelperUtils.serializeToString(stack);
//		getNode().setProperty("PrincipalStack", b64Rep);
		pStack = stack;
	}
	
	public AbstractEvent getProcessParent(){
		return eventCache.getEvent(procPred);
	}

	public AbstractEvent getCommunicationParent() {
		return eventCache.getEvent(commPred);
	}

	protected Serializable[] getParams(){
		return paramsMemo;
	}

	protected Object getReturnValue(){
		return returnValueMemo;
	}

	public void disposeNode() {
	}

	public boolean isProcessed() {
		return isProcessed ;
	}
	public void markProcessed() {
		isProcessed = true;
	}

	public List<AbstractEvent> getAllPreds() {
		ArrayList<AbstractEvent> out = new ArrayList<AbstractEvent>(2+oPreds.size());
		if(this.procPred != null)
			out.add(getProcessParent());
		if(this.commPred != null)
			out.add(getCommunicationParent());
		for(EventID evO : oPreds){
			out.add(eventCache.getEvent(evO));
		}
		return out;
	}
}
