package edu.mit.csail.aeolus.auditing.graphdb.wrappers;

import java.util.HashMap;

import edu.mit.csail.aeolus.auditing.EventID;

/**
 * Singleton class that holds a mapping of EventID -> AbstractEvent objects.
 * Right now this is all fetched from memory. This is not sustainable behavior.
 * Ultimately, this should store some in memory, and fetch old events from the 
 * SQLEventStore.
 * @author blanks
 *
 */
public class AbstractEventCache {
	private static AbstractEventCache singleton;

	private HashMap<EventID, AbstractEvent> memoryCache = new HashMap<EventID, AbstractEvent>();
	
	private AbstractEventCache(){
	}
	
	public static AbstractEventCache getInstance(){
		if(singleton == null)
			singleton = new AbstractEventCache();
		return singleton;
	}
	
	public void addEvent(EventID eid, AbstractEvent ev){
		memoryCache.put(eid, ev);
	}
	
	/**
	 * @param eid EventID of the AbstractEvent to find
	 * @return the AbstractEvent if it has been stored in the cache 
	 * (it should be there if it has already been through the LP) or null otherwise.
	 */
	public AbstractEvent getEvent(EventID eid){
		return memoryCache.get(eid);
	}
}
