package edu.mit.csail.aeolus.auditing.graphdb.wrappers;

import edu.mit.csail.aeolus.auditing.AeolusApiConstants;
import edu.mit.csail.aeolus.auditing.EventRecord;

public class EventFactory {
	public static AbstractEvent getEvent(EventRecord n){
		int op;
		try{
			op = n.getMethod();
		}catch(Exception e){
			e.printStackTrace();
			return null;
		}
		System.out.println("event factory: " + op);
		switch(op){
		case AeolusApiConstants.AUTHORITY_ACT_FOR:
			return new ASActFor(n);
		case AeolusApiConstants.AUTHORITY_START:
			return new ASStart(n);
		case AeolusApiConstants.AUTHORITY_REVOKE_DELEGATE:
			return new ASRevokeDelegate(n);
		case AeolusApiConstants.AUTHORITY_REVOKE_ACT_FOR:
			return new ASRevokeActFor(n);
		case AeolusApiConstants.AUTHORITY_DELEGATE:
			return new ASDelegate(n);
		case AeolusApiConstants.AUTHORITY_CREATE_TAG:
			return new ASCreateTag(n);
		case AeolusApiConstants.AUTHORITY_CREATE_SUBTAG:
			return new ASCreateSubtag(n);
		case AeolusApiConstants.AUTHORITY_CREATE_PRINCIPAL:		
			return new ASCreatePrincipal(n);
		case AeolusApiConstants.LOAD_CLASS:
			return new LoadClass(n);
		case AeolusApiConstants.ACT_FOR:
			return new ActForClient(n);
		case AeolusApiConstants.ADD_SECRECY:
			return new AddSecrecyEvent(n);
		case AeolusApiConstants.CALL:
			return new CallEvent(n);
		case AeolusApiConstants.CALL_REPLY:
			return new CallReply(n);
		case AeolusApiConstants.CALL_CLOSURE:
			return new CallClosure(n);
		case AeolusApiConstants.CALL_CLOSURE_REPLY:
			return new CallClosureReply(n);
		case AeolusApiConstants.ENDORSE:
			return new AddIntegrityEvent(n);
		case AeolusApiConstants.CREATE_DIRECTORY:
			return new CreateDirectory(n);
		case AeolusApiConstants.CREATE_FILE:
			return new CreateFile(n);
		case AeolusApiConstants.CREATE_NODE:
			return new CreateNode(n);
		case AeolusApiConstants.CREATE_PRINCIPAL:
			return new CreatePrincipalClient(n);
		case AeolusApiConstants.CREATE_PRINCIPAL_NEWCORE:
			return new CreatePrincipalClient(n);
		case AeolusApiConstants.CREATE_TAG:
			return new CreateTagClient(n);
		case AeolusApiConstants.CREATE_SUBTAG:
			return new CreateSubTagClient(n);
		case AeolusApiConstants.DECLASSIFY:
			return new DeclassifyEvent(n);
		case AeolusApiConstants.DELEGATE:
			return new DelegateClient(n);
		case AeolusApiConstants.FORK:
			return new ForkEvent(n);
		case AeolusApiConstants.GET_INTEGRITY_LABEL:
			return new GetIntegrityLabel(n);
		case AeolusApiConstants.GET_SERVICE:
			return new GetRemoteRPCService(n);
		case AeolusApiConstants.GET_SECRECY_LABEL:
			return new GetSecrecyLabel(n);
		case AeolusApiConstants.PI_LAUNCH:
			return new LaunchVirtualNode(n);
		case AeolusApiConstants.PI_LAUNCH_REQUEST:
			return new LaunchVNRequest(n);
		case AeolusApiConstants.LIST_DIRECTORY:
			return new ListDirectory(n);
		case AeolusApiConstants.OPEN_WRITE_FILESTREAM:
			return new OpenFileOutputStream(n);
		case AeolusApiConstants.OPEN_READ_FILESTREAM:
			return new OpenFileInputStream(n);
		case AeolusApiConstants.CLOSE_FILESTREAM:
			return new CloseFileStream(n);
		case AeolusApiConstants.READ_FILE:
			return new ReadFile(n);
		case AeolusApiConstants.RECEIVE_RPC:
			return new ReceiveRPC(n);
		case AeolusApiConstants.REGISTER_SERVICE:
			return new RegisterRPCService(n);
		case AeolusApiConstants.REMOVE_DIRECTORY:
			return new RemoveDirectory(n);
		case AeolusApiConstants.DELETE_FILE:
			return new RemoveFile(n);
		case AeolusApiConstants.REVOKE_ACT_FOR:
			return new RevokeActForClient(n);
		case AeolusApiConstants.REVOKE_DELEGATE:
			return new RevokeDelegateClient(n);
		case AeolusApiConstants.SEND_RPC:
			return new SendRPC(n);
		case AeolusApiConstants.SEND_RPC_REPLY:
			return new SendRPCReply(n);
		case AeolusApiConstants.PI_SHUTDOWN:
			return new ShutdownVirtualNode(n);
		case AeolusApiConstants.PI_SHUTDOWN_REQUEST:
			return new ShutdownVNRequest(n);
		case AeolusApiConstants.USER_THREAD_LAUNCH:
			return new UserThreadLaunch(n);
		case AeolusApiConstants.WRITE_FILE:
			return new WriteFile(n);		
		case AeolusApiConstants.REMOVE_INTEGRITY:
			return new RemoveIntegrityEvent(n);
		case AeolusApiConstants.APPLICATION_LVL:
			return new UserEvent(n);
		default:
			return new OtherEvent(n);
		}
	}
}
