package edu.mit.csail.aeolus.authority.client;

import java.util.*;

import edu.mit.csail.aeolus.utilities.Logger;
import edu.mit.csail.aeolus.utilities.LoggerLevel;

public class AlertList {
    int nextUniqueId;
    Hashtable<Integer, AlertListEntry> alertInfo;

    Logger logger;

    public AlertList()
    {
        nextUniqueId = 0;
        alertInfo = new Hashtable<Integer, AlertListEntry>();

        logger = new Logger("AlertList");
    }

    private int getNextUniqueId()
    {
        return nextUniqueId++;
    }

    // Use by CacheObjMgr
    // public void AddEntry(List<Integer> inputBasis, out Integer id)
    int addEntry(List<Integer> inputBasis){
        AlertListEntry alertEntry = new AlertListEntry(inputBasis);
        int id = getNextUniqueId();

        alertInfo.put(id, alertEntry);
        return id;
    }

    public void removeEntry(int id)
    {
        alertInfo.remove(id);
    }

    public boolean isBasisValid(int id)
    {
        AlertListEntry entry;
        
        //if (alertInfo.TryGetValue(id, out entry))
        if ((entry = alertInfo.get((Integer)id)) != null)
        {
            return entry.valid;
        }
        else
        {
            // Not found
            // This shouldn't happen since application deregister id
            logger.log(LoggerLevel.ASSERT, "IsBasisValid - id not valid");
        }
        return false;
    }

    /*
    public Boolean GetEntry(int id, out List<Integer> basis, out Boolean valid)
    {
        basis = null;
        valid = false;

        AlertListEntry entry;
        if (alertInfo.TryGetValue(id, out entry))
        {
            basis = entry.basis;
            valid = entry.valid;
            return true;
        }
        else
        {
            // Not found
            // This shouldn't happen since application deregister id
            logger.Log(LoggerLevel.ASSERT, "IsBasisValid - id not valid");
        }
        return false;
    }*/
    
    // Now returns null
    public List<Integer> getEntry(int id)
    {
    	AlertListEntry entry;
    	if (null != (entry = alertInfo.get(id)))
    	{
    		return entry.basis;
    	}
    	
    	logger.log(LoggerLevel.ASSERT, "isBasisValid - id not valid");
    	return null;
    }
    
    // debugging
    public String displayAlertList()
    {
        StringBuilder sb = new StringBuilder();

        int count = alertInfo.size();
        sb.append("--------------------------------------------\n");
        sb.append("Number of Entries: " + count + "\n");
        sb.append("--------------------------------------------\n");
        for (int i = 0; i < count; i++)
        {
            AlertListEntry entry = alertInfo.get(i);
            List<Integer> basis = entry.basis;
            Boolean valid = entry.valid;
            String isValid = valid ? "1" : "0";
            StringBuilder basisSb = new StringBuilder();
            for(int j = 0; j < basis.size(); j++)
            {
                basisSb.append(basis.get(j).toString() + " ");
            }

            sb.append(isValid + ": " + basisSb.toString() + "\n");
        }
        return sb.toString();
    }

    // Stream Messages that affect our alerts
    public void handleRevokeActFor(int pidActor, int pidActee)
    {
        // Some assumption that this code relies on:
        // - An act-for link is a direct link from pidActor to pidActee
        // - no cycles

        // Brute-force scan
        for (int i = 0; i < alertInfo.size(); i++)
        {
            AlertListEntry entry = alertInfo.get(i);

            Boolean foundActor = false;
            for (int j = 0; i < entry.basis.size(); i++)
            {
                if (foundActor == false)
                {
                    // look for actor
                    if (pidActor == entry.basis.get(j))
                    {
                        foundActor = true;
                    }
                }
                else
                {
                    // found pidActor in break cycle
                    if (pidActee == entry.basis.get(j))
                    {
                        // this basis must be invalidated
                        entry.valid = false;
                    }
                    else
                    {
                        // this basis is still valid, no further check needed
                        break;          // break out of inner for-loop on j
                    }
                }
            }
        }
    }

    public void handleRemovePrincipal(int pid)
    {
        // Brute-force scan
        for (int i = 0; i < alertInfo.size(); i++)
        {
            AlertListEntry entry = alertInfo.get(i);
            for (int j = 0; j < entry.basis.size(); j++)
            {
                if (pid == entry.basis.get(j))
                {
                    // invalidate
                    entry.valid = false;
                }
            }
        }
    }

    
    private static class AlertListEntry {
        // basis
        public List<Integer> basis;
        public Boolean valid;

        public AlertListEntry(List<Integer> inputBasis)
        {
            basis = new LinkedList<Integer>();
            for (int i = 0; i < inputBasis.size(); i++)
            {
                basis.add(inputBasis.get(i));
            }
            valid = true;
        }
    }

}
