package edu.mit.csail.aeolus.authority.client;

import java.util.Dictionary;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.List;

public class CoreLru {
	   // Implements a doubly-linked list with a O(1) lookup
    ListNode sortedListHead;            // least-recently used
    ListNode sortedListTail;            // most-recently used
    Dictionary<Integer, ListNode> quickListMap;

    public int numEvictRequests = 0;
    public int numEvictions = 0;
    
    public class ListNode 
    {
        public Integer key;
        public ListNode prev;
        public ListNode next;
    }

    public CoreLru()
    {
        sortedListHead = null;
        sortedListTail = null;
        quickListMap = new Hashtable<Integer, ListNode>();
    }

    public int GetNumCoreCached()
    {
        return quickListMap.size();
    }

    public Boolean IsCoreCached(Integer key)
    {
        if (quickListMap.get(key) == null)
        	return false;
        return true;
    }

    public List<Integer> EvictAndRemove(int numEntriesToEvict)
    {
    	List<Integer> candidates;
        {
            candidates = new LinkedList<Integer>();
            numEvictRequests++;

            for (int i = 0; i < numEntriesToEvict; i++)
            {
                if (quickListMap.size() <= 0)
                {
                    // there are no more entries that can be evicted
                    break;
                }

                // attempt to remove and remember candidate
                ListNode victim = sortedListHead;
                candidates.add(victim.key);
                numEvictions++;
                if (sortedListHead.next != null)
                {
                    // make this node head
                    sortedListHead = sortedListHead.next;   // fine if this is null
                    sortedListHead.prev = null;
                }
                else
                {
                    sortedListHead = null;
                }
                quickListMap.remove(victim.key);
                // victim will be garbage collected..
            }
        }
        return candidates;
    }

    public void NotifyAdded(Integer key)
    {
        {
            ListNode addNode = new ListNode();
            addNode.key = key;
            addNode.next = null;
            
            // add to tail
            if (sortedListTail != null)
            {
                sortedListTail.next = addNode;
                addNode.prev = sortedListTail;
            }
            else
            {
                // first node
                sortedListHead = addNode;
            }
            sortedListTail = addNode;

            // update map
            quickListMap.put(key, addNode);
        }
    }

    public void NotifyRemoved(Integer key)
    {
        {
            ListNode removeNode = quickListMap.get(key);
            if (removeNode.prev != null)
            {
                removeNode.prev.next = removeNode.next;   // not head node
            }
            else
            {
                // remove head node
                sortedListHead = removeNode.next;
            }
            if (removeNode.next != null)
            {
                removeNode.next.prev = removeNode.prev;   // not last node
            }
            else
            {
                // remove tail node
                sortedListTail = removeNode.prev;
            }
            quickListMap.remove(key);
            // removeNode should be garbage collected
        }
    }

    public void NotifyLookup(Integer key)
    {
        {
            ListNode lookupNode = quickListMap.get(key);

            // Unchain lookup Node
            if (lookupNode.prev != null)
            {
                lookupNode.prev.next = lookupNode.next;
            }
            else
            {
                // moving head node (only if there are other nodes)
                if(lookupNode.next != null) sortedListHead = lookupNode.next;
            }
            if (lookupNode.next != null)
            {
                lookupNode.next.prev = lookupNode.prev;
            }
            else
            {
                // moving last node (only if there are other ndoes)
                if(lookupNode.prev != null)  sortedListTail = lookupNode.prev;
            }
            
            // Add to the tail
            if(sortedListTail!=null) sortedListTail.next = lookupNode;
            sortedListTail = lookupNode;
        }
    }
}
