package edu.mit.csail.aeolus.authority.client;

import java.util.Dictionary;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.List;

public class EvictionLru<T> extends EvictionMgr<T> {
	// Implements a doubly-linked list with a O(1) lookup
	ListNode<T> sortedListHead; // least-recently used
	ListNode<T> sortedListTail; // most-recently used
	Dictionary<T, ListNode<T>> quickListMap;

	private static class ListNode<U> {
		public U key;
		public ListNode<U> prev;
		public ListNode<U> next;
	}

	public EvictionLru() {
		sortedListHead = null;
		sortedListTail = null;
		quickListMap = new Hashtable<T, ListNode<T>>();
	}

	public List<T> evictAndRemove(int numEntriesToEvict) {
		List<T> candidates = new LinkedList<T>();
		try {
			numEvictRequests++;

			for (int i = 0; i < numEntriesToEvict; i++) {
				if (quickListMap.size() <= 0) {
					// there are no more entries that can be evicted
					break;
				}

				// attempt to remove and remember candidate
				ListNode<T> victim = sortedListHead;
				candidates.add(victim.key);
				numEvictions++;
				if (sortedListHead.next != null) {
					// make this node head
					sortedListHead = sortedListHead.next; // fine if this is
															// null
					sortedListHead.prev = null;
				} else {
					sortedListHead = null;
				}
				quickListMap.remove(victim.key);
				// victim will be garbage collected..
			}
		} catch (Exception e) {
			System.err.println("EvictAndRemove: " + e);
		}
		return candidates;

	}

	public void notifyAdded(T key) {
		try {
			ListNode<T> addNode = new ListNode<T>();
			addNode.key = key;
			addNode.next = null;

			// add to tail
			if (sortedListTail != null) {
				sortedListTail.next = addNode;
				addNode.prev = sortedListTail;
			} else {
				// first node
				sortedListHead = addNode;
			}
			sortedListTail = addNode;

			// update map
			quickListMap.put(key, addNode);
		} catch (Exception e) {
			System.err.println("NotifyAdded: " + e);
		}
	}

	public void notifyRemoved(T key) {
		try {
			ListNode<T> removeNode = quickListMap.get(key);
			if (removeNode.prev != null) {
				removeNode.prev.next = removeNode.next; // not head node
			} else {
				// remove head node
				sortedListHead = removeNode.next;
			}
			if (removeNode.next != null) {
				removeNode.next.prev = removeNode.prev; // not last node
			} else {
				// remove tail node
				sortedListTail = removeNode.prev;
			}
			quickListMap.remove(key);
			// removeNode should be garbage collected
		} catch (Exception e) {
			System.err.println("NotifyRemoved: " + e);
		}
	}

	public void notifyLookup(T key) {
		try {
			ListNode<T> lookupNode = quickListMap.get(key);

			// Unchain lookup Node
			if (lookupNode.prev != null) {
				lookupNode.prev.next = lookupNode.next;
			} else {
				// moving head node (only if there are other nodes)
				if (lookupNode.next != null)
					sortedListHead = lookupNode.next;
			}
			if (lookupNode.next != null) {
				lookupNode.next.prev = lookupNode.prev;
			} else {
				// moving last node (only if there are other ndoes)
				if (lookupNode.prev != null)
					sortedListTail = lookupNode.prev;
			}

			// Add to the tail
			if (sortedListTail != null)
				sortedListTail.next = lookupNode;
			sortedListTail = lookupNode;
		} catch (Exception e) {
			System.err.println("NotifyLookup: " + e);
		}
	}
}
