package edu.mit.csail.aeolus.authority.server;

import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.Arrays;

import edu.mit.csail.aeolus.auditing.LoggingServiceInterface;
import edu.mit.csail.aeolus.auditing.graphdb.AeolusGraphLoggingService;
import edu.mit.csail.aeolus.runtime.RuntimeConfig;
import edu.mit.csail.aeolus.stream.server.StreamServer;

public class AuthorityServer {

	AuthorityManager mgr;
	StreamServer ss;
	
	private AuthorityServer() throws RemoteException{
		java.rmi.registry.LocateRegistry.createRegistry(1099);
		this.mgr = new AuthorityManager();

		mgr.removeAllSnapshots();
		this.mgr.createAuthorityServer();

		mgr.startServer();
		this.mgr.startStreamServer();		
	}

    public static void main(String[] args) throws RemoteException, NotBoundException {
	AeolusGraphLoggingService als;

		RuntimeConfig c = RuntimeConfig.get();
		c.finalize();
		
		String loggingHostName = c.getLoggingHostName();
		int loggingPort = c.getLoggingPort();
		String logServiceName = c.getAeolusLoggingServiceName();
		
		System.out.println(Arrays.toString(args));
		if(args.length > 0 && args[0].equals("-shutdown")){
		    Registry registry = LocateRegistry.getRegistry(loggingHostName, loggingPort);
		    LoggingServiceInterface service = (LoggingServiceInterface)registry.lookup(logServiceName);
		    service.shutdownService();
		    return;
		} else if(args.length > 0 && args[0].equals("-stats")) {
		    try {
			Registry registry = LocateRegistry.getRegistry(loggingHostName, loggingPort);
			LoggingServiceInterface service = (LoggingServiceInterface)registry.lookup(logServiceName);
			service.printStats();
		    } catch (RemoteException e) {
			System.err.println("Unable to collect stats: " + e);
			return;
		    }
		    return;
		}
		
		try {
		    new AuthorityServer();
		    als = new AeolusGraphLoggingService();
		    als.initialize();
		} catch (RemoteException e) {
		    System.err.println("Unable to start the authority server: " + e);
		    return;
		} catch (Throwable e) {
		    System.err.println("Caught " + e);
		    return;
		}

		try{
			while(true)
				try {
					Thread.sleep(1000);
				} catch (InterruptedException e) {
					als.shutdownService();
				}
		}finally{
			als.shutdownService();
		}
	}

}
