package edu.mit.csail.aeolus.filesystem;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import edu.mit.csail.aeolus.auditing.EventID;
import edu.mit.csail.aeolus.infoflow.Label;

/**
 * Entry in the file cache
 * Stores the file content and additional data associated with the file
 * 
 * @author viq
 *
 */
public class FileCacheEntry {
	
	public byte[] fileContent;
	public EventID predEid;
	public Label sLabel;
	public Label iLabel;
	public List<Integer> fileStreamHandles;
	
	public FileCacheEntry(byte[] fileContent, Label fileSLabel, Label fileILabel, EventID lastWriteEid)
	{
		this.fileContent = fileContent;
		this.sLabel = fileSLabel;
		this.iLabel = fileILabel;
		this.predEid = lastWriteEid;
		this.fileStreamHandles = Collections.synchronizedList(new ArrayList<Integer>());
	}
	
	/**
	 * Registers a file stream for reading from this file
	 */
	public void addFilestream(int handle)
	{
		this.fileStreamHandles.add(handle);
	}
	
	public void removeFilestream(int handle)
	{
		this.fileStreamHandles.remove(new Integer(handle));
	}
	
	public boolean isFilestreamRegistered(int handle)
	{
		return this.fileStreamHandles.contains(new Integer(handle));
	}

}
