package edu.mit.csail.aeolus.filesystem;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Calendar;
import java.util.Queue;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.ConcurrentLinkedQueue;

import edu.mit.csail.aeolus.auditing.AeolusLoggingClient;
import edu.mit.csail.aeolus.auditing.EventRecord;
import edu.mit.csail.aeolus.auditing.LoggingConstants;
import edu.mit.csail.aeolus.runtime.RuntimeConfig;

public class FileLogCollector {

	AeolusLoggingClient loggingClient;
	ConcurrentLinkedQueue<EventRecord> events;
	String hostname;

	public FileLogCollector() {
		hostname = "unknown";

		try {
			hostname = InetAddress.getLocalHost().getHostName();
		} catch (UnknownHostException e) {
			e.printStackTrace();
		}

		events = new ConcurrentLinkedQueue<EventRecord>();

		RuntimeConfig c = RuntimeConfig.get();
		boolean FSlogging = c.isEnableFSLogging();
		boolean FSloggingShip = c.isFS_enableLogShipment();
		if (FSlogging && FSloggingShip) {
			loggingClient = new AeolusLoggingClient();

			// schedule log shipment to the central server
			new Timer().scheduleAtFixedRate((new TimerTask() {
				public void run() {
					sendLogs();
				}
			}), 0, LoggingConstants.sendFileLogsPeriod);
		}
	}

	public void submitLogRecord(EventRecord rec) {
		events.add(rec);
	}

	/**
	 * Sends all the logs available so far to the logging server
	 */
	public void sendLogs() {
		// reset the pointer and sent the current content
		if (!events.isEmpty())
		{
			Queue<EventRecord> contents = events;
			events = new ConcurrentLinkedQueue<EventRecord>();
			loggingClient.sendFileLogToServer(hostname, Calendar.getInstance()
					.getTime(), contents);
		}
	}

}
