package edu.mit.csail.aeolus.filesystem.tests;

import java.io.FileNotFoundException;
import java.io.IOException;

import edu.mit.csail.aeolus.api.AeolusFile;
import edu.mit.csail.aeolus.api.AeolusFileInputStream;
import edu.mit.csail.aeolus.api.AeolusFileOutputStream;
import edu.mit.csail.aeolus.api.*;
import edu.mit.csail.aeolus.exceptions.AeolusException;
import edu.mit.csail.aeolus.exceptions.AuthorityException;
import edu.mit.csail.aeolus.runtime.RuntimeConfig;

public class AeolusFSBenchmark {

	private static final int chunk_size = 16384;
	
	public static void main(String [] args) throws AuthorityException, AeolusException, FileNotFoundException, IOException, InterruptedException
	{
		long start;
		long end;
		long total = 0;
		int mb = 1024 * 1024;
		int size = 10;
		byte[] single = new byte[] {(byte)117};
		byte[] chunk = new byte[chunk_size];
		RuntimeConfig c = RuntimeConfig.get();
		String fsHostName = c.getFsHostName();
		
		System.out.println("STARTING FS BENCHMARK TEST APP");
		
		Thread.sleep(1000);
		
		AeolusLabel empty = new AeolusLabel();
	
		// AeolusUser u = new AeolusUser();
		AeolusFile af = new AeolusFile(fsHostName, "/a");
		
		try
		{
			
			start = System.currentTimeMillis();
			// af.createNewFile(empty.getLabel(), empty.getLabel());
			af.createNewFile(empty, empty);
			end = System.currentTimeMillis();
			total += (end-start);
			
			
			System.out.println("File create: " + (end-start) + "ms");
			
			start = System.currentTimeMillis();
			AeolusFileOutputStream os = new AeolusFileOutputStream(fsHostName, "/a", true);
			for (int i = 0; i < size; i++)
			{
				for (int j = 0; j < mb; j++)
				{
					os.write(single);
				}
			}
			os.close();
			end = System.currentTimeMillis();
			total += (end-start);
			
			System.out.println("Byte by Byte write: " + (end-start) + "ms");
			
			
			start = System.currentTimeMillis();
			AeolusFileInputStream is = new AeolusFileInputStream(fsHostName, "/a");
			os = new AeolusFileOutputStream(fsHostName, "/a", false);
			for(int cursor = 0; cursor < size * mb; cursor += chunk_size)
			{
				is.read(chunk);
				os.write(chunk);
			}
			is.close();
			os.close();
			end = System.currentTimeMillis();
			total += (end-start);
			
			System.out.println("Chunk by Chunk read then write: " + (end-start) + "ms");
			
			start = System.currentTimeMillis();
			os = new AeolusFileOutputStream(fsHostName, "/a", false);
			for(int cursor = 0; cursor < size * mb; cursor += chunk_size)
			{
				os.write(chunk);
			}
			os.close();
			end = System.currentTimeMillis();
			total += (end-start);
			
			System.out.println("Chunk by Chunk write: " + (end-start) + "ms");
			
			start = System.currentTimeMillis();
			is = new AeolusFileInputStream(fsHostName, "/a");
			int count = 1;
			while (count > 0)
			{
				count = is.read(single);
			}
			is.close();
			end = System.currentTimeMillis();
			total += (end-start);
			
			System.out.println("Byte by Byte read: " + (end-start) + "ms");
			
			start = System.currentTimeMillis();
			is = new AeolusFileInputStream(fsHostName, "/a");
			count = 1;
			while (count > 0)
			{
				count = is.read(chunk);
			}
			is.close();
			end = System.currentTimeMillis();
			total += (end-start);
			
			System.out.println("Chunk by Chunk read: " + (end-start) + "ms");
			System.out.println("Total test runnning time: " + total + "ms");
			af.delete();
			
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		System.out.println("DONEDONEDONEDONE");
	}
	
}
