package edu.mit.csail.aeolus.filesystem.tests;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;



public class FSBenchmark {

	private static final int chunk_size = 16384;
	
	public static void main(String [] args) throws FileNotFoundException, IOException, InterruptedException
	{
		long start;
		long end;
		long total = 0;
		int mb = 1024 * 1024;
		int size = 10;
		byte[] single = new byte[] {(byte)117};
		byte[] chunk = new byte[chunk_size];
		
		System.out.println("STARTING FS BENCHMARK TEST APP");
		
		Thread.sleep(1000);
		
		try
		{
			File f = new File("/tmp/aeolusroot/localhost/b");
			f.delete();
			start = System.currentTimeMillis();
			f.createNewFile();
			end = System.currentTimeMillis();
			total += (end-start);
			
			
			System.out.println("File create: " + (end-start) + "ms");
			
			start = System.currentTimeMillis();
			FileOutputStream os = new FileOutputStream(f, true);
			for (int i = 0; i < size; i++)
			{
				for (int j = 0; j < mb; j++)
				{
					os.write(single);
				}
			}
			os.close();
			end = System.currentTimeMillis();
			total += (end-start);
			
			System.out.println("Byte by Byte write: " + (end-start) + "ms");
			
			
			start = System.currentTimeMillis();
			FileInputStream is = new FileInputStream(f);
			os = new FileOutputStream(f, false);
			for(int cursor = 0; cursor < size * mb; cursor += chunk_size)
			{
				is.read(chunk);
				os.write(chunk);
			}
			is.close();
			os.close();
			end = System.currentTimeMillis();
			total += (end-start);
			
			System.out.println("Chunk by Chunk read then write: " + (end-start) + "ms");
			
			start = System.currentTimeMillis();
			os = new FileOutputStream(f, false);
			for(int cursor = 0; cursor < size * mb; cursor += chunk_size)
			{
				os.write(chunk);
			}
			os.close();
			end = System.currentTimeMillis();
			total += (end-start);
			
			System.out.println("Chunk by Chunk write: " + (end-start) + "ms");
			
			start = System.currentTimeMillis();
			is = new FileInputStream(f);
			int count = 1;
			while (count > 0)
			{
				count = is.read(single);
			}
			is.close();
			end = System.currentTimeMillis();
			total += (end-start);
			
			System.out.println("Byte by Byte read: " + (end-start) + "ms");
			
			start = System.currentTimeMillis();
			is = new FileInputStream(f);
			count = 1;
			while (count > 0)
			{
				count = is.read(chunk);
			}
			is.close();
			end = System.currentTimeMillis();
			total += (end-start);
			
			System.out.println("Chunk by Chunk read: " + (end-start) + "ms");
			System.out.println("Total test runnning time: " + total + "ms");
			
			
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}
	
}

