package edu.mit.csail.aeolus.infoflow;


/**
 * Structure encapsulating the closure key, pid, (and version) extracted from the Closure map
 * 
 * A closure is bound to a key and an anonymous principal.
 * 
 * The principal is used by the platform to identify the authority that has been given to the closure.
 * The key is used to identify the authenticity of the code.
 * 
 * viq
 *
 */
public class ClosureMapData {

	 /** Anonymous principal associated with the closure. */
	 private PrincipalID pid;
	 
	 /** Key used to verify authenticity of the code. */
     private String key;

     /** Optional SW upgrade info (tells us what is the latest version) (mutable) */
     public String version;

     public ClosureMapData(String key, PrincipalID pid)
     {
         this.pid = pid;
         this.key = key;
         this.version = "Culture=neutral, Version=1.0.0.0";
     }

     public ClosureMapData(String key, PrincipalID pid, String version)
     {
         this.pid = pid;
         this.key = key;
         this.version = version;
     }
     
     /**
      * Returns the closure pid
      */
     public PrincipalID getClosurePid() 
     {
    	 return this.pid;
     }
     
     /**
      * Returns the closure key
      */
     public String getClosureKey() 
     {
    	 return this.key;
     }
     
     /**
      * Returns the closure version
      */
     public String getClosureVersion() 
     {
    	 return this.version;
     }
     
     /**
      * Sets the closure version
      */
     public void setClosureVersion(String newVersion) 
     {
    	 this.version = newVersion;
     }
}
