package edu.mit.csail.aeolus.infoflow;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import edu.mit.csail.aeolus.utilities.Utils;

/**
 * Represents information flow control state: 
 * - principal id, 
 * - secrecy and integrity labels
 * - basis 
 * 
 * viq
 *
 */

public class InfoFlowState implements Serializable 
{
	private static final long serialVersionUID = 6663758047319519054L;

	public List<Integer> basis;
	
	/** Principal ID */
	public PrincipalID pid;
	
	/** Secrecy label */
	public Label SLabel;
	
	/** Integrity label */
	public Label ILabel;

	/**
	 * Creates a state with empty S and I labels
	 * and a pid of -1
	 */
	public InfoFlowState()
	{
		this.basis = new ArrayList<Integer>();
		this.pid = new PrincipalID(-1);
		this.SLabel = new Label();
		this.ILabel = new Label();
	}

	/**
	 * Creates a copy of the given state
	 */
	public InfoFlowState(InfoFlowState state)
	{
		this.basis = new ArrayList<Integer>(state.basis);
		this.pid = state.pid;
		this.SLabel = new Label(state.SLabel);
		this.ILabel = new Label(state.ILabel);
	}

//	/**
//	 * Creates a new state with the given S and I labels and a given pid
//	 */
//	public InfoFlowState(int pid, String sLabelSorted, String iLabelSorted)
//	{
//		this(new PrincipalID(pid), new Label(sLabelSorted), new Label(iLabelSorted));
//	}
//
//	/**
//	 * Creates a new state with the given S and I labels and a pid of -1
//	 */
//	public InfoFlowState(Label sLabelSorted, Label iLabelSorted)
//	{
//		this(-1, sLabelSorted, iLabelSorted);
//	}

	/**
	 * Creates a new state with the given principal, labels, and
	 * basis.  The caller is responsible for copying the labels and
	 * basis as needed.
	 */
	public InfoFlowState(PrincipalID p, Label sLabel, Label iLabel, ArrayList<Integer> basis)
	{
		this.basis = basis;
		this.pid = p;
		this.SLabel = sLabel;
		this.ILabel = iLabel;
	}

	/**
	 * Creates a new state with the given S and I labels and a
	 * given principal.  The caller is responsible for copying the
	 * labels and basis as needed.
	 */
	public InfoFlowState(PrincipalID p, Label sLabel, Label iLabel)
	{
		this(p, sLabel, iLabel, new ArrayList<Integer>());
	}

	/**
	 * Creates a new state with the given S and I labels and a pid of -1
	 */
	public InfoFlowState(Label sLabel, Label iLabel)
	{
		this(new PrincipalID(-1), sLabel, iLabel);
	}

	public String toString()
	{
		StringBuilder sb = new StringBuilder();
		String delimiter = ";";

		sb.append("PID=" + this.pid.getValue());
		sb.append(delimiter);
		sb.append("S=" + this.SLabel.toString());
		sb.append(delimiter);
		sb.append("I=" + this.ILabel.toString());
		sb.append(delimiter);
		sb.append("Basis=" + Utils.listToString(this.basis));
		
		return sb.toString();
	}
	
	/**
	 * Two states are equal if:
	 * - their secrecy and integrity labels are equal
	 * - pid values are equal
	 * - basis values (and value order in the list) are equal
	 */
	public boolean equals(Object o)
	{
		if(o == null)
		{
			return false;
		}
		if(o instanceof InfoFlowState)
		{
			return (this.SLabel.equalsLabel(((InfoFlowState) o).SLabel)) &&
					(this.ILabel.equalsLabel(((InfoFlowState) o).ILabel)) &&
					(this.pid.getValue() == ((InfoFlowState) o).pid.getValue()) &&
					(this.basis.equals(((InfoFlowState) o).basis));
		}
		return false;
		
	}
}

