package edu.mit.csail.aeolus.infoflow;

/**
 * Label is a set of tags associated with a process or object in the system
 * 
 * @author viq
 * 
 * (old Label code is at the bottom of this class, but it seems way too convoluted)
 */
import java.io.Serializable;
import java.util.Arrays;
import java.util.List;
import java.util.ArrayList;

import edu.mit.csail.aeolus.utilities.ArraySet;

public final class Label implements Serializable
{
	private static final long serialVersionUID = 5276299235330330833L;
	public static final String strDelimiter = " ";

	protected final ArraySet<TagID> tags;	
	
	public Label()
	{
		tags = new ArraySet<TagID>();
	}
	
	public Label(TagID... listOfTags ){
		this(Arrays.asList(listOfTags));
	}
	
	public Label(List<TagID> listOfTags)
	{
		tags = new ArraySet<TagID>(listOfTags);
		removeSubtags();
	}
	
	private void removeSubtags(){
		ArraySet<TagID> newtids = new ArraySet<TagID>();
		for (TagID tid : tags){
			if (!hasTag(new TagID(tid.getParent()))){
				newtids.add(tid);
			}
		}
		tags.clear();
		tags.addAll(newtids);
	}
//	/**
//	 * Constructs a label from a string of integers
//	 * @requires properListOfLabels is a properly formatted string of integers
//	 * separated by Label.strDelimiter
//	 */
//	public Label(String properListOfLabels)
//	{
//		tags = convertToLabel(properListOfLabels);
//	}
	
	/**
	 * Create a new copy of l
	 */
        public Label(Label l){
		Label newl = l.clone();
		this.tags = newl.tags;
                // if(l != null){
                  //       tags = new ArraySet<TagID>(l.tags);
                // }else{
                  //       tags = new ArraySet<TagID>();
                // }
        }

	public Label clone() {
		ArrayList<TagID> newtags = new ArrayList<TagID>();
		for (TagID t: this.tags) {
			newtags.add(t.clone());
		}
		return new Label(newtags);
	}
	
	public boolean hasTag(TagID tagId)
	{
		if(tagId == null)
		{
			return false;
		}
		if (tagId.getParent() != TagID.DEFAULT_PARENT_ID)
			return (tags.contains(tagId) || hasTag(new TagID(tagId.getParent())));
		return tags.contains(tagId);
	}
	
	public boolean addTag(TagID tagId)
	{
		if(tagId == null)
		{
			return false;
		}
		if (hasTag(tagId)){
			return true;
		}else{
			tags.add(tagId);
			removeSubtags();
			return true;
		}		
	}
	
	public boolean removeTag(TagID tagId)
	{
		if(tagId == null)
		{
			return false;
		}
		return tags.remove(tagId);
	}
	public boolean removeTags(List<TagID> tagIDs){
		return tags.removeAll(tagIDs);
	}

	public boolean addTags(List<TagID> tagIDs){
		return tags.addAll(tagIDs);
	}
	
	public void clear()
	{
		tags.clear();
	}
	
	/**
	 * Check if the two labels have the same tags
	 * @param l
	 * @return true if the two labels have the same tags, else false
	 */
	public boolean equalsLabel(Label l)
	{
		// System.out.println("Label equals: " + l.tags.size() + " " + tags.size());
		if (l.tags.size() == 0 && tags.size() == 0) return true;
		return (l.tags.equals(tags));
	}

	public boolean equals(Object o) {
		if (o == null)
			return (false);
		if (!(o instanceof Label))
			return (false);
		return (tags.equals(((Label)o).tags));
	}

	/**
	 * Returns true if this label is a subset of l
	 * (i.e. if l contains all the tags in this label)
	 */
	public boolean isSubsetOf(Label l)
	{
		if(l == null)
		{
			return false;
		}
		if (l.tags.containsAll(this.tags)) return true;
		//subtags, not sure if above line is necessary anymore
		for (TagID t : this.getTags())
			if (! l.hasTag(t))
				return false;
		return true;
	}
	
	/**
	 * Returns a new label containing union
	 * of the tags of this and l
	 */
	public Label union(Label l)
	{
		Label resultLabel = new Label(this);
			
		if(l != null)
		{
			resultLabel.tags.addAll(l.tags);
		}
		
		return resultLabel;
	}
	
	public void unionInto(Label l) {
	    tags.addAll(l.tags);
	}

	/**
	 * Returns a new Label containing all the common tags
	 * between this and l
	 */
	public Label intersection(Label l)
	{
		Label resultLabel = null;
		
		if( l != null)
		{
			resultLabel = new Label(this);
			resultLabel.tags.retainAll(l.tags);
		}
		return resultLabel;
	}

	public void intersectInto(Label l) {
	    tags.retainAll(l.tags);
	}
	
	/**
	 * Returns a new label containing the difference between this and l
	 * (i.e. all the tags that are in this label but not in l)
	 */
	public Label difference(Label l)
	{
		Label resultLabel = new Label(this);
			
		if(l != null)
		{
			resultLabel.tags.removeAll(l.tags);
		}
		
		return resultLabel; 
	}
	
	public TagID[] getTags()
	{
		TagID[] tagArray = new TagID[tags.size()];
		int i = 0;
		for(TagID tag : tags)
		{
			tagArray[i] = tag;
			i++;
		}
		
		return tagArray;
	}
	
	public String toString()
	{
	    /*
		String s = "";
		
		Iterator<TagID> iter = tags.iterator();
		
		if(iter.hasNext())
		{
			s += iter.next().getValue();
		}
		while(iter.hasNext())
		{
			s += strDelimiter + iter.next().getValue();
		}
		
		
		return s;
	    */
	    // return (tags.toString());
	    String s = "";
	    for (TagID t: tags) {
		s += t.toString() + "/";
	    }
	    return s;
	}
	
	public boolean isEmpty()
	{
		return (tags.size() == 0);
	}
	
	public int size()
	{
		return tags.size();
	}

	
	/**
	 * This method should not be used
	 * If used, must ensure the string is properly formatted
	 */
	private ArraySet<TagID> convertToLabel(String listOfTags)
	{
		ArraySet<TagID> result = new ArraySet<TagID>();
		
		if((listOfTags.equals("")) || (listOfTags == null))
		{
			return result;
		}
		
		String[] tags = listOfTags.split(strDelimiter);
		
		for (int i = 0; i < tags.length; i++)
		{
			result.add(new TagID(Integer.parseInt(tags[i])));
		}
		
		return result;
	}

	public Label intersectionDestroy(Label oldI) {
		Label resultLabel = oldI;
		
		if( oldI != null){
			resultLabel.tags.retainAll(this.tags);
		}
		return resultLabel;
	}

	public Label unionDestroy(Label oldS) {
		if(oldS != null){
			oldS.tags.addAll(this.tags);
			return oldS;
		}else{
			return this;
		}
	}

	public Integer[] getTagValues() {
		Integer[] tagVals = new Integer[tags.size()];
		int i = 0;
		for(TagID tag : tags){
			tagVals[i++] = tag.getValue();
		}
		return tagVals;
	}

	public int hashCode() {
	    int hashVal = 0;
	    for (TagID tag : tags)
		hashVal += tag.hashCode();
	    return (hashVal);
	}
}
