package edu.mit.csail.aeolus.io;

import java.io.IOException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketAddress;
import java.net.SocketException;
import java.nio.channels.ServerSocketChannel;
import java.security.AccessController;
import java.security.PrivilegedActionException;
import java.security.PrivilegedExceptionAction;

public class AeolusServerSocket extends ServerSocket {
	private ServerSocket wrapped;
	
	public AeolusServerSocket(final int port) throws IOException{
		this(null);
		try {
			wrapped = AccessController.doPrivileged(
			new PrivilegedExceptionAction<ServerSocket>() {
				public ServerSocket run() throws IOException{
					return new ServerSocket(port);
				}
			});
		} catch (PrivilegedActionException e) {
			throw new IOException(e.getMessage());
		}
	}
	
	protected AeolusServerSocket(ServerSocket wrapped)
			throws IOException {
		this.wrapped = wrapped;
	}

	@Override
	public Socket accept() throws IOException {
		try {
			AeolusSocket accept = AccessController.doPrivileged(
			new PrivilegedExceptionAction<AeolusSocket>() {
				public AeolusSocket run() throws IOException{
					return new AeolusSocket(wrapped.accept());
				}
			});
			return accept;
		} catch (PrivilegedActionException e) {
			throw new IOException(e.getMessage());
		}
	}

	@Override
	public void bind(SocketAddress endpoint, int backlog) throws IOException {
		wrapped.bind(endpoint, backlog);
	}

	@Override
	public void bind(SocketAddress endpoint) throws IOException {
		wrapped.bind(endpoint);
	}

	@Override
	public void close() throws IOException {
		wrapped.close();
	}

	@Override
	public ServerSocketChannel getChannel() {
		return wrapped.getChannel();
	}

	@Override
	public InetAddress getInetAddress() {
		return wrapped.getInetAddress();
	}

	@Override
	public int getLocalPort() {
		return wrapped.getLocalPort();
	}

	@Override
	public SocketAddress getLocalSocketAddress() {
		return wrapped.getLocalSocketAddress();
	}

	@Override
	public synchronized int getReceiveBufferSize() throws SocketException {
		// TODO Auto-generated method stub
		return wrapped.getReceiveBufferSize();
	}

	@Override
	public boolean getReuseAddress() throws SocketException {
		// TODO Auto-generated method stub
		return wrapped.getReuseAddress();
	}

	@Override
	public synchronized int getSoTimeout() throws IOException {
		// TODO Auto-generated method stub
		return wrapped.getSoTimeout();
	}

	@Override
	public boolean isBound() {
		// TODO Auto-generated method stub
		return wrapped.isBound();
	}

	@Override
	public boolean isClosed() {
		// TODO Auto-generated method stub
		return wrapped.isClosed();
	}

	// @Override
	// public void setPerformancePreferences(int connectionTime, int latency,
// 			int bandwidth) {
		// TODO Auto-generated method stub
// 		wrapped.setPerformancePreferences(connectionTime, latency, bandwidth);
// 	}

	@Override
	public synchronized void setReceiveBufferSize(int size)
			throws SocketException {
		// TODO Auto-generated method stub
		wrapped.setReceiveBufferSize(size);
	}

	@Override
	public void setReuseAddress(boolean on) throws SocketException {
		// TODO Auto-generated method stub
		wrapped.setReuseAddress(on);
	}

	@Override
	public synchronized void setSoTimeout(int timeout) throws SocketException {
		// TODO Auto-generated method stub
		wrapped.setSoTimeout(timeout);
	}

	@Override
	public String toString() {
		// TODO Auto-generated method stub
		return wrapped.toString();
	}	
}
