package edu.mit.csail.aeolus.io;

import java.io.IOException;
import java.io.InputStream;

import edu.mit.csail.aeolus.api.AeolusUserInternal;
import edu.mit.csail.aeolus.infoflow.InfoFlowState;
import edu.mit.csail.aeolus.infoflow.Label;

/**
 * This class allows us to wrap an input stream with
 * calls that will check that process labels are valid.
 * @author blanks
 *
 */
public class CheckedInputStream extends InputStream{
	private InputStream wrappedStream;
	private Label integrity;
	private Label secrecy;
	
	public CheckedInputStream(InputStream wrapped, Label integrity, Label secrecy){
		this.wrappedStream = wrapped;
		this.integrity = integrity;
		this.secrecy = secrecy;
	}
	
	@Override
	public int read() throws IOException {
		boolean valid;
		try{
			AeolusUserInternal u = new AeolusUserInternal();
			InfoFlowState ifs = u.getInfoFlowState();
			valid = ifs.ILabel.isSubsetOf(integrity) && secrecy.isSubsetOf(ifs.SLabel);
		} catch (Exception e) {
			throw new IOException(e.getMessage());
		}
		if(!valid)
			throw new IOException("Invalid process labels for reading from inputstream.");
		return wrappedStream.read();
	}
	
}
