package edu.mit.csail.aeolus.runtime;

import java.rmi.RemoteException;
import java.util.List;
import java.util.Random;

import edu.mit.csail.aeolus.api.SystemNodeID;
import edu.mit.csail.aeolus.auditing.EventID;
import edu.mit.csail.aeolus.authority.api.AuthReply;
import edu.mit.csail.aeolus.exceptions.AuthorityException;
import edu.mit.csail.aeolus.authority.api.ClosureReply;
import edu.mit.csail.aeolus.authority.api.ClosureTableEntry;
import edu.mit.csail.aeolus.authority.client.api.InterfaceAuthorityClient;
import edu.mit.csail.aeolus.exceptions.AeolusException;
import edu.mit.csail.aeolus.infoflow.ClosureID;
import edu.mit.csail.aeolus.infoflow.PrincipalID;
import edu.mit.csail.aeolus.infoflow.TagID;
import edu.mit.csail.aeolus.stream.server.api.RegisterReturn;

/**
 * Stub: always replies true to authority inquiries
 * 
 * @author viq
 *
 */


public class AuthorityProxyStub implements InterfaceAuthorityClient 
{
	
	//String echo(String msg);
	
    // ---- Authority Update API ------
    /*
     * int status
     * out int pidCreated
     * out int coreIdCreated
     */
	private static AuthReply<PrincipalID> dummyPrincipal = new AuthReply<PrincipalID>(null, new PrincipalID(1));
	private static AuthReply<Void> dummyVoid = new AuthReply<Void>(null, null);
	private static AuthReply<Boolean> dummyBool = new AuthReply<Boolean>(null, true);
	
	
	public int getCurrentInvalidationNumber()
	{
		return 0;
	}
	
	public boolean getUpToInvalidation(int invalidationNumber) throws RemoteException
	{
		return true;
	}
	
	public RegisterReturn register() throws AeolusException
	{
		return new RegisterReturn(new SystemNodeID(1),new PrincipalID(1), false);
	}
	
    public AuthReply<PrincipalID> createPrincipal(PrincipalID pidCreator, EventID predecessor) throws AuthorityException
    {
    	return dummyPrincipal;
    }

    /*
     * int status
     * out int pidCreated
     * out int coreIdCreated
     */
    public AuthReply<PrincipalID> createPrincipalInNewCore(PrincipalID pidCreator, EventID predecessor) throws AuthorityException
    {
    	return dummyPrincipal;
    }

    /*
     * int status
     * out int tidCreated
     */
    public AuthReply<TagID> createTaint(PrincipalID pidCreator, EventID predecessor) throws AuthorityException
    {
    	return new AuthReply<TagID>(null, new TagID(1));

    }

    // ------------------- DELEGATION -------------------
    public AuthReply<Void> grant(PrincipalID pidRequestor, PrincipalID pidGrantor, PrincipalID pidGrantee, TagID tid, EventID predecessor) throws AuthorityException
    {
    	return dummyVoid;
    }

    public AuthReply<Void> delegateActsFor(PrincipalID pidRequestor, PrincipalID pidFrom, PrincipalID pidTo, EventID predecessor) throws AuthorityException
    {
    	return dummyVoid;
    }

    // ------------------- REVOCATION -------------------
    public AuthReply<Void> revokeGrant(PrincipalID pidRevoker, PrincipalID pidGrantor, PrincipalID pidGrantee, TagID tid, EventID predecessor) throws AuthorityException
    {
    	return dummyVoid;	
    }

    public AuthReply<Void> revokeActsFor(PrincipalID pidRevoker, PrincipalID pidActor, PrincipalID pidActee, EventID predecessor) throws AuthorityException
    {
    	return dummyVoid;
    }
    
    // ---- Authority Lookup API ----

    public AuthReply<Boolean> hasAuthority(PrincipalID pid, TagID tid)
    {
    	return dummyBool;
    }

    public AuthReply<Boolean> isActee(PrincipalID pidActor, PrincipalID pidActee)
    {
    	return dummyBool;
    }


    public AuthReply<Boolean> canSwitch(List<Integer> currBasis, PrincipalID fromPid, PrincipalID toPid){
    	return dummyBool;
    }


    public AuthReply<Boolean> canSwitch(String currBasisList, PrincipalID fromPid, PrincipalID toPid){
    	return dummyBool;
    }


    public AuthReply<Boolean> isBasisValid(List<Integer> basis)
    {
    	return dummyBool;
    }


    public AuthReply<Boolean> hasAuthority(List<Integer> currBasis, PrincipalID pid, List<TagID> tids) 
    {
    	return dummyBool;
    }
    

    
    // Closures
    
    public AuthReply<ClosureReply> createClosure(String key, EventID predecessor)
    {
    	return new AuthReply<ClosureReply>(null, null);
    }
    
    public AuthReply<Void> closureActsFor(PrincipalID pidRequestor, ClosureID cid, PrincipalID pidActee, EventID predecessor) throws AuthorityException
    {
        return dummyVoid;
    }

    public AuthReply<Void> closureGrant(PrincipalID pidRequestor, ClosureID cid, PrincipalID pidGrantor, TagID tid, EventID predecessor) throws AuthorityException
    {
        return dummyVoid;
    }

	public AuthReply<ClosureTableEntry> getClosureInfo(ClosureID closure)
			throws AeolusException {
    	return new AuthReply<ClosureTableEntry>(null, null);
	}

	public int getNextPlatformInstanceID() throws AeolusException {
		return new Random().nextInt();
	}

	public AuthReply<Boolean> isSystemNodeRegistered(SystemNodeID node) throws AeolusException {
		return new AuthReply<Boolean>(null, true);
	}

	public AuthReply<TagID> createSubTag(TagID parent, PrincipalID pidCreator,
			EventID predecessor) throws RemoteException, AuthorityException {
    	return new AuthReply<TagID>(null, new TagID(1));
	}
}
