package edu.mit.csail.aeolus.runtime;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;


/** 
 * This is a class that manages the connection settings of a Local Runtime instance.
 */
public class RuntimeConfig
{   
    // RPC-related configurations
    
	private static RuntimeConfig instance;

	@CommandLineArg(name="-hostname")
	private String hostname = "localhost";
	@CommandLineArg(name="-startPort")
	private int startPort = 2001;
	@CommandLineArg(name="-rpcPort")
	private int rpcPort = 2001;

	private String aeolusRpcServiceName = "AeolusRpc";
    
	@CommandLineArg(name="-usedummy")
	private boolean useDummyAS = false;
	
    // Audit Trails related 
	@CommandLineArg(name="-logging")
	private boolean enableLogging = true;
	@CommandLineArg(name="-logShipping")
    private boolean enableLogShipment = true;
	@CommandLineArg(name="-logHostname")
    private String loggingHostName = "localhost";
	@CommandLineArg(name="-logPort")
    private int loggingPort = 2005;
    private String aeolusLoggingServiceName = "AeolusLogging";
    
    // File System related
    
	@CommandLineArg(name="-fs")
    private boolean enableFS = true;
	@CommandLineArg(name="-fsRoot")
    private String fileSystemRootPath = "/tmp/aeolusroot/localhost";
	@CommandLineArg(name="-fsMount")
	private String fsClientMountLocation = "/tmp/aeolusroot";
	@CommandLineArg(name="-fsHostname")
    private String fsHostName = "localhost";
	@CommandLineArg(name="-fsPort")
    private int fsPort = 2020;
    private String fsServiceName = "AeolusFS";
    
	private String fsClientServiceName = "FSClient";
	@CommandLineArg(name="-fsServicePort")
	private int fsServicePort = 2020;
	@CommandLineArg(name="-fsClientPort")
	private int fsClientPort = 2021;
	
	@CommandLineArg(name="-fsCaching")
	private boolean FS_enableCaching = true;
	@CommandLineArg(name="-fsLogging")
	private boolean enableFSLogging = true;
	@CommandLineArg(name="-fsLogShip")
	private boolean FS_enableLogShipment = true;
	@CommandLineArg(name="-enableLogPP")
	private boolean isEnableLogPreProcessing = true;
	@CommandLineArg(name="-fastdispatch")
	private boolean isAnalyzerFastDispatching = true;

	@CommandLineArg(name="-monitorDispatcher")
	private boolean isMonitoringDispatcherThroughput = false;
	@CommandLineArg(name="-monitorProcessor")
	private boolean isMonitoringProcessorThroughput = false;
	@CommandLineArg(name="-measureLatency")
	private boolean isMeasuringLatency = false;

	@CommandLineArg(name="-indexEventTable")
	private boolean indexEventTable = true;

	@CommandLineArg(name="-putPartitionEvents")
	private boolean putPartitionEvents = true;

	@CommandLineArg(name="-timeEventTable")
	private boolean timeEventTable = false;
	@CommandLineArg(name="-quitWhenEventsDone")
	private boolean quitWhenEventsDone = false;

	@CommandLineArg(name="-logProcessorActive")
	private boolean logProcessorActive = true;

    // Internal - AppDomain Thread Pool
    
    private int maxWorkerThreads = 500;
    private int appDomainPoolSize = 100;	
	
	public static RuntimeConfig get(){
		if(instance == null)
			instance = new RuntimeConfig();
		return instance;
	}
	
	private boolean finalized = false;



	private static Map<String, Field> stringToField = new HashMap<String, Field>();
	
	static{
		Class<?> clazz = RuntimeConfig.class;
		for(Field f : clazz.getDeclaredFields()){
			CommandLineArg a = f.getAnnotation(CommandLineArg.class);
			if(a != null){
				stringToField.put(a.name(), f);
			}
		}
	}
	
	/**
	 * Returns the command line options left over after 
	 * processing
	 * @throws IllegalAccessException 
	 * @throws IllegalArgumentException 
	 * @throws  
	 */
	public static String[] loadFromCL(String[] args){
		try{
			RuntimeConfig c = get();
			ArrayList<String> restArgs = new ArrayList<String>();
			for(int i = 0; i < args.length; i++){
				Field f = stringToField.get(args[i]);
				if(f != null){
					Class<?> type = f.getType();
					String val = args[i+1];
					if(type.equals(Integer.class) || type.equals(Integer.TYPE)){
						f.set(c, Integer.valueOf(val));
					}else if(type.equals(Boolean.class) || type.equals(Boolean.TYPE)){
						f.set(c, val.toUpperCase().startsWith("T"));
					}else if(type.equals(String.class)){
						f.set(c, val);
					}
					i++;
				}else{
					restArgs.add(args[i]);
				}
			}
			return restArgs.toArray(new String[0]);
		}catch(Exception e){
			e.printStackTrace();
			System.exit(-1);
			return null;
		}
	}
	
	public void finalize(){
		finalized = true;
	}
	
    public String getHostname() {
    	if(!finalized)
    		throw new RuntimeException("RuntimeConfig must be finalized!");
		return hostname;
	}

	public int getStartPort() {
    	if(!finalized)
    		throw new RuntimeException("RuntimeConfig must be finalized!");
		return startPort;
	}

	public int getRpcPort() {
    	if(!finalized)
    		throw new RuntimeException("RuntimeConfig must be finalized!");
		return rpcPort;
	}

	public String getAeolusRpcServiceName() {
    	if(!finalized)
    		throw new RuntimeException("RuntimeConfig must be finalized!");
		return aeolusRpcServiceName;
	}

	public boolean isEnableLogging() {
    	if(!finalized)
    		throw new RuntimeException("RuntimeConfig must be finalized!");
		return enableLogging;
	}

	public boolean isEnableLogShipment() {
    	if(!finalized)
    		throw new RuntimeException("RuntimeConfig must be finalized!");
		return enableLogShipment;
	}

	public String getLoggingHostName() {
    	if(!finalized)
    		throw new RuntimeException("RuntimeConfig must be finalized!");
		return loggingHostName;
	}

	public int getLoggingPort() {
    	if(!finalized)
    		throw new RuntimeException("RuntimeConfig must be finalized!");
		return loggingPort;
	}

	public String getAeolusLoggingServiceName() {
    	if(!finalized)
    		throw new RuntimeException("RuntimeConfig must be finalized!");
		return aeolusLoggingServiceName;
	}

	public boolean isEnableFS() {
    	if(!finalized)
    		throw new RuntimeException("RuntimeConfig must be finalized!");
		return enableFS;
	}

	public String getFileSystemRootPath() {
    	if(!finalized)
    		throw new RuntimeException("RuntimeConfig must be finalized!");
		return fileSystemRootPath;
	}
	
	public String getFsClientMountLocation() {
    	if(!finalized)
    		throw new RuntimeException("RuntimeConfig must be finalized!");
		return fsClientMountLocation;
	}

	public String getFsHostName() {
    	if(!finalized)
    		throw new RuntimeException("RuntimeConfig must be finalized!");
		return fsHostName;
	}

	public int getFsPort() {
    	if(!finalized)
    		throw new RuntimeException("RuntimeConfig must be finalized!");
		return fsPort;
	}

	public String getFsServiceName() {
    	if(!finalized)
    		throw new RuntimeException("RuntimeConfig must be finalized!");
		return fsServiceName;
	}

	public String getFsClientServiceName() {
    	if(!finalized)
    		throw new RuntimeException("RuntimeConfig must be finalized!");
		return fsClientServiceName;
	}

	public int getFsServicePort() {
    	if(!finalized)
    		throw new RuntimeException("RuntimeConfig must be finalized!");
		return fsServicePort;
	}

	public int getFsClientPort() {
    	if(!finalized)
    		throw new RuntimeException("RuntimeConfig must be finalized!");
		return fsClientPort;
	}

	public boolean isFS_enableCaching() {
    	if(!finalized)
    		throw new RuntimeException("RuntimeConfig must be finalized!");
		return FS_enableCaching;
	}

	public boolean isEnableFSLogging() {
    	if(!finalized)
    		throw new RuntimeException("RuntimeConfig must be finalized!");
		return enableFSLogging;
	}

	public boolean isFS_enableLogShipment() {
    	if(!finalized)
    		throw new RuntimeException("RuntimeConfig must be finalized!");
		return FS_enableLogShipment;
	}

	public int getMaxWorkerThreads() {
    	if(!finalized)
    		throw new RuntimeException("RuntimeConfig must be finalized!");
		return maxWorkerThreads;
	}

	public int getAppDomainPoolSize() {
    	if(!finalized)
    		throw new RuntimeException("RuntimeConfig must be finalized!");
		return appDomainPoolSize;
	}
	
    
    public void setHostname(String hostname) {
    	if(finalized)
    		return;
		this.hostname = hostname;
	}

	public void setStartPort(int startPort) {
    	if(finalized)
    		return;
		this.startPort = startPort;
	}

	public void setRpcPort(int rpcPort) {
    	if(finalized)
    		return;
		this.rpcPort = rpcPort;
	}

	public void setAeolusRpcServiceName(String aeolusRpcServiceName) {
    	if(finalized)
    		return;
		this.aeolusRpcServiceName = aeolusRpcServiceName;
	}

	public void setEnableLogging(boolean enableLogging) {
    	if(finalized)
    		return;
		this.enableLogging = enableLogging;
	}

	public void setEnableLogShipment(boolean enableLogShipment) {
    	if(finalized)
    		return;
		this.enableLogShipment = enableLogShipment;
	}

	public void setLoggingHostName(String loggingHostName) {
    	if(finalized)
    		return;
		this.loggingHostName = loggingHostName;
	}

	public void setLoggingPort(int loggingPort) {
    	if(finalized)
    		return;
		this.loggingPort = loggingPort;
	}

	public void setAeolusLoggingServiceName(String aeolusLoggingServiceName) {
    	if(finalized)
    		return;
		this.aeolusLoggingServiceName = aeolusLoggingServiceName;
	}

	public void setEnableFS(boolean enableFS) {
    	if(finalized)
    		return;
		this.enableFS = enableFS;
	}

	public void setFileSystemRootPath(String fileSystemRootPath) {
    	if(finalized)
    		return;
		this.fileSystemRootPath = fileSystemRootPath;
	}

	public void setFsHostName(String fsHostName) {
    	if(finalized)
    		return;
		this.fsHostName = fsHostName;
	}

	public void setFsPort(int fsPort) {
    	if(finalized)
    		return;
		this.fsPort = fsPort;
	}

	public void setFsServiceName(String fsServiceName) {
    	if(finalized)
    		return;
		this.fsServiceName = fsServiceName;
	}

	public void setFsClientServiceName(String fsClientServiceName) {
    	if(finalized)
    		return;
		this.fsClientServiceName = fsClientServiceName;
	}

	public void setFsServicePort(int fsServicePort) {
    	if(finalized)
    		return;
		this.fsServicePort = fsServicePort;
	}

	public void setFsClientPort(int fsClientPort) {
    	if(finalized)
    		return;
		this.fsClientPort = fsClientPort;
	}

	public void setFS_enableCaching(boolean fSEnableCaching) {
    	if(finalized)
    		return;
		FS_enableCaching = fSEnableCaching;
	}

	public void setEnableFSLogging(boolean enableFSLogging) {
    	if(finalized)
    		return;
		this.enableFSLogging = enableFSLogging;
	}

	public void setFS_enableLogShipment(boolean fSEnableLogShipment) {
    	if(finalized)
    		return;
		FS_enableLogShipment = fSEnableLogShipment;
	}

	public void setMaxWorkerThreads(int maxWorkerThreads) {
    	if(finalized)
    		return;
		this.maxWorkerThreads = maxWorkerThreads;
	}

	public void setAppDomainPoolSize(int appDomainPoolSize) {
    	if(finalized)
    		return;
		this.appDomainPoolSize = appDomainPoolSize;
	}

	public int getResponseTimeout() {
    	if(!finalized)
    		throw new RuntimeException("RuntimeConfig must be finalized!");
		return 5000;
	}

	public boolean isUseDummyAS() {
		return useDummyAS;
	}

	public boolean isEnableLogPreProcessing() {
		return isEnableLogPreProcessing;
	}

	public boolean isAnalyzerFastDispatching() {
		return isAnalyzerFastDispatching;
	}

	public boolean isMonitoringProcessorThroughput(){
		return isMonitoringProcessorThroughput;
	}
	public boolean isMonitoringDispatcherThroughput(){
		return isMonitoringDispatcherThroughput;
	}
	public boolean isMeasuringLatency(){
		return isMeasuringLatency;
	}
	public boolean indexEventTable(){
		return indexEventTable;
	}
	public boolean isPuttingByLabel(){
		return putPartitionEvents;
	}
	public boolean isTimingEventTable(){
		return timeEventTable;
	}
	public boolean quitWhenEventTableDone(){
		return quitWhenEventsDone;
	}

	public boolean isLogProcessorActive() {
		return logProcessorActive;
	}
}
