package edu.mit.csail.aeolus.runtime;

import java.security.AccessController;
import java.security.PrivilegedExceptionAction;
import java.util.LinkedList;

import edu.mit.csail.aeolus.exceptions.AeolusException;
import edu.mit.csail.aeolus.security.AeolusClassLoader;
import edu.mit.csail.aeolus.security.AeolusSecurityManager;

/**
 * Starts up an Aeolus system node
 * 
 * @author viq
 * 
 */

public class RuntimeLoader {

	public static void main(String[] args) throws AeolusException {
		boolean enableSecurityManager = false;
		args = RuntimeConfig.loadFromCL(args);
		
		LinkedList<String> argsRest = new LinkedList<String>();
		for(String arg : args){
			if( arg.equals("-s") )
				enableSecurityManager = true;
			else
				argsRest.add(arg);
		}
		args = argsRest.toArray(new String[0]);
		
		if (args.length < 2) {
			System.out
					.println("Runtime Loader Arguments: [aeolus-args] jar-file app-class-name [app-args]");
			System.exit(-1);
		}
		String jarFile = args[0];
		String appName = args[1];
		String appArgs = "";
		if (args.length > 2) {
			for (int i = 2; i < args.length; i++) {
				appArgs += args[i] + " " ;
			}
		}
		// Initialize RuntimeConfig!!!
		RuntimeConfig.get().finalize();

		if(enableSecurityManager)
			System.setSecurityManager(new AeolusSecurityManager());

		AeolusClassLoader.initialize(jarFile);
		launch(appName, appArgs, RuntimeConfig.get().isUseDummyAS(), 
				RuntimeConfig.get().isEnableFS(), false);
	}

	/**
	 * Start the node and launch the start application
	 * 
	 * @param appName
	 *            - name of application to launch
	 * @param appArgs
	 *            - application arguments
	 * @throws AeolusException
	 */
	public static void launch(String appName, String appArgs)
			throws AeolusException {
		launch(appName, appArgs, true, false, false);
	}

	public static void launch(final String appName, final String appArgs,
			final boolean useDummy, final boolean isDataNode,
			final boolean isLoggingNode) throws AeolusException {
		try{
			final SystemNode node = AccessController
				.doPrivileged(new PrivilegedExceptionAction<SystemNode>() {
					public SystemNode run() throws AeolusException{
						return new SystemNode(useDummy, isDataNode,
								isLoggingNode);
					}
				});
			
			AccessController.doPrivileged(new PrivilegedExceptionAction<Void>() {
				public Void run() throws Exception {
					node.start(appName, appArgs);
					return null;
				}
			});
		}catch(Exception e){
			e.printStackTrace();
			if(e instanceof AeolusException)
				throw (AeolusException) e;
			throw new AeolusException(e);
		}
	}
}
