package edu.mit.csail.aeolus.runtime;

import java.security.AccessController;
import java.security.PrivilegedActionException;
import java.security.PrivilegedExceptionAction;
import java.util.concurrent.ThreadFactory;

/**
 * Customized thread factory used by the AppDomainPool 
 * for new thread creation
 * 
 * It also updates the map of the created AppDomain ids and their
 * corresponding ProxyObject (stored in ProxyObjectsMap)
 * 
 * @author viq
 *
 */
public class UserDomainFactory implements ThreadFactory{

	/** Local runtime reference */
	private SystemDomain sysd;
    private boolean enableLogging;
	
    /**
     * Factory constructor
     * @param sysd - LocalRuntime reference required for PO creation
     */
	public UserDomainFactory(SystemDomain sysd){
		this.sysd = sysd;
		this.enableLogging = RuntimeConfig.get().isEnableLogging();
	}
	
	/**
	 * Creates a new AppDomain thread and its ProxyObject
	 * Maps the ProxyObject to the thread Id
	 */
	public Thread newThread(Runnable r){
		final UserDomain newAppDomain = new UserDomain(r);
		final UserProxy po;
		if(enableLogging){
			po = new UserLoggingProxy(this.sysd, newAppDomain.getId());
		}else{
			po = new UserProxy(this.sysd, newAppDomain.getId());	
		}
		newAppDomain.setProxyObject(po);
		
		// any special AppDomain setup needs to happen here...

        // bind AppDomain id to the SystemObject
		try {
    		AccessController.doPrivileged(new PrivilegedExceptionAction<Void>() {
    			public Void run() throws Exception {
    		        UserProxyMap.storeProxyObject(newAppDomain.getId(), po);
    		        return null;
    			}});
    	}catch(PrivilegedActionException e){
    		throw new RuntimeException("Failed to create new thread.");
    	}
        return newAppDomain;
	}
}
