package edu.mit.csail.aeolus.runtime;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * ThreadPool of AppDomains - manages AppDomain creation and use.
 * This class extends the ThreadPoolExecutor class in order to 
 * provide control of certain pool functionality.
 * 
 * An AppDomain is just a Thread object associated with a 
 * new ProxyObject upon creation. A map of AppDomain ids to their 
 * ProxyObject is maintained in ProxyObjectsMap
 * 
 * @author viq
 *
 */

public class UserDomainPool extends ThreadPoolExecutor{
	/**
	 * Creates a new AppDomain pool
	 * 
	 * @param corePoolSize - number of threads to keep alive
	 * @param maxPoolSize - max number of threads to allow in the pool
	 * @param keepAliveTime - amount of time excess threads will wait for a task
	 * @param unit - time unit
	 * @param workQueue - will hold the tasks submitted to the pool
	 * @param factory - factory to use for thread creation
	 */
	public UserDomainPool(int corePoolSize, int maxPoolSize, long keepAliveTime, TimeUnit unit, 
			BlockingQueue<Runnable> workQueue, ThreadFactory factory)
	{
		super(corePoolSize, maxPoolSize, keepAliveTime, unit, workQueue, factory);
	}
	
	/**
	 * This method is called in the thread that is about to execute the new task
	 * Setup before the task is executed
	 */
	protected void beforeExecute(Thread t, Runnable r)
	{	
		// log debug/statistics info
		//logger.log("Begin executing new task in thread id = " + t.getId());
		
		super.beforeExecute(t, r);
	}
	
	/**
	 * This method is called in the thread that executed the task
	 */
	protected void afterExecute(Runnable r, Throwable t)
	{
		// log debug/statistics info
		//logger.log("Finished executing task in thread id = " + Thread.currentThread().getId());
		
		// flush the proxy object
		UserProxy po = (UserProxy) UserProxyMap.getMyProxyObject();
		po.flush();
		
		super.afterExecute(r, t);
	}
	
	// overwrite additional methods to control pool behavior
}
