package edu.mit.csail.aeolus.runtime;

import java.util.HashMap;

/**
 * Stores the mapping between an AppDomain ID and the corresponding ProxyObject
 * 
 * @author viq
 *
 */
public class UserProxyMap {
	
	/** Map of AppDomain Id to SystemObject */
    private static HashMap<Long, UserProxyInterface> proxyObjects = new HashMap<Long, UserProxyInterface>();;
	
    /**
     * Returns the ProxyObject corresponding to this thread's id
     * requires there exists a mapping for the current thread's id
     * @return ProxyObject of this thread's id
     */
	public static synchronized UserProxyInterface getMyProxyObject()
	{
		long appDomainId = Thread.currentThread().getId();
		return proxyObjects.get(appDomainId);
	}
	
	/**
	 * Binds the given AppDomain id to the given ProxyObject.
	 * This call requires RuntimePermission("storeProxyObject")
	 */
	public static synchronized void storeProxyObject(long appDomainId, UserProxyInterface po){
		if(System.getSecurityManager() != null)
			System.getSecurityManager().checkPermission(new RuntimePermission("storeProxyObject"));
		proxyObjects.put(appDomainId, po);
	}

}
