package edu.mit.csail.aeolus.security;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.lang.instrument.IllegalClassFormatException;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.AccessController;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.PrivilegedActionException;
import java.security.PrivilegedExceptionAction;
import java.util.LinkedList;
import java.util.List;

import org.apache.commons.codec.binary.Base64;

import edu.mit.csail.aeolus.auditing.AeolusApiConstants;
import edu.mit.csail.aeolus.auditing.EventID;
import edu.mit.csail.aeolus.auditing.EventRecord;
import edu.mit.csail.aeolus.runtime.SystemDomain;

public class AeolusClassLoader extends ClassLoader {
	// TODO: we will need to verify that this cannot be gotten around!
	private static String jarfile_base; 
	private static boolean initialized = false;
	private List<String> hashes = new LinkedList<String>();
	private final SystemDomain sysd;
	private int counter = 0;
	private EventID curPred;
	private ClassTransformer transformer = new ClassTransformer(this);
	
	public AeolusClassLoader(){
		this(null);
	}
	
	public AeolusClassLoader(SystemDomain sysd) {
		super();
		this.sysd = sysd;
		if(sysd != null)
			this.curPred = sysd.piLaunchEid;
	}

	@Override
	public URL findResource(String s){
		try {
			return new URL("jar:" + jarfile_base + s);
		} catch (MalformedURLException e) {
			return null;
		}
	}
    
	public synchronized Class<?> loadClass (final String name, final boolean resolve)
	throws ClassNotFoundException{
		try{
			return 	AccessController.doPrivileged(new PrivilegedExceptionAction<Class<?>>() {
			public Class<?> run() throws Exception{
//			    System.out.println("Loading " + name);
//			    if (!name.startsWith("edu.mit.csail.aeolus.testing.")) {
					try{
						return getParent().loadClass(name);
					}catch(ClassNotFoundException ex){
						System.out.println("AeolusClassLoader loading " + name);
						return loadClassHelper(name, resolve);
					}
//				} else {
//					return loadClassHelper(name, resolve);
//				}
			}
		});
		}catch(PrivilegedActionException ex){
			if(ex.getCause() instanceof ClassNotFoundException)
				throw (ClassNotFoundException) ex.getCause();
			else 
				throw new RuntimeException(ex);
		}
	}
		
	private Class<?> loadClassHelper(String name, boolean resolve) 
	throws ClassNotFoundException{
		System.out.println("Loading: " + name);
		final String classResource = name.replace ('.', '/') + ".class";
		final URL classURL = getResource (classResource);
		if(classURL == null){
			throw new ClassNotFoundException("AeolusClassLoader improperly initialized.");
		}
		
		InputStream in = null;
		try{
			in = classURL.openStream ();
			byte [] classBytes = readFully (in);

			// here is where we hash and log
			MessageDigest md = MessageDigest.getInstance("MD5");

			byte[] hash = md.digest(classBytes);
			String base64Hash = new String(Base64.encodeBase64(hash), "ASCII");
			hashes.add(name);
			hashes.add(base64Hash);
			
			if(sysd != null){
				EventID eid = new EventID(sysd.platformInstanceID, counter++);
				EventRecord record = new EventRecord(eid, curPred, AeolusApiConstants.LOAD_CLASS);
				Serializable[] params = {name, base64Hash};
				record.setParams(params);
				sysd.collector.submitLogRecord(0, record);
				curPred = eid;
			}

			byte[] newClassBytes = transformer.transform(this, name, null, this.getClass().getProtectionDomain() /* XXX */, classBytes);
			if (newClassBytes != null)
			    classBytes = newClassBytes;

			Class<?> c = this.defineClass(name, classBytes, 0, classBytes.length);	

			/*
			 * Check for static variables!
			 */
			Field[] fields = c.getDeclaredFields();
			for(Field f : fields){
				if(Modifier.isStatic(f.getModifiers()) && !Modifier.isFinal(f.getModifiers())){
					//				System.out.println("Static defs!");
					throw new ClassNotFoundException(name + 
					"\nStatic variables cannot be declared in Aeolus user classes.");
				}
			}
			if(resolve){
				System.out.println("resolving " + name);
				resolveClass(c);
			}

			AeolusCloner.verifier.verifyClass(c);

			return c;
		} catch (IOException ioe) {
			throw new ClassNotFoundException (name + " in " + jarfile_base, ioe);
		} catch (NoSuchAlgorithmException e) {
			// should technically never happen...
			throw new ClassNotFoundException (name);
		} catch (IllegalClassFormatException e) {
			throw new ClassNotFoundException (name + " (transformation problem): " + e);
		}
	}

	private byte[] readFully(InputStream in) throws IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream(20);
		byte[] buf = new byte[128];
		int read = 0;
		do{
			read = in.read(buf);
			if(read < 1)
				return baos.toByteArray();
			baos.write(buf, 0, read);			
		}while(read == 128);
		return baos.toByteArray();
	}

	public static void initialize(String jarFile) {
//		String jarHold = jarfile_base;
		if(initialized)
			return;
		initialized = true;
		try {
			jarfile_base = new File(jarFile).toURL().toString() + "!/";
		} catch (MalformedURLException e) {
			throw new RuntimeException(e);
		}
	}
}
