package edu.mit.csail.aeolus.security;

import java.awt.AWTPermission;
import java.io.FilePermission;
import java.lang.reflect.Member;
import java.net.NetPermission;
import java.net.SocketPermission;
import java.security.AccessControlException;
import java.security.AccessController;
import java.security.Permission;
import java.util.PropertyPermission;

import edu.mit.csail.aeolus.api.AeolusUserInternal;
import edu.mit.csail.aeolus.infoflow.InfoFlowState;

public class AeolusSecurityManager extends SecurityManager {
	@Override
	public void checkPermission(final Permission perm) {
		if(perm instanceof RuntimePermission && perm.getName().equals("getProtectionDomain")){
			return;
		}

		try{
			AccessController.checkPermission(perm);
		}catch(AccessControlException ex){
			if(perm instanceof PropertyPermission){
				if(perm.getActions().equals("write"))
					throw new SecurityException(ex);
			}else if(perm instanceof FilePermission){
				AeolusUserInternal user = new AeolusUserInternal();
				InfoFlowState iflow = user.getInfoFlowState();
				if(perm.getActions().equals("read")){
					if(! iflow.ILabel.isEmpty())
						throw new SecurityException(ex);
				}else if(perm.getActions().equals("write")){
					if(! iflow.SLabel.isEmpty())
						throw new SecurityException(ex);
				}else{
					if(! iflow.SLabel.isEmpty() || ! iflow.ILabel.isEmpty())
						throw new SecurityException(ex);
				}
			}else if(perm instanceof SocketPermission || perm instanceof AWTPermission || perm instanceof NetPermission){
				// Sockets are both In/Out ?
//				AeolusUser user = new AeolusUser();
//				try{
//					InfoFlowState iflow = user.getInfoFlowState();
//					if(! iflow.SLabel.isEmpty() || ! iflow.ILabel.isEmpty())
//						throw new SecurityException(ex);
//				}catch(NullPointerException npe){
					throw new SecurityException(ex);
//				}
			}else{
//				for(final StackTraceElement st :	ex.getStackTrace()){
//					AccessController.doPrivileged(new PrivilegedAction<Void>() {
//						public Void run() {
//							System.out.println( st.getClassName() + " -> " + st.getClass().getProtectionDomain().implies(perm));
//							return null;
//						}
//					});
//				}
				throw new SecurityException(ex);			
			}
		}
	}

	@Override
	public void checkMemberAccess(Class<?> clazz, int which) {
		if( which == Member.DECLARED ){
			if(clazz == null)
				return;
			if(clazz.getName().startsWith("edu.mit.csail.aeolus")){
				this.checkPermission(new RuntimePermission("accessDeclaredAeolusMember"));
			}
		}
	}
}
