package edu.mit.csail.aeolus.security;

import java.io.*;
import java.util.*;
import java.lang.instrument.ClassFileTransformer;
import java.lang.instrument.IllegalClassFormatException;
import java.lang.reflect.Method;
import java.security.ProtectionDomain;

import edu.mit.csail.aeolus.api.*;
import edu.mit.csail.aeolus.infoflow.InfoFlowState;

import org.objectweb.asm.*;
import org.objectweb.asm.tree.*;
import org.objectweb.asm.util.*;

/**
 * This class came from another project.  Its purpose is to provide
 * reflection on objects that aren't loaded yet, so that recursive
 * immutability analysis can actually work.  (XXX Did we actually wind
 * up using this?)
 */
class ClassInfo {

    private Type type;

    private ClassLoader loader;

    int access;

    String superClass;

    String[] interfaces;

    public ClassInfo(final String type, final ClassLoader loader) {
	this .loader = loader;
	this .type = Type.getObjectType(type);
	String s = type.replace('.', '/') + ".class";
	InputStream is = null;
	ClassReader cr;
	try {
	    is = loader.getResourceAsStream(s);
	    cr = new ClassReader(is);
	} catch (IOException e) {
	    throw new RuntimeException(e);
	} finally {
	    if (is != null) {
		try {
		    is.close();
		} catch (Exception e) {
		}
	    }
	}

	// optimized version
	int h = cr.header;
	ClassInfo.this .access = cr.readUnsignedShort(h);
	char[] buf = new char[2048];
	// String name = cr.readClass( cr.header + 2, buf);

	int v = cr.getItem(cr.readUnsignedShort(h + 4));
	ClassInfo.this.superClass = v == 0 ? null : cr.readUTF8(v, buf);
	ClassInfo.this.interfaces = new String[cr
						.readUnsignedShort(h + 6)];
	h += 8;
	for (int i = 0; i < interfaces.length; ++i) {
	    interfaces[i] = cr.readClass(h, buf);
	    h += 2;
	}
    }

    String getName() {
	return type.getInternalName();
    }

    Type getType() {
	return type;
    }

    int getModifiers() {
	return access;
    }

    ClassInfo getSuperclass() {
	if (superClass == null) {
	    return null;
	}
	return new ClassInfo(superClass, loader);
    }

    ClassInfo[] getInterfaces() {
	if (interfaces == null) {
	    return new ClassInfo[0];
	}
	ClassInfo[] result = new ClassInfo[interfaces.length];
	for (int i = 0; i < result.length; ++i) {
	    result[i] = new ClassInfo(interfaces[i], loader);
	}
	return result;
    }

    boolean isInterface() {
	return (getModifiers() & Opcodes.ACC_INTERFACE) > 0;
    }

    private boolean implementsInterface(final ClassInfo that) {
	for (ClassInfo c = this ; c != null; c = c.getSuperclass()) {
	    ClassInfo[] tis = c.getInterfaces();
	    for (int i = 0; i < tis.length; ++i) {
		ClassInfo ti = tis[i];
		if (ti.type.equals(that.type)
		    || ti.implementsInterface(that)) {
		    return true;
		}
	    }
	}
	return false;
    }

    private boolean isSubclassOf(final ClassInfo that) {
	for (ClassInfo c = this ; c != null; c = c.getSuperclass()) {
	    if (c.getSuperclass() != null
		&& c.getSuperclass().type.equals(that.type)) {
		return true;
	    }
	}
	return false;
    }

    public boolean isAssignableFrom(final ClassInfo that) {
	if (this  == that) {
	    return true;
	}

	if (that.isSubclassOf(this )) {
	    return true;
	}

	if (that.implementsInterface(this )) {
	    return true;
	}

	if (that.isInterface()
	    && getType().getDescriptor().equals(
						"Ljava/lang/Object;")) {
	    return true;
	}

	return false;
    }
}
