package edu.mit.csail.aeolus.serversinfo;

import edu.mit.csail.aeolus.api.SystemNodeID;
import edu.mit.csail.aeolus.authority.api.ClosureTableEntry;
import edu.mit.csail.aeolus.authority.api.CompoundTagTableEntry;
import edu.mit.csail.aeolus.authority.api.GrantTableEntry;
import edu.mit.csail.aeolus.authority.api.NodeTableEntry;
import edu.mit.csail.aeolus.authority.api.PrincipalTableEntry;
import edu.mit.csail.aeolus.authority.api.TaintTableEntry;
import edu.mit.csail.aeolus.stream.server.api.StreamMessageOpcode;

public class StreamMessageParser {
	final static String operandDelimiter = ";";
	final static String listDelimiter = ",";

	public static StreamMessageEntry generateCreateSnapshot(
			String snapshotDbName) {
		return new StreamMessageEntry(StreamMessageOpcode.S_SNAPSHOT_ADD,
				snapshotDbName);
	}

	public static StreamMessageEntry generateCreatePrincipal(int pidCreator,
			int pidCreated, int coreIdCreator, int coreIdCreated) {

		StreamMessageOpcode opCode = StreamMessageOpcode.S_PRINCIPAL_ADD;
		String streamMessage = "" + pidCreator + operandDelimiter + pidCreated
				+ operandDelimiter + coreIdCreator + operandDelimiter
				+ coreIdCreated;
		return new StreamMessageEntry(opCode, streamMessage);
	}

	public static StreamMessageEntry generateCreateTaint(int pidCreator,
			int tidCreated) {
		StreamMessageOpcode requestType = StreamMessageOpcode.S_TAINT_ADD;
		String streamMessage = "" + pidCreator + operandDelimiter + tidCreated;
		return new StreamMessageEntry(requestType, streamMessage);
	}

	public static StreamMessageEntry generateCreateClosure(int cid,
			int pidClosure, String key, int coreId) {
		StreamMessageOpcode requestType = StreamMessageOpcode.S_CLOSURE_ADD;
		String streamMessage = "" + cid + operandDelimiter + pidClosure
				+ operandDelimiter + key;
		return new StreamMessageEntry(requestType, streamMessage);
	}

	public static StreamMessageEntry generateGrant(int pidGrantor,
			int pidGrantee, int tid) {
		StreamMessageOpcode requestType = StreamMessageOpcode.S_GRANT_ADD;
		String streamMessage = "" + pidGrantor + operandDelimiter + pidGrantee
				+ operandDelimiter + tid;
		return new StreamMessageEntry(requestType, streamMessage);
	}

	public static StreamMessageEntry generateDelegateActsFor(int pidFrom,
			int pidTo, int coreFrom, int coreTo) {
		StreamMessageOpcode requestType = StreamMessageOpcode.S_ACTSFOR_ADD;
		String streamMessage = "" + pidFrom + operandDelimiter + pidTo
				+ operandDelimiter + coreFrom + operandDelimiter + coreTo;
		return new StreamMessageEntry(requestType, streamMessage);
	}

	public static StreamMessageEntry generateRevokeGrant(int pidGrantor,
			int pidGrantee, int tid) {
		StreamMessageOpcode requestType = StreamMessageOpcode.S_GRANT_REMOVE;
		String streamMessage = "" + pidGrantor + operandDelimiter + pidGrantee
				+ operandDelimiter + tid;
		return new StreamMessageEntry(requestType, streamMessage);
	}

	public static StreamMessageEntry generateRevokeActsFor(int pidActor,
			int pidActee) {
		StreamMessageOpcode requestType = StreamMessageOpcode.S_ACTSFOR_REMOVE;
		String streamMessage = "" + pidActor + operandDelimiter + pidActee;
		return new StreamMessageEntry(requestType, streamMessage);
	}

	public static StreamMessageEntry generateRemoveSnapshot(
			String snapshotDbName) {
		StreamMessageOpcode requestType = StreamMessageOpcode.S_SNAPSHOT_REMOVE;
		String streamMessage = snapshotDbName;
		return new StreamMessageEntry(requestType, streamMessage);
	}
	
	public static StreamMessageEntry generateRegisterNode(int systemNodeId, String hostname){
		StreamMessageOpcode requestType = StreamMessageOpcode.NODE_REGISTERED;
		String streamMessage = "" + systemNodeId + operandDelimiter + hostname;
		return new StreamMessageEntry(requestType, streamMessage);
	}

	public static StreamMessageEntry generateRemoveNode(int systemNodeId){
		StreamMessageOpcode requestType = StreamMessageOpcode.NODE_REMOVED;
		String streamMessage = "" + systemNodeId;
		return new StreamMessageEntry(requestType, streamMessage);
	}
	
	public static String unpackCreateSnapshot(String streamMessage) {
		return unpack(streamMessage)[0];
	}

	public static String unpackRemoveSnapshot(String streamMessage) {
		return unpack(streamMessage)[0];
	}

	public static PrincipalTableEntry unpackCreatePrincipal(String streamMessage) {
		String[] s = unpack(streamMessage);
		PrincipalTableEntry p = new PrincipalTableEntry(Integer.parseInt(s[1]),
				Integer.parseInt(s[0]), Integer.parseInt(s[2]), Integer
						.parseInt(s[3]));
		return p;
	}

	public static TaintTableEntry unpackCreateTaint(String streamMessage,
			int coreIdCreator) {
		String[] s = unpack(streamMessage);
		TaintTableEntry tt = new TaintTableEntry(Integer.parseInt(s[0]),
				Integer.parseInt(s[1]), coreIdCreator);
		return tt;
	}

	public static CompoundTagTableEntry unpackCreateCompoundLink(
			String streamMessage, int coreIdCreator) {
		String[] s = unpack(streamMessage);
		CompoundTagTableEntry tt = new CompoundTagTableEntry(Integer.parseInt(s[0]),
				Integer.parseInt(s[1]), coreIdCreator);
		return tt;
	}
	
	public static NodeTableEntry unpackRegisterNode(String streamMessage) {
		String[] s = unpack(streamMessage);
		NodeTableEntry tt = new NodeTableEntry(Integer.parseInt(s[0]),s[1]);
		return tt;
	}
	
	public static SystemNodeID unpackRemoveNode(String streamMessage) {
		String[] s = unpack(streamMessage);
		return new SystemNodeID(Integer.parseInt(s[0]));
	}

	public static ClosureTableEntry unpackCreateClosure(String streamMessage,
			int coreId) {
		String[] s = unpack(streamMessage);
		ClosureTableEntry c = new ClosureTableEntry(Integer.parseInt(s[0]),
				Integer.parseInt(s[1]), s[2], coreId);
		return c;
	}

	public static GrantTableEntry unpackGrant(String streamMessage, int coreId) {
		String[] s = unpack(streamMessage);
		GrantTableEntry gt = new GrantTableEntry(Integer.parseInt(s[0]),
				Integer.parseInt(s[1]), Integer.parseInt(s[2]), coreId);
		return gt;
	}

	public static DelegateActsForMsg unpackDelegateActsFor(String streamMessage) {
		String[] s = unpack(streamMessage);
		DelegateActsForMsg d = new DelegateActsForMsg(Integer.parseInt(s[0]),
				Integer.parseInt(s[1]), Integer.parseInt(s[2]), Integer
						.parseInt(s[3]));
		return d;
	}

	public static RevokeGrantMsg unpackRevokeGrant(String streamMessage) {
		String[] s = unpack(streamMessage);
		RevokeGrantMsg d = new RevokeGrantMsg(Integer.parseInt(s[0]), Integer
				.parseInt(s[1]), Integer.parseInt(s[2]));
		return d;
	}

	public static RevokeActsForMsg unpackRevokeActsFor(String streamMessage) {
		String[] s = unpack(streamMessage);
		RevokeActsForMsg d = new RevokeActsForMsg(Integer.parseInt(s[0]),
				Integer.parseInt(s[1]));
		return d;
	}

	private static String[] unpack(String streamMessage) {
		String[] tokens = streamMessage.split(operandDelimiter);
		int numTokens = tokens.length;
		if (numTokens > 4) {
			return java.util.Arrays.copyOf(tokens, 4);
		}
		return tokens;
	}

	public static StreamMessageEntry generateCreateCompoundLink(int tidChild,
			int tidParent) {
		StreamMessageOpcode requestType = StreamMessageOpcode.S_COMPOUNDLINK_ADD;
		String streamMessage = "" + tidChild + operandDelimiter + tidParent;
		return new StreamMessageEntry(requestType, streamMessage);
	}
}
