package edu.mit.csail.aeolus.stream.client;

import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.util.concurrent.ConcurrentLinkedQueue;

import edu.mit.csail.aeolus.auditing.EventID;
import edu.mit.csail.aeolus.authority.api.AuthorityConfiguration;
import edu.mit.csail.aeolus.authority.api.RedoLogEntry;
import edu.mit.csail.aeolus.authority.client.CacheObjectManager;
import edu.mit.csail.aeolus.exceptions.AeolusException;
import edu.mit.csail.aeolus.stream.client.api.IStreamClient;
import edu.mit.csail.aeolus.stream.server.api.InterfaceStreamServer;
import edu.mit.csail.aeolus.stream.server.api.RegisterReturn;
import edu.mit.csail.aeolus.stream.server.api.SnapshotNumAndName;
import edu.mit.csail.aeolus.stream.server.api.StreamMessageOpcode;
import edu.mit.csail.aeolus.utilities.RMIUtil;

public class StreamClient implements IStreamClient {

	private CacheObjectManager cacheMgr;
	private boolean isLive = false;
	private SnapshotNumAndName lastSnapshot;
	private InterfaceStreamServer ss;

	private ConcurrentLinkedQueue<RedoLogEntry> streamMessages;
	
	public StreamClient(CacheObjectManager cacheMgr){
		this.cacheMgr = cacheMgr;
		this.streamMessages = new ConcurrentLinkedQueue<RedoLogEntry>();
	}

	private class pushStreamMessage implements Runnable{
		private EventID event;
		public pushStreamMessage(EventID event){
			this.event = event;
		}
		public void run() {
			cacheMgr.handleStreamMessage(streamMessages, event);
		}
	}
	
	public void handleInvalidationPacket(InvalidationPacket ip){
		for(RedoLogEntry e : ip.invalidations){
			this.handleStreamMessage(e.seqNum, e.coreId, e.requestType, e.message, ip.lastEvent);
		}
	}
	
	public void handleInvalidationPacketBlocking(InvalidationPacket ip){
		for(RedoLogEntry e : ip.invalidations){
			this.handleStreamMessageBlocking(e.seqNum, e.coreId, e.requestType, e.message, ip.lastEvent);
		}
	}
	
	private void handleStreamMessage(int seqNum, int coreId, StreamMessageOpcode requestType,
			String streamMessage, EventID lastEvent) {
		this.streamMessages.add(new RedoLogEntry(seqNum, coreId, requestType,streamMessage));

		Thread t = new Thread(new pushStreamMessage(lastEvent));
		t.start();
//		cacheMgr.handleStreamMessage(seqNum, coreId, requestType, streamMessage);
	}
	
	private void handleStreamMessageBlocking(int seqNum, int coreId, StreamMessageOpcode requestType,
			String streamMessage, EventID lastEvent) {
		RedoLogEntry e = new RedoLogEntry(seqNum, coreId, requestType, streamMessage);
		this.streamMessages.add(e);
		(new pushStreamMessage(lastEvent)).run();
	}

	public RegisterReturn register() throws AeolusException {
		try{
			IStreamClient export = (IStreamClient)UnicastRemoteObject.exportObject(this, 0);
		
			    // Bind the remote object's stub in the registry
			Registry remote_registry = LocateRegistry.getRegistry(AuthorityConfiguration.AuthorityHostName);
			// Each stream client gets a "unique" name so that we can run multiple
			// on the same host. -- TODO: better uniqueness
			String clientName = AuthorityConfiguration.StreamClientServiceName + 
				(int)(Math.random() * 1000);

			Registry registry = RMIUtil.getOrCreate(1099);

			try{
				registry.rebind(clientName, export);
			}catch(RemoteException e){
				e.printStackTrace();
			}
				
			this.ss = (InterfaceStreamServer) 
				remote_registry.lookup(AuthorityConfiguration.StreamServiceName);
			
			RegisterReturn r = ss.register(clientName);
			lastSnapshot = r.ss;
			
			isLive = true;
			return r;
		}catch(RemoteException e){
			throw new AeolusException(e);
		} catch (NotBoundException e) {
			throw new AeolusException(e);
		}
	}
	
	public boolean isAlive() {
		return isLive;
	}

	public SnapshotNumAndName getLastSnapshot() {
		return lastSnapshot;
	}

	public void requestStream() {
		try {
			this.ss.streamNow();
		} catch (RemoteException e) {
			e.printStackTrace();
		}
	}

}
