package edu.mit.csail.aeolus.utilities;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.ObjectStreamClass;
import java.io.Serializable;
import java.util.List;
import java.util.Random;

import org.apache.commons.codec.binary.Base64;

import edu.mit.csail.aeolus.runtime.UserProxy;

/**
 * General utility functions
 * viq
 *
 */
public class HelperUtils {
	
	private static class CustomObjectInputStream extends ObjectInputStream {
	    private ClassLoader classLoader;
	    public CustomObjectInputStream(InputStream in, ClassLoader classLoader) throws IOException{
	    	super(in);
	    	this.classLoader = classLoader;
	    }
	    	    
	    protected Class<?> resolveClass(ObjectStreamClass desc) throws ClassNotFoundException {
	        return Class.forName(desc.getName(), false, classLoader);
	    }
	}

	public static String serializeToString(Serializable s) {
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		try{
			ObjectOutputStream out = new ObjectOutputStream(bos);
			out.writeObject(s);
			out.flush();
			out.close();
        
			String base64 = new String(Base64.encodeBase64(bos.toByteArray()), "ASCII");
			return base64;
		}catch(IOException ex){
			// Should never happen with a byte array stream...
			throw new RuntimeException(ex);
		}
	}
	
	public static Object deserializeFromB64String(String base64){
		try{
			byte[] objectData = Base64.decodeBase64(base64);
			
			ObjectInputStream in = new ObjectInputStream(
					new ByteArrayInputStream(objectData));
			Object copy = in.readObject();
			in.close();

			return copy;
		}catch(Exception e){
			throw new RuntimeException(e);
		}
	}
	
	/**
	 * This method deep-copies an object. It is shared-object safe, in that it uses
	 * the userd argument to set up a safe serialization context.
	 * @param userd
	 * @param objectToCopy
	 * @return
	 */
//	public static Object deepCopy(UserProxy userd, Serializable objectToCopy)
//	{
//		if(objectToCopy == null)
//			return null;
//		
//		Object copy = null;
//		try
//		{
//			// write object to a byte array stream
//			ByteArrayOutputStream bos = new ByteArrayOutputStream();
//	        ObjectOutputStream out = new ObjectOutputStream(bos);
//	        userd.setSafeSerializingContext();
//	        userd.createSerializationMap();
//	        
//	        out.writeObject(objectToCopy);
//	        out.flush();
//	        out.close();
//	        
//	        // read object from the byte array
//	        ObjectInputStream in = new CustomObjectInputStream(new ByteArrayInputStream(bos.toByteArray()), 
//	        		objectToCopy.getClass().getClassLoader());
//	        copy = in.readObject();
//	        in.close();
//	        userd.popSerializationMap();
//	        userd.unsetSafeSerializingContext();
//		}
//		catch (Exception e)
//		{	
//			e.printStackTrace();
//		}
//        return copy;
//	}
	
	/**
	 * Appends list b to a
	 * 
	 * This function just calls list.addAll() from Java Collections
	 * (function is here since it exists in Winnie's code and might be called from somewhere...) 
	 * 
	 * @param aList - list to append to
	 * @param bList - list to append
	 * @return list a concatenated with b
	 */
	public static List<Integer> appendList(List<Integer> aList, List<Integer> bList)
    {
        aList.addAll(bList);
        return aList;
    }

	/**
	 * Generates a string of length 'size' of random characters
	 * @param size - length of general string
	 * @return
	 */
    public static String randomString(int size)
    {
        StringBuilder builder = new StringBuilder();
        Random random = new Random();
        for (int i = 0; i < size; i++)
        {
        	int code = (int) Math.floor(26 * random.nextDouble() + 65);
            builder.append((char) code);
        }

        return builder.toString();
    }

    /**
     * Removes white spaces from s
     * @param s - string to remove spaces from
     * @return string with no white spaces
     */
    public static String removeSpaces(String s)
    {
        return s.replace(" ", "");
    }
    
    /**
     * Joins the string values of List input with the string value of
     * delim between each value of the input.
     * @param delim
     * @param input
     * @return
     */
    public static String joinString(Object delim, List<?> input){
        if (input == null) {
        	return "";
        }

        if (input.size() == 0) {
        	return "";
        }

        StringBuilder sb = new StringBuilder();
        
        // prepend delimiter to all but the first integer 
        boolean first = true;
        for (Object i : input){
        	if(!first)
            	sb.append(delim.toString());
            sb.append(i.toString());
            first = false;
        }
        return sb.toString();
    }

}
