package edu.mit.csail.aeolus.utilities;

/**
 * Logs Aeolus events (e.g. shared object creation in ProxyObject)  
 * @rewrite viq
 */

public class Logger {

	private static boolean enableRemoteDebug = false;
    private static boolean globalDisabled =false;
    private static boolean enableLoggingToFile = false;

    private static LoggerLevel globalCurrLevel = LoggerLevel.DEBUG;

    // Per-Instance Threshold and Prefix
    private LoggerLevel currLevel;
    private String prefixStr;

    // ---- Constructors ----
    public Logger()
    {
        //this.currLevel = LoggerLevel.VERBOSE;
        this.prefixStr = "";
    }

    /**
     * Use local threshold
     * @param currLevel
     */
    public Logger(LoggerLevel currLevel)
    {
        this.currLevel = currLevel;
        this.prefixStr = "";
    }

    /**
     * Use global threshold
     * @param prefix
     */
    public Logger(String prefix)
    {
        this.currLevel = globalCurrLevel;
        this.prefixStr = prefix;
    }

    /**
     * Use local threshold
     * @param currLevel
     * @param prefixStr
     */
    public Logger(LoggerLevel currLevel, String prefixStr)
    {
        this.currLevel = currLevel;
        this.prefixStr = prefixStr;
    }

    public void log(LoggerLevel l, String msg)
    {
        if (enableLoggingToFile)
        {
            //FileStream fstream = new FileStream(logFilename, FileMode.Append, FileAccess.Write);
            //StreamWriter wstream = new StreamWriter(fstream);
            //wstream.WriteLine(" " + prefixStr + ": " + msg);
            //wstream.Close();
            //fstream.Close();
        }

        // INFO FLOW ERRORS are always alerted
        if (l == LoggerLevel.IFCERR)
        {
            System.out.println(" " + prefixStr + ": " + msg);
        }

        if (globalDisabled) return;

        if (l.compareTo(currLevel) >= 0)
        {
        	System.out.println(" " + prefixStr + ": " + msg);
        }

        if (l.compareTo(LoggerLevel.DEBUG) >= 0)
        {
            if (enableRemoteDebug)
            {
                try
                {
                    //    RemoteDebuggerClient dclient = new RemoteDebuggerClient("");
                    //    dclient.Log(" " + prefixStr + ": " + msg);
                }
                catch (Exception e)
                {
                }
            }
        }
    }

    public void log(String msg)
    {
        if (globalDisabled) return;

        String outputmsg = " " + prefixStr + ": " + msg;

        System.out.println(outputmsg);

        if (enableLoggingToFile)
        {
            //FileStream fstream = new FileStream(logFilename, FileMode.Append, FileAccess.Write);
            //StreamWriter wstream = new StreamWriter(fstream);
            //wstream.WriteLine(outputmsg);
            //wstream.Close();
            //fstream.Close();
        }

    }

    //---------------- static version for general messages ----------------
    public static void genericLog(LoggerLevel level, String msg)
    {
    	System.out.println(msg);

        if (enableLoggingToFile)
        {
            //FileStream fstream = new FileStream(logFilename, FileMode.Append, FileAccess.Write);
            //StreamWriter wstream = new StreamWriter(fstream);
            //wstream.WriteLine(msg);
            //wstream.Close();
            //fstream.Close();
        }
    }
}
