package edu.mit.csail.aeolus.utilities;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.List;

import edu.mit.csail.aeolus.filesystem.FileAttributes;
import edu.mit.csail.aeolus.infoflow.InfoFlowState;
import edu.mit.csail.aeolus.infoflow.Label;
import edu.mit.csail.aeolus.infoflow.TagID;

/**
 * 
 * Some useful copy/conversion/toString functions
 * 
 * viq
 *
 */

public class Utils {
	
	public static String listStringDelimiter = ",";
    
	/**
	 * Makes a deep copy of the object by first serializing the object 
	 * and then reversing the process and deserializing it.
	 * 
	 * @param serializableObj - object to copy, must be Serializable 
	 * @return deep copy of serializableObj
	 */
	public static Object deepCopy(Object serializableObj)
    {	
		Object copy = null;
		
		try {
			// serialize
			ByteArrayOutputStream inMemByteArray = new ByteArrayOutputStream();
			ObjectOutputStream out = new ObjectOutputStream(inMemByteArray);
			out.writeObject(serializableObj);
			out.close();
			// deserialize
			ByteArrayInputStream memByteArray = new ByteArrayInputStream(inMemByteArray.toByteArray());
			ObjectInputStream in = new ObjectInputStream(memByteArray);
			copy = in.readObject();
			in.close();
		} 
		catch (IOException e) {
			// TODO
			Logger.genericLog(LoggerLevel.DEBUG, "Utils: deepCopy " + e.getMessage() + " \n");
		}
		catch (ClassNotFoundException e)
		{
			// TODO
			Logger.genericLog(LoggerLevel.DEBUG, "Utils: deepCopy " + e.getMessage() + " \n");
		}
	
		return copy;
    }

	/**
	 * Returns a list of objects as strings, separated by the delimiter ','
	 * 
	 * Calls the toString() method to assemble the list
	 * 
	 * @param input - list of values
	 * @return string representation of elements
	 */
    public static String listToString(List<? extends Object> input)
    {
        if (input == null) 
        {
        	return "";
        }

        if (input.size() == 0) 
        {
        	return "";
        }

        StringBuilder sb = new StringBuilder();
        
        // append delimiter to all but the first integer 
        boolean first = true;
        for (Object i : input)
        {
            if(first)
            {
            	sb.append(i.toString());
            	first = false;
            } 
            else 
            {
            	sb.append(listStringDelimiter + i.toString());
            }
        }
        return sb.toString();
    }
    
//    public static String listTagIDToString(List<TagID> input)
//    {
//        if (input == null) 
//        {
//        	return "";
//        }
//
//        if (input.size() == 0) 
//        {
//        	return "";
//        }
//
//        StringBuilder sb = new StringBuilder();
//        
//        // append delimiter to all but the first integer 
//        boolean first = true;
//        for (TagID i : input)
//        {
//            if(first)
//            {
//            	sb.append(i.getValue());
//            	first = false;
//            } 
//            else 
//            {
//            	sb.append(listStringDelimiter + i.toString());
//            }
//        }
//        return sb.toString();
//    }
    
    public static List<Integer> listTagIDToInt(List<TagID> input)
    {
    	List<Integer> output = new ArrayList<Integer>();

        if (input == null)
        {
            return output;
        }
        
        for (TagID i : input)
        {
            output.add(i.getValue());
        }
       
        return output;
    }

    /**
     * Returns a list of integers given a string representation
     * of a list of integers
     * 
     * Changed slightly from Winnie's implementation
     * 
     * @param input - string representation of a list of integers separated by ','
     * @return list of integers
     */
    public static List<Integer> stringToListInt(String input)
    {
        List<Integer> output = new ArrayList<Integer>();

        if ((input == "") || (input == null))
        {
            return output;
        }

        String[] tokens = input.split(listStringDelimiter);
        for (int i = 0; i < tokens.length; i++)
        {
        	try
        	{
        		output.add(new Integer(tokens[i]));
        	}
        	catch (NumberFormatException e)
        	{
        		// this should not happen...
        		// throw exception?
        		Logger.genericLog(LoggerLevel.DEBUG, "Utils: stringToListInt conversion problem. \n");
        	}
        }
        return output;
    }

    /**
     * Returns a string representation of the given state
     * @param state
     * @return string state representation
     */
    public static String formatInfoFlowState(InfoFlowState state)
    {
    	String newline = "\n";
        StringBuilder sb = new StringBuilder();
        sb.append("Basis: " + listToString(state.basis) + newline);
        sb.append("P: " + state.pid.getValue() + newline);
        sb.append("S: " + state.SLabel.toString() + newline);
        sb.append("I: " + state.ILabel.toString());
        return sb.toString();
    }
    
	public static FileAttributes parseAttributes(String metadataContent, String path)
	{
		String[] attrs = metadataContent.split("\\|");
		String sLabelStr = attrs[attrs.length-2].replace("[", "");
		sLabelStr = sLabelStr.replace("]", "");
		String iLabelStr = attrs[attrs.length-1].replace("[", "");
		iLabelStr = iLabelStr.replace("]", "");
		Label sLabel = new Label();
		Label iLabel = new Label();
		for (String s : sLabelStr.split(","))
			if (s != null && !s.trim().equals(""))
				sLabel.addTag(new TagID(Integer.parseInt(s.trim())));
		for (String s : iLabelStr.split(","))
			if (s != null && !s.trim().equals(""))
				iLabel.addTag(new TagID(Integer.parseInt(s.trim())));

		FileAttributes fileAttr = new FileAttributes(sLabel, iLabel,
				path + attrs[0], path + attrs[1]);	
		
		return fileAttr;
	}
	
}
